@@ -1184,6 +1184,60 @@
 
 ;;; Mathematical Alphanumeric Symbols		1D400--1D7FF
 
+;; Bold upright symbols
+("<b-up-A>"	"#1D400")
+("<b-up-B>"	"#1D401")
+("<b-up-C>"	"#1D402")
+("<b-up-D>"	"#1D403")
+("<b-up-E>"	"#1D404")
+("<b-up-F>"	"#1D405")
+("<b-up-G>"	"#1D406")
+("<b-up-H>"	"#1D407")
+("<b-up-I>"	"#1D408")
+("<b-up-J>"	"#1D409")
+("<b-up-K>"	"#1D40A")
+("<b-up-L>"	"#1D40B")
+("<b-up-M>"	"#1D40C")
+("<b-up-N>"	"#1D40D")
+("<b-up-O>"	"#1D40E")
+("<b-up-P>"	"#1D40F")
+("<b-up-Q>"	"#1D410")
+("<b-up-R>"	"#1D411")
+("<b-up-S>"	"#1D412")
+("<b-up-T>"	"#1D413")
+("<b-up-U>"	"#1D414")
+("<b-up-V>"	"#1D415")
+("<b-up-W>"	"#1D416")
+("<b-up-X>"	"#1D417")
+("<b-up-Y>"	"#1D418")
+("<b-up-Z>"	"#1D419")
+("<b-up-a>"	"#1D41A")
+("<b-up-b>"	"#1D41B")
+("<b-up-c>"	"#1D41C")
+("<b-up-d>"	"#1D41D")
+("<b-up-e>"	"#1D41E")
+("<b-up-f>"	"#1D41F")
+("<b-up-g>"	"#1D420")
+("<b-up-h>"	"#1D421")
+("<b-up-i>"	"#1D422")
+("<b-up-j>"	"#1D423")
+("<b-up-k>"	"#1D424")
+("<b-up-l>"	"#1D425")
+("<b-up-m>"	"#1D426")
+("<b-up-n>"	"#1D427")
+("<b-up-o>"	"#1D428")
+("<b-up-p>"	"#1D429")
+("<b-up-q>"	"#1D42A")
+("<b-up-r>"	"#1D42B")
+("<b-up-s>"	"#1D42C")
+("<b-up-t>"	"#1D42D")
+("<b-up-u>"	"#1D42E")
+("<b-up-v>"	"#1D42F")
+("<b-up-w>"	"#1D430")
+("<b-up-x>"	"#1D431")
+("<b-up-y>"	"#1D432")
+("<b-up-z>"	"#1D433")
+
 ;; Bold symbols
 ("<b-A>"	"#1D468")
 ("<b-B>"	"#1D469")
