@@ -586,7 +586,7 @@
 ("keyboard" "nppimist")
 ("keyboard equivalent" "vastine nppimistll")
 ("keyboard modifiers" "nppinmuuntimet")
-("keyboard shorthand" "nppimistlyhenne")
+("keyboard shortcut" "nppimistlyhenne")
 ("keymaps" "nppinkartat")
 ("keyword" "avainsana")
 ("keywords" "avainsanat")
