@@ -7,10 +7,9 @@
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -25,6 +24,8 @@
 (define latex-cyrillic-catcode? #f)
 (define latex-style "generic")
 (define latex-style-hyp 'generic-style%)
+(define latex-packages '())
+(define latex-amsthm-hyp 'no-amsthm-package%)
 
 (define latex-uses-table (make-ahash-table))
 (define latex-catcode-table (make-ahash-table))
@@ -45,6 +46,11 @@
   (set! latex-style sty)
   (set! latex-style-hyp (string->symbol (string-append sty "-style%"))))
 
+(tm-define (latex-set-packages ps)
+  (set! latex-packages ps)
+  (when (in? "amsthm" ps)
+    (set! latex-amsthm-hyp 'amsthm-package%)))
+
 (tm-define (latex-book-style?)
   (in? latex-style '("book")))
 
@@ -54,8 +60,8 @@
 
 (define (latex-replace-catcode s)
   (or (if latex-cyrillic-catcode?
-	  (drd-ref cyrillic-catcodes% s)
-	  (drd-ref iso-latin-catcodes% s))
+	  (logic-ref cyrillic-catcodes% s)
+	  (logic-ref corkT1-to-latex-catcodes% s))
       s))
 
 (tm-define (latex-expand-catcodes t)
@@ -104,7 +110,7 @@
 	((func? t '!recurse 1)
 	 (latex-expand-macros (latex-substitute (cadr t) args)))
 	((func? t '!translate 1)
-	 (translate (cadr t) "english" latex-language))
+	 (translate-from-to (cadr t) "english" latex-language))
 	((list? t) (map (cut latex-substitute <> args) t))
 	(else t)))
 
@@ -113,12 +119,16 @@
   (if (npair? t) t
       (let* ((head  (car t))
 	     (tail  (map latex-expand-macros (cdr t)))
-	     (body  (drd-ref latex-texmacs-macro% head latex-style-hyp))
-	     (arity (drd-ref latex-texmacs-arity% head latex-style-hyp))
+	     (body  (logic-ref latex-texmacs-macro% head
+			     latex-style-hyp latex-amsthm-hyp))
+	     (arity (logic-ref latex-texmacs-arity% head
+			     latex-style-hyp latex-amsthm-hyp))
 	     (env   (and (func? head '!begin)
-			 (drd-ref latex-texmacs-environment% (cadr head))))
+			 (logic-ref latex-texmacs-environment% (cadr head)
+				  latex-style-hyp latex-amsthm-hyp)))
 	     (envar (and (func? head '!begin)
-			 (drd-ref latex-texmacs-env-arity% (cadr head)))))
+			 (logic-ref latex-texmacs-env-arity% (cadr head)
+				  latex-style-hyp latex-amsthm-hyp))))
 	(cond ((and body (== (length tail) arity))
 	       (latex-substitute body t))
 	      ((and env (== (length tail) 1) (== (length (cddr head)) envar))
@@ -133,30 +143,35 @@
   (cond ((== t '---) "#-#-#")
 	((number? t) (string-append "#" (number->string t)))
 	((func? t '!recurse 1) (latex-expand-def (cadr t)))
-	((func? t '!translate 1) (translate (cadr t) "english" latex-language))
+	((func? t '!translate 1)
+	 (translate-from-to (cadr t) "english" latex-language))
 	((list? t) (map latex-expand-def t))
 	(else t)))
 
 (define (latex-macro-defs-sub t)
   (when (pair? t)
     (for-each latex-macro-defs-sub (cdr t))
-    (let* ((body  (drd-ref latex-texmacs-macro% (car t) latex-style-hyp))
-	   (arity (drd-ref latex-texmacs-arity% (car t) latex-style-hyp)))
+    (let* ((body  (logic-ref latex-texmacs-macro% (car t)
+			   latex-style-hyp latex-amsthm-hyp))
+	   (arity (logic-ref latex-texmacs-arity% (car t)
+			   latex-style-hyp latex-amsthm-hyp)))
       (when (and body (== (length t) (+ arity 1)))
 	(ahash-set! latex-macro-table (car t)
 		    (list arity (latex-expand-def body)))
 	(latex-macro-defs-sub body)))
     (let* ((body  (and (func? (car t) '!begin)
-		       (drd-ref latex-texmacs-environment% (cadar t))))
+		       (logic-ref latex-texmacs-environment% (cadar t))))
 	   (arity (and (func? (car t) '!begin)
-		       (drd-ref latex-texmacs-env-arity% (cadar t)))))
+		       (logic-ref latex-texmacs-env-arity% (cadar t)))))
       (when (and body (== (length (cddar t)) arity))
 	(ahash-set! latex-env-table (cadar t)
 		    (list arity (latex-expand-def body)))
 	(latex-macro-defs-sub body)))
-    (with body (or (drd-ref latex-texmacs-preamble% (car t))
+    (with body (or (logic-ref latex-texmacs-preamble% (car t)
+			    latex-style-hyp latex-amsthm-hyp)
 		   (and (func? (car t) '!begin)
-			(drd-ref latex-texmacs-env-preamble% (cadar t))))
+			(logic-ref latex-texmacs-env-preamble% (cadar t)
+				 latex-style-hyp latex-amsthm-hyp)))
       (when body
 	(ahash-set! latex-preamble-table (car t) body)))))
 
@@ -226,7 +241,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (latex-command-uses s)
-  (with packlist (drd-ref-list latex-needs% s)
+  (with packlist (logic-ref-list latex-needs% s)
     (when packlist
       (for-each (cut ahash-set! latex-uses-table <> #t) packlist))))
 
@@ -241,35 +256,41 @@
       (for-each latex-use-which-package (cdr l)))))
 
 (define (latex-use-package-compare l r)
-  (let* ((tl (drd-ref latex-package-priority% l))
-	 (tr (drd-ref latex-package-priority% r))
+  (let* ((tl (logic-ref latex-package-priority% l))
+	 (tr (logic-ref latex-package-priority% r))
 	 (vl (if tl tl 999999))
 	 (vr (if tr tr 999999)))
     (< vl vr)))
 
+(define (latex-as-use-package l1)
+  (let* ((l2 (sort l1 latex-use-package-compare))
+	 (l3 (map force-string l2))
+	 (l4 (list-intersperse l3 ","))
+	 (s  (apply string-append l4)))
+    (if (== s "") "" (string-append "\\usepackage{" s "}\n"))))
+
 (tm-define (latex-use-package-command doc)
   (:synopsis "Return the usepackage command for @doc")
   (set! latex-uses-table (make-ahash-table))
   (latex-use-which-package doc)
-  (let* ((l1 (map car (ahash-table->list latex-uses-table)))
-	 (l2 (sort l1 latex-use-package-compare))
-	 (l3 (map force-string l2))
-	 (l4 (list-intersperse l3 ","))
-	 (s  (apply string-append l4)))
-    (if (== s "") "" (string-append "\\usepackage{" s "}\n"))))
+  (let* ((l1 latex-packages)
+	 (s1 (latex-as-use-package (list-difference l1 '("amsthm"))))
+	 (l2 (map car (ahash-table->list latex-uses-table)))
+	 (s2 (latex-as-use-package (list-difference l2 l1))))
+    (string-append s1 s2)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Page size settings
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (latex-preamble-language lan)
-  (if (drd-ref latex-preamble-language-def% lan)
-      (string-append (drd-ref latex-preamble-language-def% lan) "\n")
+  (if (logic-ref latex-preamble-language-def% lan)
+      (string-append (logic-ref latex-preamble-language-def% lan) "\n")
       ""))
 
 (define (latex-preamble-page-type init)
   (let* ((page-type (ahash-ref init "page-type"))
-	 (page-size (drd-ref latex-paper-type% page-type)))
+	 (page-size (logic-ref latex-paper-type% page-type)))
     (if page-size `(!append (geometry ,page-size) "\n") "")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -288,6 +309,7 @@
 	 (pre-catcode  (latex-catcode-defs Text))
 	 (pre-uses     (latex-use-package-command Text)))
     (values
+      (if (in? "amsthm" latex-packages) "[amsthm]" "")
       (string-append pre-uses)
       (string-append pre-page)
       (string-append pre-language pre-catcode pre-macro))))
