@@ -42,6 +42,7 @@
 (define tmtex-japanese? #f)
 (define tmtex-korean? #f)
 (define tmtex-taiwanese? #f)
+(define tmtex-cyrillic? #f)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Style
@@ -65,7 +66,8 @@
   (set! tmtex-korean? (== lan "korean"))
   (set! tmtex-taiwanese? (== lan "taiwanese"))
   (set! tmtex-oriental? (or tmtex-chinese? tmtex-japanese?
-			    tmtex-korean? tmtex-taiwanese?)))
+			    tmtex-korean? tmtex-taiwanese?))
+  (set! tmtex-cyrillic? (in? lan '("bulgarian" "russian" "ukrainian"))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Initialization from options
@@ -98,16 +100,17 @@
 ;; Data
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-table tmtex-table-props%
+(logic-table tmtex-table-props%
   (block ("" "l" "" #t))
   (block* ("" "c" "" #t))
   (tabular ("" "l" "" #f))
   (tabular* ("" "c" "" #f))
   (matrix ((,(string->symbol "left(")) "c" (,(string->symbol "right)")) #f))
   (det ((left|) "c" (right|) #f))
+  (stack ("" "c" "" #f))
   (choice ((left\{) "l" (right.) #f)))
 
-(drd-table tex-with-cmd%
+(logic-table tex-with-cmd%
   (("font-family" "rm") tmtextrm)
   (("font-family" "ss") tmtextsf)
   (("font-family" "tt") tmtexttt)
@@ -142,7 +145,7 @@
   (("par-mode" "left") (!begin "flushleft"))
   (("par-mode" "right") (!begin "flushright")))
 
-(drd-table tex-assign-cmd%
+(logic-table tex-assign-cmd%
   (("font-family" "rm") rmfamily)
   (("font-family" "ss") ssfamily)
   (("font-family" "tt") ttfamily)
@@ -226,7 +229,7 @@
 
 (define tex-apply
   (lambda l
-    (if (or (tmtex-math-mode?) (drd-in? (car l) tmpre-sectional%)) l
+    (if (or (tmtex-math-mode?) (logic-in? (car l) tmpre-sectional%)) l
 	(list '!group l))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -253,6 +256,7 @@
 	((string-starts? s "bbb-") (tmtex-modified-token 'mathbbm s 4))
 	((string-starts? s "b-cal-")
 	 (tex-apply 'tmmathbf (tmtex-modified-token 'mathcal s 6)))
+	((string-starts? s "b-up-") (tmtex-modified-token 'mathbf s 5))
 	((string-starts? s "b-") (tmtex-modified-token 'tmmathbf s 2))
 	((and (string-starts? s "#") tmtex-oriental?)
 	 (cond (tmtex-japanese?
@@ -271,8 +275,12 @@
 		       ;;(cv (string-convert qs "Cork" "cp950")) ; Taiwanese ?
 		       (cv (string-convert qs "Cork" "UTF-8")))
 		  (list '!widechar (string->symbol cv))))))
+	((and (string-starts? s "#") tmtex-cyrillic?)
+	 	(let* ((qs (string-append "<" s ">"))
+		       (cv (string-convert qs "Cork" "UTF-8")))
+		   (list '!widechar (string->symbol cv))))	 	
 	(else (let ((ss (list (string->symbol s))))
-		(cond ((not (drd-in? (car ss) latex-symbol%))
+		(cond ((not (logic-in? (car ss) latex-symbol%))
 		       (display* "TeXmacs] non converted symbol: " s "\n")
 		       "")
 		      (group? (list '!group ss))
@@ -321,7 +329,7 @@
   (receive (p q) (list-break l (lambda (c) (not (char-alphabetic? c))))
     (let* ((op (list->string p))
 	   (tail (tmtex-math-list q)))
-      (if (drd-in? (string->symbol op) latex-operator%)
+      (if (logic-in? (string->symbol op) latex-operator%)
 	  (cons (list '!symbol (tex-apply (string->symbol op))) tail)
 	  (cons (tex-apply 'tmop op) tail)))))
 
@@ -464,6 +472,9 @@
 (define (tmtex-document l)
   (cons '!document (tmtex-list l)))
 
+(define (tmtex-date l)
+  (tmtex-default "today" l))
+
 (define (tmtex-para l)
   (cons '!paragraph (tmtex-list l)))
 
@@ -647,6 +658,22 @@
 (define (tmtex-big l)
   (list (string->symbol (tmtex-big-decode (car l)))))
 
+(define (tmtex-decode-long-arrow s)
+  (cond ((nstring? s) 'xrightarrow)
+        ((and (string-starts? s "<rubber-") (string-ends? s ">"))
+         (tmtex-decode-long-arrow (substring s 8 (- (string-length s) 1))))
+        ((in? s '("minus" "leftarrow" "rightarrow" "leftrightarrow"
+                  "equal" "Leftarrow" "Rightarrow" "Leftrightarrow"
+                  "mapsto" "mapsfrom"))
+         (string->symbol (string-append "x" s)))
+        (else 'xrightarrow)))
+
+(define (tmtex-long-arrow l)
+  (with cmd (tmtex-decode-long-arrow (car l))
+    (if (== (length l) 2)
+        (list cmd (tmtex (cadr l)))
+        (list cmd (list '!option (tmtex (caddr l))) (tmtex (cadr l))))))
+
 (define (tmtex-prime-list l)
   (if (null? l) l
       (cond ((== (car l) #\<)
@@ -827,7 +854,7 @@
 	   (tmtex-table-args-assemble lb (cons (car lb) rb) l))))
 
 (define (tmtex-table-apply key x)
-  (let* ((props (drd-ref tmtex-table-props% key)))
+  (let* ((props (logic-ref tmtex-table-props% key)))
     (if props
 	(let* ((env (if (tmtex-math-mode?) 'array 'tabular))
 	       (before (car props))
@@ -852,7 +879,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (tmtex-get-with-cmd var val)
-  (drd-ref tex-with-cmd% (list var val)))
+  (logic-ref tex-with-cmd% (list var val)))
 
 (define (tmtex-get-assign-cmd var val)
   (if (== var "font-size")
@@ -869,7 +896,7 @@
 	      ((< x 22.5) 'huge)
 	      ((< x 50) 'Huge)
 	      (else #f)))
-      (drd-ref tex-assign-cmd% (list var val))))
+      (logic-ref tex-assign-cmd% (list var val))))
 
 (define (tmlength->texlength len)
   ;; TODO: rewrite (quote x) -> x and (tmlen ...) -> ...pt
@@ -948,7 +975,7 @@
 
 (define (tmtex-var-name var)
   (cond ((nstring? var) "")
-	((drd-in? (string->symbol var) tmtex-protected%)
+	((logic-in? (string->symbol var) tmtex-protected%)
 	 (string-append "tm" var))
 	((<= (string-length var) 1) var)
 	(else (list->string (tmtex-var-name-sub (string->list var))))))
@@ -1485,10 +1512,10 @@
 
 (define (tmtex-apply key args)
   (let ((n (length args))
-	(r (drd-ref tmtex-methods% key)))
+	(r (logic-ref tmtex-methods% key)))
     (if (in? key '(quote quasiquote unquote)) (set! r tmtex-noop))
     (if r (r args)
-	(let ((p (drd-ref tmtex-tmstyle% key)))
+	(let ((p (logic-ref tmtex-tmstyle% key)))
 	  (if (and p (or (= (cadr p) -1) (= (cadr p) n)))
 	      ((car p) (symbol->string key) args)
 	      (if (and (= n 1)
@@ -1522,7 +1549,7 @@
 ;; Dispatching
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-dispatcher tmtex-methods%
+(logic-dispatcher tmtex-methods%
   ((:or unknown uninit error raw-data) tmtex-noop)
   (document tmtex-document)
   (para tmtex-para)
@@ -1566,6 +1593,7 @@
   (mid tmtex-mid)
   (right tmtex-right)
   (big tmtex-big)
+  (long-arrow tmtex-long-arrow)
   (lprime tmtex-lprime)
   (rprime tmtex-rprime)
   (below tmtex-below)
@@ -1605,10 +1633,12 @@
   (syntax tmtex-syntax)
 
   ((:or or xor and not plus minus times over div mod
-	merge length range number date translate change-case find-file
+	merge length range number translate change-case find-file
 	is-tuple look-up
 	equal unequal less lesseq greater greatereq) tmtex-noop)
 
+  (date tmtex-date)
+
   ((:or cm-length mm-length in-length pt-length
 	bp-length dd-length pc-length cc-length
 	fs-length fbs-length em-length
@@ -1649,7 +1679,7 @@
   (!file tmtex-file)
   (!arg tmtex-tex-arg))
 
-(drd-table tmtex-tmstyle%
+(logic-table tmtex-tmstyle%
   ((:or hide-preamble show-preamble) (,tmtex-default -1))
   (hide-part (,tmtex-hide-part -1))
   (show-part (,tmtex-show-part -1))
@@ -1662,7 +1692,8 @@
   (appendix (,tmtex-appendix 1))
   ((:or theorem proposition lemma corollary proof axiom definition
 	notation conjecture remark note example exercise problem warning
-	convention quote-env quotation verse)
+	convention quote-env quotation verse solution question answer
+	acknowledgments)
    (,tmtex-std-env 1))
   ((:or verbatim code) (,tmtex-verbatim 1))
   (center (,tmtex-std-env 1))
@@ -1760,7 +1791,7 @@
   ((:or cite-author* cite-author*-link) (,tmtex-cite-author* 1))
   ((:or cite-year cite-year-link) (,tmtex-cite-year 1)))
 
-(drd-group tmtex-protected%
+(logic-group tmtex-protected%
   a b c d i j k l o r t u v H L O P S
   aa ae bf cr dh dj dp em fi ge gg ht if in it le lg ll lu lq mp mu
   ne ng ni nu oe or pi pm rm rq sb sc sf sl sp ss th to tt wd wp wr xi
@@ -1795,7 +1826,7 @@
 	(substring s 6 (string-length s))
 	s)))
 
-(define (drd-first-list name)
+(define (logic-first-list name)
   (let* ((l1 (query (cons name '('first 'second))))
 	 (l2 (map (cut assoc-ref <> 'first) l1)))
     (map as-string l2)))
@@ -1804,10 +1835,10 @@
   `(associate ,name (xmacro "x" (eval-args "x"))))
 
 (tm-define (tmtex-env-patch t)
-  (let* ((l1 (drd-first-list 'tmtex-methods%))
-	 (l2 (drd-first-list 'tmtex-tmstyle%))
-	 (l3 (map as-string (drd-apply-list '(latex-tag%))))
-	 (l4 (map as-string (drd-apply-list '(latex-symbol%))))
+  (let* ((l1 (logic-first-list 'tmtex-methods%))
+	 (l2 (logic-first-list 'tmtex-tmstyle%))
+	 (l3 (map as-string (logic-apply-list '(latex-tag%))))
+	 (l4 (map as-string (logic-apply-list '(latex-symbol%))))
 	 (l5 (list-difference l3 l4))
 	 (l6 (map as-string (collect-user-defs (tree->stree t))))
 	 (l7 (list-difference (list-union l2 (list-union l5 l6)) l1)))
