@@ -198,6 +198,9 @@
 (define (alternative-context? t)
   (alternative-tag? (tree-label t)))
 
+(tm-define (switch-select t i)
+  (texmacs-error "switch-select" "invalid context"))
+
 (tm-define (switch-select t i)
   (:require (alternative-context? t))
   (switch-set-range t 0 :last #f)
@@ -434,10 +437,11 @@
 		"page-odd" "page-even" "page-right"
 		"par-width" "page-odd-shift" "page-even-shift"
 		"page-top" "page-bot" "page-height-margin")
-  (with t (buffer-tree)
-    (when (and (tm-func? t 'document 1) (switch-context? (tree-ref t 0)))
-      (tree-assign-node (tree-ref t 0) 'document)
-      (tree-set! t (tree-ref t 0))
+  (let* ((t (buffer-tree))
+         (c (tree-children t)))
+    (when (and (tm-func? t 'document) (switch-context? (cAr c)))
+      (tree-assign-node (cAr c) 'document)
+      (tree-set! t `(document ,@(cDr c) ,@(tree-children (cAr c))))
       (for-each dynamic-make-slide (tree-children t)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -537,3 +541,25 @@
 (tm-define (screens-switch-to which)
   (and-with t (tree-innermost 'screens)
     (switch-to t which :start)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Balloons
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (show-texmacs-balloon body balloon)
+  (with w (widget-texmacs-output balloon)
+    (with (x1 y1 x2 y2) (tree-bounding-rectangle body)
+      (show-balloon w x1 (- y1 (* 5 256))))))
+
+(tm-define (show-mouse-over-balloon)
+  (:secure #t)
+  (with-action body
+    (and-with balloon (tree-ref body :next)
+      (show-texmacs-balloon body balloon))))
+
+(tm-define (show-help-balloon)
+  (:secure #t)
+  (with-action body
+    (and-with balloon (tree-ref body :next)
+      (show-texmacs-balloon
+       body `(colored-frame "pastel yellow" ,balloon)))))
