@@ -16,6 +16,14 @@
 	(dynamic fold-edit)))
 
 (kbd-map
+  ("ir-up" (screens-switch-to :previous))
+  ("ir-down" (screens-switch-to :next))
+  ("ir-left" (dynamic-traverse-buffer :previous))
+  ("ir-right" (dynamic-traverse-buffer :next))
+  ("ir-center" (dynamic-traverse-buffer :next))
+  ("ir-play" (dynamic-traverse-buffer :next))
+  ("ir-pause" (dynamic-traverse-buffer :next))
+  ("ir-menu" (toggle-full-screen-mode))
   ("F9" (dynamic-operate-on-buffer :first))
   ("F10" (dynamic-traverse-buffer :previous))
   ("F11" (dynamic-traverse-buffer :next))
@@ -23,4 +31,10 @@
   ("S-F9" (dynamic-first))
   ("S-F10" (dynamic-previous))
   ("S-F11" (dynamic-next))
-  ("S-F12" (dynamic-last)))
+  ("S-F12" (dynamic-last))
+  ("C-F9" (toggle-full-screen-mode)))
+
+(kbd-map
+  (:mode in-screens?)
+  ("pageup" (screens-switch-to :previous))
+  ("pagedown" (screens-switch-to :next)))
