@@ -12,7 +12,8 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (texmacs-module (dynamic fold-menu)
-  (:use (dynamic fold-edit)))
+  (:use (dynamic fold-edit)
+        (generic generic-menu)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Menus for direct folding and switching
@@ -266,16 +267,16 @@
 (tm-menu (standard-focus-icons t)
   (:require (tree-is? t 'screens))
   (dynamic (focus-style-icons t))  
-  (glue #f #f 5 0)
+  //
   (minibar (dynamic (focus-insert-icons t)))
-  (glue #f #f 5 0)
+  //
   (minibar (dynamic (focus-tag-icons t)))
-  (glue #f #f 5 0)
+  //
   (with i (tree-index (tree-down t))
     (mini #t
       (=> (eval (get-slide-name (tree-ref t i) i))
           (dynamic (focus-slides-menu t)))))
   (assuming (slide-propose-title? t)
-    (glue #f #f 5 0)
+    //
     (minibar
      ((balloon "Title" "Insert title") (slide-insert-title t)))))
