@@ -12,15 +12,35 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (texmacs-module (generic document-menu)
-  (:use (generic document-edit)))
+  (:use (generic document-edit)
+        (generic generic-menu)
+        (texmacs menus file-menu)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Project menu
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (include-list base t)
+  (cond ((tree-is? t 'document)
+         (apply append (map (cut include-list base <>) (tree-children t))))
+        ((and (tree-is? t 'include) (tree-atomic? (tree-ref t 0)))
+         (list (url-relative base (tree->string (tree-ref t 0)))))
+        (else (list))))
+
+(tm-define (project-file-list)
+  (if (project-attached?)
+      (let* ((prj (project-get))
+             (t (buffer->tree prj)))
+        (include-list prj t))
+      (list)))
+
+(tm-define (project-menu)
+  (buffer-list-menu (project-file-list)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Submenus for the Document menu and the iconbars
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(menu-bind project-menu
-  (link project-buffer-menu))
-
 (menu-bind document-style-menu
   ("No style" (init-style "none"))
   ---
@@ -625,7 +645,10 @@
        (=> (eval pack)
 	   ("Remove package" (init-remove-package pack)))))
    (=> (balloon (icon "tm_add.xpm") "Add style package")
-       (link add-package-menu))))
+       (link add-package-menu))
+   (assuming (tree-is-buffer? t)
+     ((balloon (icon "tm_focus_help.xpm") "Describe tag")
+      (focus-help)))))
 
 (tm-menu (focus-document-icons t)
   (minibar
@@ -639,11 +662,7 @@
 (tm-menu (standard-focus-icons t)
   (:require (tree-is-buffer? t))
   (dynamic (focus-style-icons t))  
-  (glue #f #f 5 0)
+  //
   (dynamic (focus-document-icons t))  
-  (glue #f #f 5 0)
-  (minibar
-   ((balloon (icon "tm_focus_help.xpm") "Describe tag")
-    (focus-help)))
-  (glue #f #f 5 0)   
+  //   
   (dynamic (focus-document-extra-icons t)))
