@@ -12,7 +12,9 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (texmacs-module (graphics graphics-menu)
-  (:use (graphics graphics-edit)))
+  (:use (graphics graphics-env)
+	(graphics graphics-main)
+        (graphics graphics-edit)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Submenus
@@ -20,31 +22,96 @@
 
 ;; FIXME: provide automatic checkmarks for these actions
 
-(menu-bind graphics-geometry-menu
-  (-> "Extents"
-      ("Default" (graphics-remove-property "gr-geometry"))
-      ---
-      ("Width" (interactive graphics-set-width))
-      ("Height" (interactive graphics-set-height))
-      (-> "Alignment"
-	  ("Top" (graphics-set-geo-valign "top"))
-	  ("Center" (graphics-set-geo-valign "center"))
-	  ("Bottom" (graphics-set-geo-valign "bottom"))))
-  (-> "Frame"
-      ("Default" (graphics-remove-property "gr-frame"))
-      ---
-      (group "Cartesian")
-      (-> "Unit"
-	  ("1 cm" (graphics-set-unit "1cm"))
-	  ("1 inch" (graphics-set-unit "1in"))
-	  ---
-	  ("Other" (interactive graphics-set-unit)))
-      (-> "Origin"
-	  ("Centered" (graphics-set-origin "0.5gw" "0.5gh"))
-	  ("Baseline" (graphics-set-origin "0cm" "0.5gh"))
-	  ("Axis" (graphics-set-origin "0cm" (length-add "0.5gh" "1yfrac")))
-	  ---
-	  ("Other" (interactive graphics-set-origin)))))
+(menu-bind graphics-extents-menu
+  ("Width" (interactive graphics-set-width))
+  ("Height" (interactive graphics-set-height)))
+
+(menu-bind graphics-auto-crop-menu
+  ("Crop" (graphics-toggle-auto-crop))
+  ---
+  (when (graphics-auto-crop?)
+    (group "Padding")
+    ("none" (graphics-set-crop-padding "0spc"))
+    ("1 spc" (graphics-set-crop-padding "1spc"))
+    ("1 em" (graphics-set-crop-padding "1em"))
+    ---
+    ("Other" (interactive graphics-set-crop-padding))))
+
+(menu-bind graphics-alignment-menu
+  ("Top" (graphics-set-geo-valign "top"))
+  ("Axis" (graphics-set-geo-valign "axis"))
+  ("Center" (graphics-set-geo-valign "center"))
+  ("Bottom" (graphics-set-geo-valign "bottom")))
+
+(menu-bind graphics-resize-menu
+  (group "Width")
+  ("Fast decrease" (graphics-decrease-hsize-fast))
+  ("Slow decrease" (graphics-decrease-hsize))
+  ("Slow increase" (graphics-increase-hsize))
+  ("Fast increase" (graphics-increase-hsize-fast))
+  ---
+  (group "Height")
+  ("Fast decrease" (graphics-decrease-vsize-fast))
+  ("Slow decrease" (graphics-decrease-vsize))
+  ("Slow increase" (graphics-increase-vsize))
+  ("Fast increase" (graphics-increase-vsize-fast)))
+
+(menu-bind graphics-frame-unit-menu
+  ("1 cm" (graphics-set-unit "1cm"))
+  ("1 inch" (graphics-set-unit "1in"))
+  ;;("5 em" (graphics-set-unit "5em"))
+  ---
+  ("Other" (interactive graphics-set-unit)))
+
+(menu-bind graphics-frame-origin-menu
+  ("Center" (graphics-set-origin "0.5gw" "0.5gh"))
+  ("Left top" (graphics-set-origin "0gw" "1gh"))
+  ("Left axis" (graphics-set-origin "0gw" (length-add "0.5gh" "1yfrac")))
+  ("Left center" (graphics-set-origin "0gw" "0.5gh"))
+  ("Left bottom" (graphics-set-origin "0gw" "0gh"))
+  ---
+  ("Other" (interactive graphics-set-origin)))
+
+(menu-bind graphics-move-menu
+  (group "Slow")
+  ("Left" (graphics-move-origin-left))
+  ("Right" (graphics-move-origin-right))
+  ("Down" (graphics-move-origin-down))
+  ("Up" (graphics-move-origin-up))
+  ---
+  (group "Fast")
+  ("Left" (graphics-move-origin-left-fast))
+  ("Right" (graphics-move-origin-right-fast))
+  ("Down" (graphics-move-origin-down-fast))
+  ("Up" (graphics-move-origin-up-fast)))
+
+(menu-bind graphics-zoom-menu
+  ("Zoom in" (graphics-zoom-in))
+  ("Zoom out" (graphics-zoom-out))
+  ---
+  ("10%" (graphics-set-zoom 0.1))
+  ("25%" (graphics-set-zoom 0.25))
+  ("50%" (graphics-set-zoom 0.5))
+  ("75%" (graphics-set-zoom 0.75))
+  ("100%" (graphics-set-zoom 1.0))
+  ("150%" (graphics-set-zoom 1.5))
+  ("200%" (graphics-set-zoom 2.0))
+  ("300%" (graphics-set-zoom 3.0))
+  ("400%" (graphics-set-zoom 4.0))
+  ("500%" (graphics-set-zoom 5.0))
+  ("1000%" (graphics-set-zoom 10.0)))
+
+(menu-bind graphics-global-menu
+  (group "Graphics")
+  (-> "Size" (link graphics-extents-menu))
+  (-> "Resize" (link graphics-resize-menu))
+  (-> "Crop" (link graphics-auto-crop-menu))
+  (-> "Alignment" (link graphics-alignment-menu))
+  ---
+  (-> "Unit" (link graphics-frame-unit-menu))
+  (-> "Origin" (link graphics-frame-origin-menu))
+  (-> "Move" (link graphics-move-menu))
+  (-> "Zoom" (link graphics-zoom-menu)))
 
 (menu-bind graphics-visual-grid-menu
   (-> "Type"
@@ -191,14 +258,15 @@
   (link graphics-visual-grid-menu))
 
 (menu-bind graphics-mode-menu
-  ("Point" (graphics-set-mode "point"))
-  ("Line" (graphics-set-mode "line"))
-  ("Polygon" (graphics-set-mode "cline"))
-  ("Spline" (graphics-set-mode "spline"))
-  ("Closed spline" (graphics-set-mode "cspline"))
-  ("Arc" (graphics-set-mode "arc"))
-  ("Circle" (graphics-set-mode "carc"))
-  ("Text box" (graphics-set-mode "text-at"))
+  ("Point" (graphics-set-mode '(edit point)))
+  ("Line" (graphics-set-mode '(edit line)))
+  ("Polygon" (graphics-set-mode '(edit cline)))
+  ("Spline" (graphics-set-mode '(edit spline)))
+  ("Closed spline" (graphics-set-mode '(edit cspline)))
+  ("Arc" (graphics-set-mode '(edit arc)))
+  ("Circle" (graphics-set-mode '(edit carc)))
+  ("Text" (graphics-set-mode '(edit text-at)))
+  ("Mathematics" (graphics-set-mode '(edit math-at)))
   ---
   ("Set properties" (graphics-set-mode '(group-edit props)))
   ("Move objects" (graphics-set-mode '(group-edit move)))
@@ -206,6 +274,21 @@
   ("Rotate objects" (graphics-set-mode '(group-edit rotate)))
   ("Group/ungroup" (graphics-set-mode '(group-edit group-ungroup))))
 
+(menu-bind graphics-opacity-menu
+  ("0%" (graphics-set-opacity "0%"))
+  ("10%" (graphics-set-opacity "10%"))
+  ("20%" (graphics-set-opacity "20%"))
+  ("30%" (graphics-set-opacity "30%"))
+  ("40%" (graphics-set-opacity "40%"))
+  ("50%" (graphics-set-opacity "50%"))
+  ("60%" (graphics-set-opacity "60%"))
+  ("70%" (graphics-set-opacity "70%"))
+  ("80%" (graphics-set-opacity "80%"))
+  ("90%" (graphics-set-opacity "90%"))
+  ("100%" (graphics-set-opacity "100%"))
+  ---
+  ("Other" (interactive graphics-set-opacity)))
+
 (menu-bind graphics-color-menu
   ;;("Default" (graphics-set-color "default"))
   ("None" (graphics-set-color "none"))
@@ -271,8 +354,8 @@
   (group "Style")
   ;;("Default" (graphics-set-dash-style "default"))
   ;;--
-  ("---"    (graphics-set-dash-style "default"))
-  (". . . . ."    (graphics-set-dash-style "10"))
+  ("-----" (graphics-set-dash-style "default"))
+  (". . . . ." (graphics-set-dash-style "10"))
   ("- - - - -" (graphics-set-dash-style "11100"))
   ("- . - . -" (graphics-set-dash-style "1111010"))
   ;;---
@@ -288,12 +371,27 @@
   ("Other" (interactive graphics-set-dash-style-unit)))
 
 (menu-bind graphics-line-arrows-menu
-  ;;("Default" (graphics-set-line-arrows "default"))
-  ;;---
-  ;;("---" (graphics-set-line-arrows 0))
-  ("---" (graphics-set-line-arrows "default"))
-  ("--->" (graphics-set-line-arrows 1))
-  ("<--->" (graphics-set-line-arrows 2)))
+  (group "Right arrow")
+  ("None" (graphics-set-arrow-end "default"))
+  ("--->" (graphics-set-arrow-end "<gtr>"))
+  ("---|>" (graphics-set-arrow-end "|<gtr>"))
+  ("--->>" (graphics-set-arrow-end "<gtr><gtr>"))
+  ("---<" (graphics-set-arrow-end "<less>"))
+  ("---<|" (graphics-set-arrow-end "<less>|"))
+  ("---<<" (graphics-set-arrow-end "<less><less>"))
+  ("---|" (graphics-set-arrow-end "|"))
+  ("---o" (graphics-set-arrow-end "o"))
+  ---
+  (group "Left arrow")
+  ("None" (graphics-set-arrow-begin "default"))
+  ("<---" (graphics-set-arrow-begin "<less>"))
+  ("<|---" (graphics-set-arrow-begin "<less>|"))
+  ("<<---" (graphics-set-arrow-begin "<less><less>"))
+  (">---" (graphics-set-arrow-begin "<gtr>"))
+  ("|>---" (graphics-set-arrow-begin "|<gtr>"))
+  (">>---" (graphics-set-arrow-begin "<gtr><gtr>"))
+  ("|---" (graphics-set-arrow-begin "|"))
+  ("o---" (graphics-set-arrow-begin "o")))
 
 (menu-bind graphics-fill-color-menu
   ;;("Default" (graphics-set-fill-color "default"))
@@ -307,61 +405,54 @@
   ("Other" (interactive graphics-set-fill-color)))
 
 (menu-bind graphics-text-halign-menu
-  ;;("Default" (graphics-set-textat-halign "default"))
+  ;;("Default" (graphics-set-text-at-halign "default"))
   ;;---
-  ;;("Left" (graphics-set-textat-halign "left"))
-  ("Left" (graphics-set-textat-halign "default"))
-  ("Center" (graphics-set-textat-halign "center"))
-  ("Right" (graphics-set-textat-halign "right")))
+  ;;("Left" (graphics-set-text-at-halign "left"))
+  ("Left" (graphics-set-text-at-halign "default"))
+  ("Center" (graphics-set-text-at-halign "center"))
+  ("Right" (graphics-set-text-at-halign "right")))
 
 (menu-bind graphics-text-valign-menu
-  ;;("Default" (graphics-set-textat-valign "default"))
+  ;;("Default" (graphics-set-text-at-valign "default"))
   ;;---
-  ("Bottom" (graphics-set-textat-valign "bottom"))
-  ;;("Base" (graphics-set-textat-valign "base"))
-  ("Base" (graphics-set-textat-valign "default"))
-  ("Center" (graphics-set-textat-valign "center"))
-  ("Top" (graphics-set-textat-valign "top")))
-
-(menu-bind graphics-enable-change-properties-menu
-  ("Color"  (graphics-toggle-color-enabled))
-  ("Fill color" (graphics-toggle-fill-color-enabled))
-  ("Point style" (graphics-toggle-point-style-enabled))
-  ("Line width" (graphics-toggle-line-width-enabled))
-  ("Dash style" (graphics-toggle-dash-style-enabled))
-  ("Dash unit" (graphics-toggle-dash-style-unit-enabled))
-  ("Line arrows" (graphics-toggle-line-arrows-enabled))
-  ("Text box horizontal alignment" (graphics-toggle-textat-halign-enabled))
-  ("Text box vertical alignment" (graphics-toggle-textat-valign-enabled)))
+  ("Bottom" (graphics-set-text-at-valign "bottom"))
+  ;;("Base" (graphics-set-text-at-valign "base"))
+  ("Base" (graphics-set-text-at-valign "default"))
+  ("Axis" (graphics-set-text-at-valign "axis"))
+  ("Center" (graphics-set-text-at-valign "center"))
+  ("Top" (graphics-set-text-at-valign "top")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Menus for graphics mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind graphics-insert-menu
-  (-> "Geometry" (link graphics-geometry-menu))
-  (-> "Grids" (link graphics-grids-menu))
+  (-> "Geometry" (link graphics-global-menu))
+  (-> "Grid" (link graphics-grids-menu))
   ---
-  (link graphics-mode-menu)
-  ;;(-> "Enable change" (link graphics-enable-change-properties-menu))
-  )
+  (link graphics-mode-menu))
 
 (menu-bind graphics-focus-menu
   (-> (eval (upcase-first (gr-mode->string (graphics-mode))))
       (link graphics-mode-menu))
-  (assuming (func? (graphics-mode) 'edit)
+  (assuming (nnull? (graphics-mode-attributes (graphics-mode)))
     ---
     (assuming (graphics-mode-attribute? (graphics-mode) "color")
       (-> "Color" (link graphics-color-menu)))
     (assuming (graphics-mode-attribute? (graphics-mode) "fill-color")
       (-> "Fill color" (link graphics-fill-color-menu)))
+    (assuming (graphics-mode-attribute? (graphics-mode) "opacity")
+      (assuming (== (get-preference "experimental alpha") "on")
+        (-> "Opacity" (link graphics-opacity-menu))))
     (assuming (graphics-mode-attribute? (graphics-mode) "point-style")
       (-> "Point style" (link graphics-point-style-menu)))
     (assuming (graphics-mode-attribute? (graphics-mode) "line-width")
       (-> "Line width" (link graphics-line-width-menu)))
     (assuming (graphics-mode-attribute? (graphics-mode) "dash-style")
       (-> "Line dashes" (link graphics-dash-menu)))
-    (assuming (graphics-mode-attribute? (graphics-mode) "line-arrows")
+    (assuming
+        (or (graphics-mode-attribute? (graphics-mode) "arrow-begin")
+            (graphics-mode-attribute? (graphics-mode) "arrow-end"))
       (-> "Line arrows" (link graphics-line-arrows-menu)))
     (assuming (graphics-mode-attribute? (graphics-mode) "text-at-halign")
       (-> "Horizontal alignment" (link graphics-text-halign-menu)))
@@ -374,7 +465,7 @@
 
 (tm-menu (graphics-global-icons)
   (=> (balloon (icon "tm_graphics_geometry.xpm") "Graphics geometry")
-      (link graphics-geometry-menu))
+      (link graphics-global-menu))
   (=> (balloon (icon "tm_graphics_grid.xpm") "Graphics grids")
       (link graphics-grids-menu)))
 
@@ -383,32 +474,33 @@
   ;;    (link graphics-mode-menu))
   ((check (balloon (icon "tm_point_mode.xpm") "Insert points")
           "v" (== (graphics-mode) '(edit point)))
-   (graphics-set-mode "point"))
+   (graphics-set-mode '(edit point)))
   ((check (balloon (icon "tm_line_mode.xpm") "Insert lines")
           "v" (== (graphics-mode) '(edit line)))
-   (graphics-set-mode "line"))
+   (graphics-set-mode '(edit line)))
   ((check (balloon (icon "tm_cline_mode.xpm") "Insert polygons")
           "v" (== (graphics-mode) '(edit cline)))
-   (graphics-set-mode "cline"))
+   (graphics-set-mode '(edit cline)))
   ((check (balloon (icon "tm_spline_mode.xpm") "Insert splines")
           "v" (== (graphics-mode) '(edit spline)))
-   (graphics-set-mode "spline"))
+   (graphics-set-mode '(edit spline)))
   ((check (balloon (icon "tm_cspline_mode.xpm") "Insert closed splines")
           "v" (== (graphics-mode) '(edit cspline)))
-   (graphics-set-mode "cspline"))
+   (graphics-set-mode '(edit cspline)))
   ((check (balloon (icon "tm_arc_mode.xpm") "Insert arcs")
           "v" (== (graphics-mode) '(edit arc)))
-   (graphics-set-mode "arc"))
+   (graphics-set-mode '(edit arc)))
   ((check (balloon (icon "tm_carc_mode.xpm") "Insert circles")
           "v" (== (graphics-mode) '(edit carc)))
-   (graphics-set-mode "carc"))
-  ((check (balloon (icon "tm_textat_mode.xpm") "Insert text boxes")
+   (graphics-set-mode '(edit carc)))
+  ((check (balloon (icon "tm_textat_mode.xpm") "Insert text")
           "v" (== (graphics-mode) '(edit text-at)))
-   (graphics-set-mode "text-at")))
+   (graphics-set-mode '(edit text-at)))
+  ((check (balloon (icon "tm_math.xpm") "Insert mathematics")
+          "v" (== (graphics-mode) '(edit math-at)))
+   (graphics-set-mode '(edit math-at))))
 
 (tm-menu (graphics-group-property-icons)
-  ;;(=> (balloon (icon "tm_toggle_change_props.xpm") "Enable change property")
-  ;;    (link graphics-enable-change-properties-menu))
   ((check (balloon (icon "tm_edit_props.xpm") "Change objects properties")
           "v" (== (graphics-mode) '(group-edit props)))
    (graphics-set-mode '(group-edit props))))
@@ -456,6 +548,15 @@
         (assuming (and (!= col "default") (!= col "none"))
           (=> (color (eval col) #f #f 25 17)
               (link graphics-fill-color-menu))))))
+  (assuming (== (get-preference "experimental alpha") "on")
+    (assuming (graphics-mode-attribute? (graphics-mode) "opacity")
+      /
+      (mini #t
+        (group "Opacity:")
+        (let* ((o (graphics-get-property "gr-opacity"))
+               (s (if (== o "default") "100%" o)))
+          (=> (eval s)
+              (link graphics-opacity-menu))))))
   (assuming (graphics-mode-attribute? (graphics-mode) "point-style")
     /
     (mini #t
@@ -464,42 +565,42 @@
              (s (if (== ps "default") "disk" ps)))
 	(=> (eval s)
 	    (link graphics-point-style-menu)))))
-  (assuming (graphics-mode-attribute? (graphics-mode) "line-width")
+  (assuming
+      (or (graphics-mode-attribute? (graphics-mode) "line-width")
+          (graphics-mode-attribute? (graphics-mode) "dash-style"))
     /
     (mini #t
-      (group "Line width:")
+      (group "Line style:")
       (let* ((lw (graphics-get-property "gr-line-width"))
              (s (if (== lw "default") "1ln" lw)))
 	(=> (eval s)
-	    (link graphics-line-width-menu)))))
-  (assuming (graphics-mode-attribute? (graphics-mode) "dash-style")
-    /
-    (mini #t
-      (group "Dash:")
+	    (link graphics-line-width-menu)))
       (let* ((dash (graphics-get-property "gr-dash-style"))
              (s (decode-dash dash)))
         (=> (eval s)
             (link graphics-dash-menu)))))
-  (assuming (graphics-mode-attribute? (graphics-mode) "line-arrows")
+  (assuming
+      (or (graphics-mode-attribute? (graphics-mode) "arrow-begin")
+          (graphics-mode-attribute? (graphics-mode) "arrow-end"))
     /
     (mini #t
       (group "Arrows:")
-      (let* ((arrows (graphics-get-property "gr-line-arrows"))
-             (s (decode-arrows arrows)))
+      (let* ((arrow-begin (graphics-get-property "gr-arrow-begin"))
+             (arrow-end (graphics-get-property "gr-arrow-end"))
+             (s (string-append (decode-arrow arrow-begin)
+                               "---"
+                               (decode-arrow arrow-end))))
         (=> (eval s)
             (link graphics-line-arrows-menu)))))
-  (assuming (graphics-mode-attribute? (graphics-mode) "text-at-halign")
+  (assuming (or (graphics-mode-attribute? (graphics-mode) "text-at-halign")
+                (graphics-mode-attribute? (graphics-mode) "text-at-valign"))
     /
     (mini #t
-      (group "Horizontal alignment:")
+      (group "Alignment:")
       (let* ((al (graphics-get-property "gr-text-at-halign"))
              (s (if (== al "default") "left" al)))
 	(=> (eval s)
-	    (link graphics-text-halign-menu)))))
-  (assuming (graphics-mode-attribute? (graphics-mode) "text-at-halign")
-    /
-    (mini #t
-      (group "Vertical alignment:")
+	    (link graphics-text-halign-menu)))
       (let* ((al (graphics-get-property "gr-text-at-valign"))
              (s (if (== al "default") "base" al)))
 	(=> (eval s)
@@ -514,6 +615,7 @@
         ((== s '(edit arc)) "arc")
         ((== s '(edit carc)) "circle")
         ((== s '(edit text-at)) "text")
+        ((== s '(edit math-at)) "mathematics")
         ((== s '(group-edit props)) "properties")
         ((== s '(group-edit move)) "move")
         ((== s '(group-edit zoom)) "resize")
@@ -534,4 +636,5 @@
     (=> (balloon (eval (upcase-first (gr-mode->string (graphics-mode))))
                  "Current graphical mode")
         (link graphics-mode-menu)))
-  (link graphics-property-icons))
+  (assuming (nnull? (graphics-mode-attributes (graphics-mode)))
+    (link graphics-property-icons)))
