@@ -16,7 +16,7 @@
 ;; TODO: scheme file caching using (set! primitive-load ...) and
 ;; (set! %search-load-path)
 
-;; (debug-enable 'backtrace 'debug)
+;;(debug-enable 'backtrace 'debug)
 ;; (define load-indent 0)
 ;; (define old-primitive-load primitive-load)
 ;; (define (new-primitive-load . x)
@@ -39,30 +39,35 @@
 (inherit-modules (kernel library base) (kernel library list)
 		 (kernel library tree) (kernel library content))
 (inherit-modules (kernel regexp regexp-match) (kernel regexp regexp-select))
-(inherit-modules (kernel drd drd-rules) (kernel drd drd-query)
-		 (kernel drd drd-data))
-(inherit-modules (kernel texmacs tm-overload) (kernel texmacs tm-define)
+(inherit-modules (kernel logic logic-rules) (kernel logic logic-query)
+		 (kernel logic logic-data))
+(inherit-modules (kernel texmacs tm-define)
 		 (kernel texmacs tm-preferences) (kernel texmacs tm-modes)
 		 (kernel texmacs tm-plugins) (kernel texmacs tm-secure)
 		 (kernel texmacs tm-convert) (kernel texmacs tm-dialogue)
-		 (kernel texmacs tm-language)  (kernel texmacs tm-file-system))
+		 (kernel texmacs tm-language) (kernel texmacs tm-file-system)
+                 (kernel texmacs tm-states))
 (inherit-modules (kernel gui gui-markup)
                  (kernel gui menu-define) (kernel gui menu-widget)
 		 (kernel gui kbd-define) (kernel gui kbd-handlers)
-		 (kernel gui gui-widget) (kernel gui gui-factory)
-		 (kernel gui gui-form))
+                 (kernel gui menu-test)
+		 (kernel old-gui old-gui-widget)
+                 (kernel old-gui old-gui-factory)
+		 (kernel old-gui old-gui-form)
+                 (kernel old-gui old-gui-test))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
 ;(display "Booting utilities\n")
 (lazy-define (utils cas cas-out) cas->stree)
 (use-modules (utils misc markup-funcs))
+(use-modules (utils handwriting handwriting))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
 ;(display "Booting BibTeX style modules\n")
 (use-modules (bibtex bib-utils))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
-;(display "Booting main TeXmacs functionality\n")
+;;(display "Booting main TeXmacs functionality\n")
 (use-modules (texmacs texmacs tm-server) (texmacs texmacs tm-view)
 	     (texmacs texmacs tm-files) (texmacs texmacs tm-print))
 (use-modules (texmacs keyboard config-kbd))
@@ -128,15 +133,18 @@
 ;(display "Booting table mode\n")
 (lazy-keyboard (table table-kbd) in-table?)
 (lazy-menu (table table-menu) insert-table-menu)
+(lazy-define (table table-edit) table-resize-notify)
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
 ;(display "Booting graphics mode\n")
+(lazy-keyboard (graphics graphics-kbd) in-active-graphics?)
 (lazy-menu (graphics graphics-menu) graphics-menu graphics-icons)
 (lazy-define (graphics graphics-edit)
+             graphics-busy?
 	     graphics-reset-context graphics-undo-enabled
-	     graphics-insert-point graphics-remove-point
-	     graphics-last-point graphics-start-drag
-	     graphics-dragging graphics-end-drag)
+	     graphics-release-left graphics-release-middle
+	     graphics-release-right graphics-start-drag-left
+	     graphics-dragging-left graphics-end-drag-left)
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
 ;(display "Booting formal languages\n")
@@ -147,17 +155,20 @@
 ;(display "Booting dynamic features\n")
 (lazy-keyboard (dynamic fold-kbd) always?)
 (lazy-keyboard (dynamic scripts-kbd) always?)
+(lazy-keyboard (dynamic calc-kbd) always?)
 (lazy-menu (dynamic fold-menu) insert-fold-menu dynamic-menu dynamic-icons)
 (lazy-menu (dynamic session-menu) insert-session-menu session-help-icons)
 (lazy-menu (dynamic scripts-menu) scripts-eval-menu scripts-plot-menu
 	   plugin-eval-menu plugin-eval-toggle-menu plugin-plot-menu)
+(lazy-menu (dynamic calc-menu) calc-table-menu calc-insert-menu)
 (lazy-define (dynamic session-edit) scheme-eval)
-(lazy-define (dynamic form-edit) form-ref form-set! form-toggle)
+(lazy-define (dynamic calc-edit) calc-ready? calc-table-renumber)
 (lazy-initialize (dynamic session-menu) (in-session?))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
 ;(display "Booting documentation\n")
-(lazy-keyboard (doc tmdoc-kbd) in-tmdoc?)
+(lazy-keyboard (doc tmdoc-kbd) in-manual?)
+(lazy-menu (doc tmdoc-menu) tmdoc-menu tmdoc-icons)
 (lazy-menu (doc help-menu) help-menu)
 (lazy-define (doc tmdoc) tmdoc-expand-help tmdoc-expand-help-manual
 	     tmdoc-expand-this tmdoc-include)
@@ -175,7 +186,7 @@
 (lazy-format (convert bibtex init-bibtex) bibtex)
 (lazy-format (convert images init-images)
 	     postscript pdf xfig xmgrace svg xpm jpeg ppm gif png pnm)
-(lazy-define (convert rewrite init-rewrite) texmacs->verbatim)
+(lazy-define (convert rewrite init-rewrite) texmacs->code texmacs->verbatim)
 (lazy-define (convert html tmhtml-expand) tmhtml-env-patch)
 (lazy-define (convert latex latex-drd) latex-arity latex-type)
 (lazy-define (convert latex textm) textm-finalize)
