@@ -79,19 +79,21 @@
 ;; Common programming constructs
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(provide-public call/cc call-with-current-continuation)
+(define-public-macro (when cond? . body)
+  `(if ,cond? (begin ,@body)))
 
-(define-public-macro (with-cc cont . body)
-  `(call/cc (lambda (,cont) ,@body)))
-
-(define-public-macro (when cond . body)
-  `(if ,cond (begin ,@body)))
+(define-public-macro (unless cond? . body)
+  `(if (not ,cond?) (begin ,@body)))
 
 (define-public-macro (with var val . body)
   (if (pair? var)
       `(apply (lambda ,var ,@body) ,val)
       `(let ((,var ,val)) ,@body)))
 
+(define-public-macro (with-define fun fun-body . body)
+  `(let ((,(car fun) (lambda ,(cdr fun) ,fun-body)))
+     ,@body))
+
 (define-public-macro (with-global var val . body)
   (let ((old (gensym)) (new (gensym)))
     `(let ((,old ,var))
