@@ -202,6 +202,9 @@
 (define-public (symbol-ends? s1 s2)
   (string-ends? (symbol->string s1) (symbol->string s2)))
 
+(define-public (symbol-drop s n)
+  (string->symbol (string-drop (symbol->string s) n)))
+
 (define-public (symbol-drop-right s n)
   (string->symbol (string-drop-right (symbol->string s) n)))
 
@@ -272,16 +275,24 @@
 	((null? (cdr l)) (car l))
 	(else (url-or (car l) (list->url (cdr l))))))
 
+(define-public (url-read-directory u wc)
+  (with d (url-expand (url-complete (url-append u (url-wildcard wc)) "r"))
+    (url->list d)))
+
 (define-public (buffer->tree u)
-  (with t (get-buffer-tree u)
+  (with t (buffer-get-tree u)
     (and (tree-active? t) t)))
 
 (define-public (tree->buffer t)
   (and-with p (tree->path t)
-    (get-name-buffer-path p)))
+    (path->buffer p)))
+
+(define-public (buffer->path u)
+  (with t (buffer->tree u)
+    (and t (tree->path t))))
 
 (define-public (buffer-list)
-  (url->list (get-all-buffers)))
+  (url->list (all-buffers)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Redirections
