@@ -87,7 +87,10 @@
 	  (-> "Series"
 	      ("Light" (make-with "math-font-series" "light"))
 	      ("Medium" (make-with "math-font-series" "medium"))
-	      ("Bold" (make-with "math-font-series" "bold")))))
+	      ("Bold" (make-with "math-font-series" "bold"))))
+      (-> "Shape"
+	  ("Normal" (make-with "math-font-shape" "normal"))
+	  ("Upight" (make-with "math-font-shape" "right"))))
   (if (not (real-math-font? (get-env "math-font")))
       (-> "Variant"
 	  ("Roman" (make-with "math-font-family" "mr"))
@@ -132,6 +135,8 @@
       ---
       (group "Text")
       (-> "Color" (link color-menu))
+      (if (== (get-preference "experimental alpha") "on")
+          (-> "Opacity" (link opacity-menu)))
       (-> "Scripts" (link local-supported-scripts-menu))
       (-> "Space" (link horizontal-space-menu))
       (-> "Transform" (link transform-menu))
