@@ -694,6 +694,52 @@
   ("< = = !" "<Longleftarrowlim>")
   ("< = = > !" "<Longleftrightarrowlim>")
 
+  ("- ^" (make-long-arrow "<minus>"))
+  ("- > ^" (make-long-arrow "<rightarrow>"))
+  ("< - ^" (make-long-arrow "<leftarrow>"))
+  ("< - > ^" (make-long-arrow "<leftrightarrow>"))
+  ("| - > ^" (make-long-arrow "<mapsto>"))
+  ("| var - > ^" (make-long-arrow "<mapsto>"))
+  ("< - | ^" (make-long-arrow "<mapsfrom>"))
+  ("= ^" (make-long-arrow "<equal>"))
+  ("= > ^" (make-long-arrow "<Rightarrow>"))
+  ("< = ^" (make-long-arrow "<Leftarrow>"))
+  ("< = > ^" (make-long-arrow "<Leftrightarrow>"))
+  ("- - ^" (make-long-arrow "<minus>"))
+  ("- - > ^" (make-long-arrow "<rightarrow>"))
+  ("< - - ^" (make-long-arrow "<leftarrow>"))
+  ("< - - > ^" (make-long-arrow "<leftrightarrow>"))
+  ("| - - > ^" (make-long-arrow "<mapsto>"))
+  ("| var - - > ^" (make-long-arrow "<mapsto>"))
+  ("< - - | ^" (make-long-arrow "<mapsfrom>"))
+  ("= = ^" (make-long-arrow "<equal>"))
+  ("= = > ^" (make-long-arrow "<Rightarrow>"))
+  ("< = = ^" (make-long-arrow "<Leftarrow>"))
+  ("< = = > ^" (make-long-arrow "<Leftrightarrow>"))
+
+  ("- _" (make-long-arrow* "<minus>"))
+  ("- > _" (make-long-arrow* "<rightarrow>"))
+  ("< - _" (make-long-arrow* "<leftarrow>"))
+  ("< - > _" (make-long-arrow* "<leftrightarrow>"))
+  ("| - > _" (make-long-arrow* "<mapsto>"))
+  ("| var - > _" (make-long-arrow* "<mapsto>"))
+  ("< - | _" (make-long-arrow* "<mapsfrom>"))
+  ("= _" (make-long-arrow* "<equal>"))
+  ("= > _" (make-long-arrow* "<Rightarrow>"))
+  ("< = _" (make-long-arrow* "<Leftarrow>"))
+  ("< = > _" (make-long-arrow* "<Leftrightarrow>"))
+  ("- - _" (make-long-arrow* "<minus>"))
+  ("- - > _" (make-long-arrow* "<rightarrow>"))
+  ("< - - _" (make-long-arrow* "<leftarrow>"))
+  ("< - - > _" (make-long-arrow* "<leftrightarrow>"))
+  ("| - - > _" (make-long-arrow* "<mapsto>"))
+  ("| var - - > _" (make-long-arrow* "<mapsto>"))
+  ("< - - | _" (make-long-arrow* "<mapsfrom>"))
+  ("= = _" (make-long-arrow* "<equal>"))
+  ("= = > _" (make-long-arrow* "<Rightarrow>"))
+  ("< = = _" (make-long-arrow* "<Leftarrow>"))
+  ("< = = > _" (make-long-arrow* "<Leftrightarrow>"))
+
   ("@" "<circ>")
   ("@ /" "<varnothing>")
   ("@ +" "<oplus>")
@@ -1051,6 +1097,59 @@
   ("math:bold j var" "<b-jmath>")
   ("math:bold l var" "<b-ell>")
 
+  ("math:bold:up a" "<b-up-a>")
+  ("math:bold:up b" "<b-up-b>")
+  ("math:bold:up c" "<b-up-c>")
+  ("math:bold:up d" "<b-up-d>")
+  ("math:bold:up e" "<b-up-e>")
+  ("math:bold:up f" "<b-up-f>")
+  ("math:bold:up g" "<b-up-g>")
+  ("math:bold:up h" "<b-up-h>")
+  ("math:bold:up i" "<b-up-i>")
+  ("math:bold:up j" "<b-up-j>")
+  ("math:bold:up k" "<b-up-k>")
+  ("math:bold:up l" "<b-up-l>")
+  ("math:bold:up m" "<b-up-m>")
+  ("math:bold:up n" "<b-up-n>")
+  ("math:bold:up o" "<b-up-o>")
+  ("math:bold:up p" "<b-up-p>")
+  ("math:bold:up q" "<b-up-q>")
+  ("math:bold:up r" "<b-up-r>")
+  ("math:bold:up s" "<b-up-s>")
+  ("math:bold:up t" "<b-up-t>")
+  ("math:bold:up u" "<b-up-u>")
+  ("math:bold:up v" "<b-up-v>")
+  ("math:bold:up w" "<b-up-w>")
+  ("math:bold:up x" "<b-up-x>")
+  ("math:bold:up y" "<b-up-y>")
+  ("math:bold:up z" "<b-up-z>")
+  ("math:bold:up A" "<b-up-A>")
+  ("math:bold:up B" "<b-up-B>")
+  ("math:bold:up C" "<b-up-C>")
+  ("math:bold:up D" "<b-up-D>")
+  ("math:bold:up E" "<b-up-E>")
+  ("math:bold:up F" "<b-up-F>")
+  ("math:bold:up G" "<b-up-G>")
+  ("math:bold:up H" "<b-up-H>")
+  ("math:bold:up I" "<b-up-I>")
+  ("math:bold:up J" "<b-up-J>")
+  ("math:bold:up K" "<b-up-K>")
+  ("math:bold:up L" "<b-up-L>")
+  ("math:bold:up M" "<b-up-M>")
+  ("math:bold:up N" "<b-up-N>")
+  ("math:bold:up O" "<b-up-O>")
+  ("math:bold:up P" "<b-up-P>")
+  ("math:bold:up Q" "<b-up-Q>")
+  ("math:bold:up R" "<b-up-R>")
+  ("math:bold:up S" "<b-up-S>")
+  ("math:bold:up T" "<b-up-T>")
+  ("math:bold:up U" "<b-up-U>")
+  ("math:bold:up V" "<b-up-V>")
+  ("math:bold:up W" "<b-up-W>")
+  ("math:bold:up X" "<b-up-X>")
+  ("math:bold:up Y" "<b-up-Y>")
+  ("math:bold:up Z" "<b-up-Z>")
+
   ("math:bold:greek a" "<b-alpha>")
   ("math:bold:greek b" "<b-beta>")
   ("math:bold:greek g" "<b-gamma>")
