@@ -177,7 +177,9 @@
       ---
       (tile 8 (link vertical-arrow-menu))
       ---
-      (tile 6 (link long-arrow-menu)))
+      (tile 6 (link long-arrow-menu))
+      ---
+      (link extensible-arrow-menu))
   (-> "Negation"
       ("General negation" (key-press "/"))
       ---
@@ -536,6 +538,10 @@
   (symbol "<longhookrightarrow>")
   (symbol "<longmapsto>"))
 
+(menu-bind extensible-arrow-menu
+  ("Extensible left arrow" (make-long-arrow "<leftarrow>"))
+  ("Extensible right arrow" (make-long-arrow "<rightarrow>")))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Negations
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -789,6 +795,60 @@
   (symbol "<b-Y>")
   (symbol "<b-Z>"))
 
+(menu-bind bold-up-alpha-menu
+  (symbol "<b-up-a>")
+  (symbol "<b-up-b>")
+  (symbol "<b-up-c>")
+  (symbol "<b-up-d>")
+  (symbol "<b-up-e>")
+  (symbol "<b-up-f>")
+  (symbol "<b-up-g>")
+  (symbol "<b-up-h>")
+  (symbol "<b-up-i>")
+  (symbol "<b-up-j>")
+  (symbol "<b-up-k>")
+  (symbol "<b-up-l>")
+  (symbol "<b-up-m>")
+  (symbol "<b-up-n>")
+  (symbol "<b-up-o>")
+  (symbol "<b-up-p>")
+  (symbol "<b-up-q>")
+  (symbol "<b-up-r>")
+  (symbol "<b-up-s>")
+  (symbol "<b-up-t>")
+  (symbol "<b-up-u>")
+  (symbol "<b-up-v>")
+  (symbol "<b-up-w>")
+  (symbol "<b-up-x>")
+  (symbol "<b-up-y>")
+  (symbol "<b-up-z>")
+  (symbol "<b-up-A>")
+  (symbol "<b-up-B>")
+  (symbol "<b-up-C>")
+  (symbol "<b-up-D>")
+  (symbol "<b-up-E>")
+  (symbol "<b-up-F>")
+  (symbol "<b-up-G>")
+  (symbol "<b-up-H>")
+  (symbol "<b-up-I>")
+  (symbol "<b-up-J>")
+  (symbol "<b-up-K>")
+  (symbol "<b-up-L>")
+  (symbol "<b-up-M>")
+  (symbol "<b-up-N>")
+  (symbol "<b-up-O>")
+  (symbol "<b-up-P>")
+  (symbol "<b-up-Q>")
+  (symbol "<b-up-R>")
+  (symbol "<b-up-S>")
+  (symbol "<b-up-T>")
+  (symbol "<b-up-U>")
+  (symbol "<b-up-V>")
+  (symbol "<b-up-W>")
+  (symbol "<b-up-X>")
+  (symbol "<b-up-Y>")
+  (symbol "<b-up-Z>"))
+
 (menu-bind bold-greek-menu
   (symbol "<b-alpha>")
   (symbol "<b-beta>")
@@ -1021,7 +1081,9 @@
       ---
       (tile 8 (link vertical-arrow-menu))
       ---
-      (tile 6 (link long-arrow-menu)))
+      (tile 6 (link long-arrow-menu))
+      ---
+      (link extensible-arrow-menu))
   (=> (balloon (icon "tm_unequal.xpm") "Insert a negation")
       (tile 9 (link negation-menu-1))
       ---
@@ -1041,6 +1103,8 @@
       ---
       (tile 13 (link bold-alpha-menu))
       ---
+      (tile 13 (link bold-up-alpha-menu))
+      ---
       (tile 15 (link bold-greek-menu)))
   (=> (balloon (icon "tm_cal.xpm")
 	       "Insert a calligraphic character")
