@@ -18,25 +18,65 @@
     (texmacs texmacs tm-files)
     (texmacs texmacs tm-print)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Dynamic menu for existing buffers
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-menu (buffer-list-menu l)
+  (for (name l)
+    (let* ((abbr (buffer-get-short-name name))
+           (abbr* (if (== abbr "") (url->string (url-tail name)) abbr))
+           (mod? (buffer-modified? name))
+           (short-name (string-append abbr* (if mod? " *" "")))
+           (long-name (url->string name)))
+      ((balloon (eval short-name) (eval long-name))
+       (switch-to-buffer name)))))
+
+(tm-define (buffer-more-recent? b1 b2)
+  (>= (buffer-last-visited b1)
+      (buffer-last-visited b2)))
+
+(tm-define (buffer-sorted-list)
+  (with l (list-filter (buffer-list) buffer-in-menu?)
+    (list-sort l buffer-more-recent?)))
+
+(tm-define (buffer-menu-list aux?)
+  (let* ((l1 (list-filter (buffer-list) buffer-in-menu?))
+         (l2 (list-filter l1 (lambda (x) (== aux? (aux-buffer? x)))))
+         (l3 (list-sort l2 buffer-more-recent?)))
+    (sublist l3 0 (min (length l3) 10))))
+
+(tm-define (buffer-same-list)
+  (buffer-menu-list (aux-buffer? (this-buffer))))
+
+(tm-define (buffer-other-list)
+  (buffer-menu-list (not (aux-buffer? (this-buffer)))))
+
+(tm-define (buffer-same-menu)
+  (buffer-list-menu (buffer-same-list)))
+
+(tm-define (buffer-other-menu)
+  (buffer-list-menu (buffer-other-list)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Dynamic menu for recent files
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(tm-menu (file-list-menu l)
+  (for (name l)
+    (let* ((short-name (url->string (url-tail name))))
+      ((balloon (eval short-name) (eval name)) (load-buffer name)))))
+
 (tm-define (recent-file-list nr)
   (with l (map cdar (learned-interactive "recent-buffer"))
     (sublist l 0 (min (length l) nr))))
 
 (tm-define (recent-unloaded-file-list nr)
   (let* ((l1 (map cdar (learned-interactive "recent-buffer")))
-         (l2 (map url->string (url->list (get-all-buffers))))
+         (l2 (map url->string (buffer-list)))
          (dl (list-difference l1 l2)))
     (sublist dl 0 (min (length dl) nr))))
 
-(tm-menu (file-list-menu l)
-  (for (name l)
-    (let* ((short-name (url->string (url-tail name))))
-      ((balloon (eval short-name) (eval name)) (load-buffer name)))))
-
 (tm-define (recent-file-menu)
   (file-list-menu (recent-file-list 25)))
 
@@ -163,7 +203,10 @@
   (when (cursor-has-future?)
     ("Forward" (cursor-history-forward)))
   ---
-  (link buffer-menu)
+  (link buffer-same-menu)
+  (if (nnull? (buffer-other-list))
+      ---
+      (link buffer-other-menu))
   (if (nnull? (recent-unloaded-file-list 1))
       ---
       (link recent-unloaded-file-menu))
