@@ -20,8 +20,8 @@
 
 (menu-bind texmacs-extra-menu)
 (menu-bind texmacs-extra-icons)
-(tm-define (buffer-menu) (get-buffer-menu))
-(tm-define (project-buffer-menu) (get-project-buffer-menu))
+(menu-bind plugin-menu)
+(menu-bind plugin-icons)
 (tm-define (style-menu) (get-style-menu))
 (tm-define (add-package-menu) (get-add-package-menu))
 (tm-define (remove-package-menu) (get-remove-package-menu))
@@ -38,13 +38,19 @@
   (=> "Edit" (link edit-menu))
   (if (in-graphics?)
       (=> "Insert" (link graphics-insert-menu))
+      (link texmacs-extra-menu)
       (=> "Focus" (link graphics-focus-menu)))
   (if (not (in-graphics?))
       (=> "Insert" (link insert-menu))
+      (if (in-manual?)
+	  (=> "Manual" (link tmdoc-menu)))
       (if (or (in-source?) (with-source-tool?))
 	  (=> "Source" (link source-menu)))
       (if (with-linking-tool?)
 	  (=> "Link" (link link-menu)))
+      (if (in-presentation?)
+	  (=> "Dynamic" (link dynamic-menu)))
+      (link plugin-menu)
       (link texmacs-extra-menu)
       (=> "Focus" (link focus-menu))
       (=> "Format" (link format-menu)))
@@ -80,10 +86,14 @@
       (-> "Focus" (link graphics-focus-menu)))
   (if (not (in-graphics?))
       (-> "Insert" (link insert-menu))
+      (if (in-manual?)
+	  (-> "Manual" (link tmdoc-menu)))
       (if (or (in-source?) (with-source-tool?))
 	  (-> "Source" (link source-menu)))
       (if (with-linking-tool?)
 	  (-> "Link" (link link-menu)))
+      (if (in-presentation?)
+	  (-> "Dynamic" (link dynamic-menu)))
       (-> "Format" (link format-menu)))
   (-> "Document" (link document-menu))
   (if (== (get-init-tree "sectional-short-style") (tree 'macro "false"))
@@ -146,7 +156,28 @@
   ((balloon (icon "tm_reload.xpm") "Reload")
    (revert-buffer))
   ((balloon (icon "tm_forward.xpm") "Browse forward")
-   (cursor-history-forward)))
+   (cursor-history-forward))
+  (if (in-presentation?)
+    |
+    (link dynamic-icons)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; The TeXmacs side tools
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (upward-context-trees t)
+  (cond ((tree-is-buffer? t) (list t))
+        ((tree-atomic? t) (upward-context-trees (tree-up t)))
+        (else (cons t (upward-context-trees (tree-up t))))))
+
+(tm-widget (texmacs-side-tool t)
+  ((eval (symbol->string (tree-label t)))
+   (tree-select t)))
+
+(tm-widget (texmacs-side-tools)
+  (for (t (upward-context-trees (cursor-tree)))
+    (dynamic (texmacs-side-tool t))
+    ===))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The mode dependent icon bar
