@@ -21,6 +21,9 @@
 (define (get-default-interactive-questions)
   (if (or (like-gnome?) (like-macos?) (like-windows?)) "popups" "footer"))
 
+(define (get-default-show-table-cells)
+  (if (qt-gui?) "on" "off"))
+
 (define (notify-look-and-feel var val)
   (set-message "Restart in order to let the new look and feel take effect"
 	       "configure look and feel"))
@@ -62,7 +65,7 @@
   ("language" (get-locale-language) notify-language)
   ("fast environments" "on" notify-fast-environments)
   ("show full context" "on" (lambda args (noop)))
-  ("show table cells" "off" (lambda args (noop)))
+  ("show table cells" (get-default-show-table-cells) (lambda args (noop)))
   ("show focus" "on" (lambda args (noop)))
   ("show only semantic focus" "on" (lambda args (noop)))
   ("semantic editing" "off" (lambda args (noop)))
@@ -80,9 +83,11 @@
   ("scripting language" "none" notify-scripting-language)
   ("debugging tool" "off" notify-tool)
   ("linking tool" "off" notify-tool)
+  ("presentation tool" "off" notify-tool)
   ("source tool" "off" notify-tool)
   ("versioning tool" "off" notify-tool)
-  ("remote connections" "off" notify-tool))
+  ("remote connections" "off" notify-tool)
+  ("experimental alpha" "off" notify-tool))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Properties of some built-in routines
@@ -98,7 +103,7 @@
   (string<=? (symbol->string s1) (symbol->string s2)))
 
 (define (get-function-list)
-  (list-sort (map car (ahash-table->list ovl-table)) symbol<=?))
+  (list-sort (map car (ahash-table->list tm-defined-table)) symbol<=?))
 
 (define (get-interactive-function-list)
   (let* ((funs (get-function-list))
@@ -116,11 +121,9 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (tm-define (safely-kill-buffer)
-  (dialogue
-    (if (or (not (buffer-unsaved?))
-	    (dialogue-confirm?
-	     "The buffer has not been saved. Really close it?" #f))
-	(kill-buffer))))
+  (if (not (buffer-unsaved?)) (kill-buffer)
+      (user-confirm "The buffer has not been saved. Really close it?" #f  
+        (lambda (answ) (when answ (kill-buffer))))))
 
 (tm-define (safely-kill-window)
   (if (<= (get-nr-windows) 1)
@@ -128,8 +131,6 @@
       (kill-window)))
 
 (tm-define (safely-quit-TeXmacs)
-  (dialogue
-    (if (or (not (exists-unsaved-buffer?))
-	    (dialogue-confirm?
-	     "There are unsaved files. Really quit?" #f))
-	(quit-TeXmacs))))
+  (if (not (exists-unsaved-buffer?)) (quit-TeXmacs)
+      (user-confirm "There are unsaved files. Really quit?" #f  
+        (lambda (answ) (when answ (quit-TeXmacs))))))
