@@ -5,10 +5,9 @@
 ;; DESCRIPTION : setting the view preferences and properties
 ;; COPYRIGHT   : (C) 2001  Joris van der Hoeven
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -18,16 +17,21 @@
 ;; View preferences
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define (main-icon-bar-default)
+  (if (like-macos?) "off" "on"))
+
 (define (notify-header var val)
   (show-header (== val "on")))
 
 (define (notify-icon-bar var val)
   (cond ((== var "main icon bar")
 	 (show-icon-bar 0 (== val "on")))
-	((== var "context dependent icons")
+	((== var "mode dependent icons")
 	 (show-icon-bar 1 (== val "on")))
+	((== var "focus dependent icons")
+	 (show-icon-bar 2 (== val "on")))
 	((== var "user provided icons")
-	 (show-icon-bar 2 (== val "on")))))
+	 (show-icon-bar 3 (== val "on")))))
 
 (define (notify-status-bar var val)
   (show-footer (== val "on")))
@@ -36,13 +40,25 @@
   (set-default-shrinking-factor (string->number val))
   (set-shrinking-factor (string->number val)))
 
+(define (notify-remote-control var val)
+  (ahash-set! remote-control-remap val var))
+
 (define-preferences
   ("header" "on" notify-header)
-  ("main icon bar" "on" notify-icon-bar)
-  ("context dependent icons" "on" notify-icon-bar)
+  ("main icon bar" (main-icon-bar-default) notify-icon-bar)
+  ("mode dependent icons" "on" notify-icon-bar)
+  ("focus dependent icons" "on" notify-icon-bar)
   ("user provided icons" "off" notify-icon-bar)
   ("status bar" "on" notify-status-bar)
-  ("shrinking factor" "5" notify-shrinking-factor))
+  ("shrinking factor" "5" notify-shrinking-factor)
+  ("ir-up" "home" notify-remote-control)
+  ("ir-down" "end" notify-remote-control)
+  ("ir-left" "pageup" notify-remote-control)
+  ("ir-right" "pagedown" notify-remote-control)
+  ("ir-center" "S-return" notify-remote-control)
+  ("ir-play" "F5" notify-remote-control)
+  ("ir-pause" "escape" notify-remote-control)
+  ("ir-menu" "." notify-remote-control))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Changing the view properties
@@ -73,6 +89,11 @@
   (:check-mark "v" full-screen-edit?)
   (full-screen-mode (not (full-screen-edit?)) (not (full-screen-edit?))))
 
+(tm-define (toggle-remote-control-mode)
+  (:synopsis "Toggle remote keyboard control mode.")
+  (:check-mark "v" remote-control-mode?)
+  (set! remote-control-flag? (not remote-control-flag?)))
+
 (define (test-shrinking-factor? n)
   (= (get-shrinking-factor) n))
 
