@@ -427,33 +427,21 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind text-format-icons
-  (if (style-has? "std-markup-dtd")
+  (if (not (style-has? "std-markup-dtd"))
+      (=> (balloon (icon "tm_parstyle.xpm") "Set paragraph mode")
+          ((balloon (icon "tm_left.xpm") "Align text to the left")
+           (make-line-with "par-mode" "left"))
+          ((balloon (icon "tm_center.xpm") "Center text")
+           (make-line-with "par-mode" "center"))
+          ((balloon (icon "tm_right.xpm") "Align text to the right")
+           (make-line-with "par-mode" "right"))
+          ((balloon (icon "tm_justify.xpm") "Justify text")
+           (make-line-with "par-mode" "justify")))
+      (=> (balloon (icon "tm_parindent.xpm") "Set paragraph margins")
+          ("Left margin" (make-interactive-line-with "par-left"))
+          ("Right margin" (make-interactive-line-with "par-right"))
+          ("First indentation" (make-interactive-line-with "par-first")))
       /)
-  (=> (balloon (icon "tm_parstyle.xpm") "Set paragraph mode")
-      ((balloon (icon "tm_left.xpm") "Align text to the left")
-       (make-line-with "par-mode" "left"))
-      ((balloon (icon "tm_center.xpm") "Center text")
-       (make-line-with "par-mode" "center"))
-      ((balloon (icon "tm_right.xpm") "Align text to the right")
-       (make-line-with "par-mode" "right"))
-      ((balloon (icon "tm_justify.xpm") "Justify text")
-       (make-line-with "par-mode" "justify")))
-  (=> (balloon (icon "tm_parindent.xpm") "Set paragraph margins")
-      ("Left margin" (make-interactive-line-with "par-left"))
-      ("Right margin" (make-interactive-line-with "par-right"))
-      ("First indentation" (make-interactive-line-with "par-first")))
-  (if (and (style-has? "env-float-dtd") (detailed-menus?))
-      (if (not (inside? 'float))
-	  (=> (balloon (icon "tm_pageins.xpm") "Make a page insertion")
-	      (link insert-page-insertion-menu)))
-      (if (inside? 'float)
-	  (=> (balloon (icon "tm_floatpos.xpm") "Position floating object")
-	      (link position-float-menu))))
-  ;;((balloon (icon "tm_footnote.xpm") "Insert a footnote") ())
-  ;;((balloon (icon "tm_margin.xpm") "Insert a marginal note") ())
-  ;;((balloon (icon "tm_floating.xpm") "Insert a floating object") ())
-  ;;((balloon (icon "tm_multicol.xpm") "Start multicolumn context") ())
-  /
   (if (and (style-has? "std-markup-dtd") (not (in-source?)))
       ;;((balloon
       ;;(text (roman rm bold right 12 600) "S")
