@@ -14,7 +14,8 @@
 (texmacs-module (text text-menu)
   (:use (text format-text-edit)
         (text std-text-edit)
-        (text tm-structure)))
+        (text tm-structure)
+        (generic document-menu)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Document headers
@@ -82,11 +83,13 @@
       ("Axiom" (make 'axiom))
       ("Definition" (make 'definition))
       ("Notation" (make 'notation))
+      ("Convention" (make 'convention))
       ---
       ("Remark" (make 'remark))
       ("Note" (make 'note))
       ("Example" (make 'example))
       ("Warning" (make 'warning))
+      ("Acknowledgments" (make 'acknowledgments*))
       ---)
   ("Exercise" (make 'exercise))
   ("Problem" (make 'problem))
@@ -103,6 +106,14 @@
   ("Underlined" (make 'underlined))
   ("Lines around" (make 'bothlined))
   ("Framed" (make 'framed))
+  ---
+  ("Indent" (make 'indent))
+  ("Jump in" (make 'jump-in))
+  ("Compact" (make 'compact))
+  ---
+  ("Centered" (make 'center))
+  ("Left aligned" (make 'left-aligned))
+  ("Right aligned" (make 'right-aligned))
   (if (style-has? "ornaments-dtd")
       ---
       ("Manila paper" (make 'manila-paper))
@@ -323,11 +334,25 @@
   (if (style-has? "std-list-dtd")
       (=> (balloon (icon "tm_list.xpm") "Insert a list")
 	  (link list-menu)))
+  (if (and (style-has? "env-float-dtd") (detailed-menus?))
+      ;;((balloon (icon "tm_footnote.xpm") "Insert a footnote") ())
+      ;;((balloon (icon "tm_margin.xpm") "Insert a marginal note") ())
+      ;;((balloon (icon "tm_floating.xpm") "Insert a floating object") ())
+      ;;((balloon (icon "tm_multicol.xpm") "Start multicolumn context") ())
+      (if (not (inside? 'float))
+	  (=> (balloon (icon "tm_pageins.xpm") "Make a page insertion")
+	      (link insert-page-insertion-menu)))
+      (if (inside? 'float)
+	  (=> (balloon (icon "tm_floatpos.xpm") "Position floating object")
+	      (link position-float-menu))))
   (if (style-has? "section-base-dtd")
       (=> (balloon (icon "tm_index.xpm")
 		   "Insert automatically generated content")
 	  (link automatic-menu)))
+  (if (style-has? "std-markup-dtd")
+      /)
   (link text-format-icons)
+  (if (in-manual?) (link tmdoc-icons))
   (link texmacs-insert-icons))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -396,7 +421,7 @@
 (tm-menu (focus-ancestor-icons t)
   (:require (doc-title-context? t))
   (minibar (dynamic (focus-title-icons)))
-  (glue #f #f 5 0))
+  //)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Focus menus for entering authors
@@ -433,9 +458,9 @@
 (tm-menu (focus-ancestor-icons t)
   (:require (doc-author-context? t))
   (minibar (dynamic (focus-title-icons)))
-  (glue #f #f 5 0)
+  //
   (minibar (dynamic (focus-author-icons)))
-  (glue #f #f 5 0))
+  //)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Focus menus for sections
@@ -465,7 +490,7 @@
 
 (tm-menu (focus-extra-icons t)
   (:require (section-context? t))
-  (glue #f #f 5 0)
+  //
   (mini #t
     (=> (eval (tm/section-get-title-string t))
 	(link focus-section-menu))))
