@@ -83,13 +83,33 @@
 (define (go-to-next-inside-sub fun l)
   (do ((p (cursor-path) (cursor-path))
        (q (begin (fun) (cursor-path)) (begin (fun) (cursor-path))))
-      ((or (== p q) (innermost-pattern q l)) (noop))))
+      ((or (== p q) (innermost-pattern q l))
+       ;;(display* "  End " q ", " (path->tree (cDr q)) "\n")
+       q)
+    ;;(display* "  Next " q ", " (path->tree (cDr q)) "\n")
+    (noop)))
 
 (tm-define (go-to-next-inside fun . l)
   (with p (cursor-path)
+    ;;(display* "  First " p ", " (path->tree (cDr p)) ", " (procedure-source fun) ", " l "\n")
     (go-to-next-inside-sub fun l)
     (if (not (innermost-pattern (cursor-path) l)) (go-to p))))
 
+(define (go-to-next-such-that fun pred?)
+  (do ((p (cursor-path) (cursor-path))
+       (q (begin (fun) (cursor-path)) (begin (fun) (cursor-path))))
+      ((or (== p q) (pred? (path->tree (cDr q))))
+       ;;(display* "End " q ", " (path->tree (cDr q)) "\n")
+       q)
+    ;;(display* "Next " q ", " (path->tree (cDr q)) "\n")
+    (noop)))
+
+(tm-define (go-to-next-such-that fun pred?)
+  (with p (cursor-path)
+    ;;(display* "First " p ", " (path->tree (cDr p)) ", " (procedure-source fun) ", " pred? "\n")
+    (go-to-next-such-that fun pred?)
+    (if (not (pred? (path->tree (cDr (cursor-path))))) (go-to p))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Routines for cursor movement
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -190,29 +210,23 @@
 ;; Changing buffers
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (menu-flatten x)
-  (cond ((string? x) x)
-        ((pair? x) (apply string-append (map menu-flatten (cdr x))))
-        (else "")))
+(define (list-abbrs)
+  (map buffer-get-short-name (buffer-sorted-list)))
 
-(define (list-buffers)
-  (with bm (list-filter (get-buffer-menu) pair?)
-    (with fun (lambda (p) (cons (menu-flatten (car p)) (cdr p)))
-      (map fun bm))))
-
-(define (menu-name p)
-  (menu-flatten (car p)))
+(define (abbr->buffer abbr)
+  (assoc-ref (map (lambda (x) (cons (buffer-get-short-name x) x))
+                  (buffer-sorted-list))
+             abbr))
 
 (tm-define (go-to-buffer name)
   (:argument  name "Switch to buffer")
-  (:proposals name (map car (list-buffers)))
-  (let* ((m  (list-buffers))
-	 (l1 (assoc name m))
-	 (l2 (assoc (string-append name " *") m)))
-    (cond (l1 ((cadr l1)))
-	  (l2 ((cadr l2)))
-	  (else (set-message `(concat "Error: no buffer " ,name)
-			     "switch to buffer")))))
+  (:proposals name (list-abbrs))
+  (cond ((abbr->buffer name)
+         (switch-to-buffer (abbr->buffer name)))
+        ((in? (string->url name) (buffer-list))
+         (switch-to-buffer (string->url name)))
+        (else (set-message `(concat "Error: no buffer " (verbatim ,name))
+                           "switch to buffer"))))
 
 (tm-define (with-active-buffer-sub name cmd)
   (let ((old (get-name-buffer)))
