@@ -16,6 +16,8 @@
 #include "url.hpp"
 
 tree upgrade_tex (tree t);
+bool textm_class_flag= false;
+//bool textm_class_flag= true;
 static bool textm_appendices= false;
 static bool textm_unicode   = false;
 static bool textm_natbib    = false;
@@ -65,9 +67,14 @@ filter_preamble (tree t) {
   for (i=0; i<n; i++) {
     tree u= t[i];
     if (is_tuple (u, "\\title") ||
-	     is_tuple (u, "\\author") ||
-	     is_tuple (u, "\\address"))
+        is_tuple (u, "\\author") ||
+        is_tuple (u, "\\address"))
       title_info << u;
+    else if (is_tuple (u, "\\affiliation")) {
+      tree v= copy (u);
+      v[0]= "\\address";
+      title_info << v;
+    }
     else if (is_tuple (u, "\\thanks")) {
       tree v= copy (u);
       v[0]= "\\title-thanks";
@@ -199,6 +206,7 @@ latex_symbol_to_tree (string s) {
       if (s == "\\dots") return "<ldots>";
       if (s == "\\infin") return "<infty>";
       if (s == "\\rang") return "<rangle>";
+      if (s == "\\today") return compound ("date", "");
       if (s == "\\tableofcontents")
 	return compound ("table-of-contents", "toc", tree (DOCUMENT, ""));
       if (s == "\\bgroup") return "";
@@ -219,6 +227,13 @@ latex_symbol_to_tree (string s) {
       if (s == "\\gets") return "<leftarrow>";
     }
 
+    if (latex_type (s) == "symbol") {
+      if (s == "\\lnot") return "<neg>";
+      if (s == "\\land") return "<wedge>";
+      if (s == "\\lor") return "<vee>";
+      return "<" * s(1,N(s)) * ">";
+    }
+
     if (latex_type (s) == "texmacs") {
       if (s == "\\tmdummy")  return "";
     }
@@ -295,7 +310,6 @@ latex_symbol_to_tree (string s) {
     if (latex_type (s) == "control") return s(1,N(s));
     if ((s == "\\ldots") && (command_type ("!mode") != "math")) return "...";
     if (s == "\\bignone") return tree (BIG, ".");
-    if (latex_type (s) == "symbol")  return "<" * s(1,N(s)) * ">";
     if (latex_type (s) == "big-symbol") {
       if (s(0,4)=="\\big") return tree (BIG, s(4,N(s)));
       else return tree (BIG, s(1,N(s)));
@@ -730,7 +744,35 @@ latex_command_to_tree (tree t) {
     return compound ("reset-counter", v2e (t[1]));
   if (is_tuple (t, "\\addtocounter", 2)) // FIXME: only inc works
     return compound ("inc-counter", v2e (t[1]));
-  if (is_tuple (t, "\\setlength")) return "";
+  if (is_tuple (t, "\\setlength", 2)) {
+    if (!textm_class_flag) return "";
+    else {
+      string len= (is_atomic (t[1])? t[1]->label: v2e (t[1]));
+      tree val= l2e (t[2]);
+      if (len == "\\oddsidemargin") len= "tex-odd-side-margin";
+      if (len == "\\evensidemargin") len= "tex-even-side-margin";
+      if (len == "\\textwidth") len= "tex-text-width";
+      if (len == "\\topmargin") len= "tex-top-margin";
+      if (len == "\\headheight") len= "tex-head-height";
+      if (len == "\\headsep") len= "tex-head-sep";
+      if (len == "\\topskip") len= "tex-top-skip";
+      if (len == "\\textheight") len= "tex-text-height";
+      if (len == "\\footskip") len= "tex-foot-skip";
+      if (len == "\\footnotesep") len= "tex-footnote-sep";
+      if (len == "\\columnsep") len= "tex-column-sep";
+      if (len == "\\marginparwidth") len= "tex-margin-par-width";
+      if (len == "\\par-indent") len= "par-first";
+      if (len == "\\jot") len= "tex-jot";
+      if (len == "\\mathindent") len= "tex-math-indent";
+      if (len == "\\abovedisplayskip") len= "tex-above-display-skip";
+      if (len == "\\belowdisplayskip") len= "tex-below-display-skip";
+      if (len == "\\abovedisplayshortskip")
+	len= "tex-above-display-short-skip";
+      if (len == "\\belowdisplayshortskip")
+	len= "tex-below-display-short-skip";
+      return tree (ASSIGN, len, tree (MACRO, val));
+    }
+  }
   if (is_tuple (t, "\\addtolength")) return "";
   if (is_tuple (t, "\\enlargethispage")) return "";
   if (is_tuple (t, "\\mathop", 1)) return l2e (t[1]);
@@ -889,6 +931,49 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\href", 2))
     return tree (APPLY, "hlink", l2e (t[2]), t2e (t[1]));
 
+  if (is_tuple (t, "\\xminus", 1))
+    return tree (LONG_ARROW, "<rubber-minus>", l2e (t[1]));
+  if (is_tuple (t, "\\xleftarrow", 1))
+    return tree (LONG_ARROW, "<rubber-leftarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xrightarrow", 1))
+    return tree (LONG_ARROW, "<rubber-rightarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xleftrightarrow", 1))
+    return tree (LONG_ARROW, "<rubber-leftrightarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xmapsto", 1))
+    return tree (LONG_ARROW, "<rubber-mapsto>", l2e (t[1]));
+  if (is_tuple (t, "\\xmapsfrom", 1))
+    return tree (LONG_ARROW, "<rubber-mapsfrom>", l2e (t[1]));
+  if (is_tuple (t, "\\xequal", 1))
+    return tree (LONG_ARROW, "<rubber-equal>", l2e (t[1]));
+  if (is_tuple (t, "\\xLeftarrow", 1))
+    return tree (LONG_ARROW, "<rubber-Leftarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xRightarrow", 1))
+    return tree (LONG_ARROW, "<rubber-Rightarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xLeftrightarrow", 1))
+    return tree (LONG_ARROW, "<rubber-Leftrightarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xminus*", 2))
+    return tree (LONG_ARROW, "<rubber-minus>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xleftarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-leftarrow>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xrightarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-rightarrow>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xleftrightarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-leftrightarrow>",
+                 l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xmapsto*", 2))
+    return tree (LONG_ARROW, "<rubber-mapsto>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xmapsfrom*", 2))
+    return tree (LONG_ARROW, "<rubber-mapsfrom>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xequal*", 2))
+    return tree (LONG_ARROW, "<rubber-equal>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xLeftarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-Leftarrow>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xRightarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-Rightarrow>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xLeftrightarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-Leftrightarrow>",
+                 l2e (t[1]), l2e (t[2]));
+
   // Start TeXmacs specific markup
   if (is_tuple (t, "\\tmmathbf", 1))
     return tree (CONCAT,
@@ -1675,18 +1760,25 @@ finalize_misc (tree t) {
   if (is_atomic (t)) return t;
   else if (is_compound (t, "verbatim", 1) &&
            is_atomic (t[0]) && is_hyper_link (t[0]->label)) {
-    return compound ("href", t[0]);
+    return compound ("href", finalize_misc (t[0]));
   }
   else if (is_func (t, WITH, 3) && t[0] == FONT_FAMILY && t[1] == "tt" &&
            is_atomic (t[2]) && is_hyper_link (t[2]->label)) {
-    return compound ("href", t[2]);
+    return compound ("href", finalize_misc (t[2]));
   }
   else if (is_compound (t, "flushleft", 1) ||
            is_compound (t, "leftaligned", 1))
-    return compound ("left-aligned", t[0]);
+    return compound ("left-aligned", finalize_misc (t[0]));
   else if (is_compound (t, "flushright", 1) ||
            is_compound (t, "rightaligned", 1))
-    return compound ("right-aligned", t[0]);
+    return compound ("right-aligned", finalize_misc (t[0]));
+  else if (is_compound (t, "acknowledgments", 1))
+    return compound ("acknowledgments*", finalize_misc (t[0]));
+  else if (is_compound (t, "text", 1) &&
+           is_func (t[0], WITH, 3) &&
+           t[0][0] == "font-family" &&
+           t[0][1] == "rm")
+    return compound ("math-up", finalize_misc (t[0][2]));
   else {
     int i, n= N(t);
     tree r (t, n);
@@ -1727,7 +1819,7 @@ latex_to_tree (tree t1) {
   tree t2= is_document? filter_preamble (t1): t1;
   //cout << "\n\nt2= " << t2 << "\n\n";
   tree t3= parsed_latex_to_tree (t2);
-  // cout << "\n\nt3= " << t3 << "\n\n";
+  //cout << "\n\nt3= " << t3 << "\n\n";
   tree t4= finalize_document (t3);
   // cout << "\n\nt4= " << t4 << "\n\n";
   tree t5= is_document? finalize_preamble (t4, style): t4;
