@@ -4,10 +4,9 @@
 * DESCRIPTION: conversion of xml and html strings into logical html trees
 * COPYRIGHT  : (C) 2000  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "convert.hpp"
@@ -103,7 +102,7 @@ static hashmap<string,string> html_entity ("");
 void load_html_entities (hashmap<string, string> table, string fname) {
   string s;
   if (DEBUG_VERBOSE) cout << "TeXmacs] Loading " << fname << "\n";
-  if (load_string (url ("$TEXMACS_PATH/langs/encoding", fname), s)) return;
+  if (load_string (url ("$TEXMACS_PATH/langs/encoding", fname), s, false)) return;
   tree t= block_to_scheme_tree (s);
   if (!is_tuple (t)) return;
 
@@ -308,7 +307,7 @@ xml_html_parser::expand_entities (string s) {
       int start= i++;
       if (i<n && s[i] == '#') {
 	i++;
-	if (i<n && s[i] == 'x' || s[i] == 'X') {
+	if (i<n && (s[i] == 'x' || s[i] == 'X')) {
 	  i++;
 	  while (i<n && is_hex_digit (s[i])) i++;
 	}
@@ -692,6 +691,7 @@ xml_html_parser::finalize_space (tree t) {
       }
     if (!is_tuple (t[n-1], "attr"))
       last= n-1;
+    (void) first; (void) last;
     for (i=2; i<n; i++) {
       if (is_atomic (t[i])) {
 	if (finalize_preserve_space (t[1]->label)) r << t[i];
