@@ -181,35 +181,55 @@ extern string CELL_BLOCK;
 extern string CELL_ROW_NR;
 extern string CELL_COL_NR;
 
-extern string POINT_STYLE;
-extern string LINE_WIDTH;
-extern string DASH_STYLE;
-extern string DASH_STYLE_UNIT;
-extern string LINE_ARROWS;
-extern string LINE_CAPS;
-extern string FILL_COLOR;
-extern string FILL_STYLE;
-extern string TEXTAT_HALIGN;
-extern string TEXTAT_VALIGN;
-
 extern string GR_GEOMETRY;
 extern string GR_FRAME;
 extern string GR_MODE;
-extern string GR_OPACITY;
-extern string GR_COLOR;
-extern string GR_POINT_STYLE;
-extern string GR_LINE_WIDTH;
-extern string GR_DASH_STYLE;
-extern string GR_DASH_STYLE_UNIT;
-extern string GR_LINE_ARROWS;
-extern string GR_FILL_COLOR;
-extern string GR_TEXTAT_HALIGN;
-extern string GR_TEXTAT_VALIGN;
+extern string GR_AUTO_CROP;
+extern string GR_CROP_PADDING;
 extern string GR_GRID;
 extern string GR_GRID_ASPECT;
 extern string GR_EDIT_GRID;
 extern string GR_EDIT_GRID_ASPECT;
 
+extern string GR_MAGNIFY;
+extern string GR_OPACITY;
+extern string GR_COLOR;
+extern string GR_POINT_STYLE;
+extern string GR_LINE_WIDTH;
+extern string GR_LINE_JOIN;
+extern string GR_LINE_CAPS;
+extern string GR_LINE_EFFECTS;
+extern string GR_DASH_STYLE;
+extern string GR_DASH_STYLE_UNIT;
+extern string GR_ARROW_BEGIN;
+extern string GR_ARROW_END;
+extern string GR_ARROW_LENGTH;
+extern string GR_ARROW_HEIGHT;
+extern string GR_FILL_COLOR;
+extern string GR_FILL_STYLE;
+extern string GR_TEXT_AT_HALIGN;
+extern string GR_TEXT_AT_VALIGN;
+extern string GR_TEXT_AT_MARGIN;
+
+extern string GID;
+extern string MAGNIFY;
+extern string POINT_STYLE;
+extern string LINE_WIDTH;
+extern string LINE_JOIN;
+extern string LINE_CAPS;
+extern string LINE_EFFECTS;
+extern string DASH_STYLE;
+extern string DASH_STYLE_UNIT;
+extern string ARROW_BEGIN;
+extern string ARROW_END;
+extern string ARROW_LENGTH;
+extern string ARROW_HEIGHT;
+extern string FILL_COLOR;
+extern string FILL_STYLE;
+extern string TEXT_AT_HALIGN;
+extern string TEXT_AT_VALIGN;
+extern string TEXT_AT_MARGIN;
+
 extern string SRC_STYLE;
 extern string SRC_SPECIAL;
 extern string SRC_COMPACT;
