@@ -28,7 +28,7 @@ class tree_position_rep: public observer_rep {
 public:
   tree_position_rep (tree ref, int index2): ptr (ref.rep), index (index2) {}
   int get_type () { return OBSERVER_POSITION; }
-  ostream& print (ostream& out) { return out << " " << index; }
+  tm_ostream& print (tm_ostream& out) { return out << " " << index; }
 
   void notify_assign      (tree& ref, tree t);
   void notify_insert      (tree& ref, int pos, int nr);
@@ -139,8 +139,10 @@ tree_position_rep::notify_var_join (tree& ref, tree t, int offset) {
   if (is_atomic (ref))
     reattach_at (this, t, index + offset);
   else {
-    if (index == 0) reattach_right (this, t[offset-1]);
-    else reattach_right (this, t);
+    if (index != 0) reattach_right (this, t);
+    else if (offset > 0) reattach_right (this, t[offset-1]);
+    else if (N(t) != 0) reattach_left (this, t[0]);
+    else reattach_left (this, t);
   }
   // cout << "position -> " << obtain_position (observer (this)) << "\n";
 }
@@ -169,6 +171,7 @@ tree_position_rep::notify_remove_node (tree& ref, int pos) {
 
 void
 tree_position_rep::notify_detach (tree& ref, tree closest, bool right) {
+  (void) ref;
   // cout << "Notify detach " << ref << ", " << closest <<", "<< right << "\n";
   if (right) reattach_right (this, closest);
   else reattach_left (this, closest);
