@@ -41,13 +41,12 @@
 ******************************************************************************/
 
 editor_rep::editor_rep ():
-  simple_widget_rep (),
-  drd (buf->abbr, std_drd), et (the_et), rp (buf->rp) {}
+  simple_widget_rep (), cvw (NULL), mvw (NULL),
+  drd (buf->buf->abbr, std_drd), et (the_et), rp (buf->rp) {}
 
 editor_rep::editor_rep (server_rep* sv2, tm_buffer buf2):
-  simple_widget_rep (),
-  sv (sv2), buf (buf2), drd (buf->abbr, std_drd),
-  et (the_et), rp (buf2->rp) {}
+  simple_widget_rep (), sv (sv2), cvw (NULL), mvw (NULL), buf (buf2),
+  drd (buf->buf->abbr, std_drd), et (the_et), rp (buf2->rp) {}
 
 edit_main_rep::edit_main_rep (server_rep* sv, tm_buffer buf):
   editor_rep (sv, buf), props (UNKNOWN), ed_obs (edit_observer (this))
@@ -140,7 +139,7 @@ edit_main_rep::tex_buffer () {
 
 url
 edit_main_rep::get_name () {
-  return buf->name;
+  return buf->buf->name;
 }
 
 void
@@ -302,9 +301,12 @@ edit_main_rep::print_snippet (url name, tree t) {
   if (b->x4 - b->x3 >= 5*PIXEL && b->y4 - b->y3 >= 5*PIXEL) {
     if (ps) make_eps (name, b, dpi);
     else {
-      url temp= url_temp ("eps");
+      url temp= url_temp (".eps");
       make_eps (temp, b, dpi);
+      ::remove (name);
       system ("convert", temp, name);
+      if (!exists (name))
+        cout << "TeXmacs] warning, failed to create image " << name << "\n";
       ::remove (temp);
     }
   }
@@ -320,6 +322,7 @@ edit_main_rep::print_snippet (url name, tree t) {
 void
 edit_main_rep::footer_eval (string s) {
   // s= unslash (s); // FIXME: dirty fix; should not be necessary
+  s= tm_decode (s);
   string r= object_to_string (eval (s));
   set_message (verbatim (r), "evaluate expression");
 }
