@@ -16,6 +16,7 @@
 #include "analyze.hpp"
 #include "timer.hpp"
 #include "Bridge/impl_typesetter.hpp"
+#include "new_style.hpp"
 #ifdef EXPERIMENTAL
 #include "../../Style/Environment/std_environment.hpp"
 #endif // EXPERIMENTAL
@@ -38,12 +39,30 @@ edit_typeset_rep::edit_typeset_rep ():
   the_style (TUPLE),
   cur (hashmap<string,tree> (UNINIT)),
   pre (UNINIT), init (UNINIT), fin (UNINIT),
-  env (drd, is_aux (buf->name)? buf->extra: buf->name,
-       buf->ref, (buf->prj==NULL? buf->ref: buf->prj->ref),
-       buf->aux, (buf->prj==NULL? buf->aux: buf->prj->aux)),
+  env (drd, is_aux (buf->buf->name)? buf->buf->extra: buf->buf->name,
+       buf->data->ref, (buf->prj==NULL? buf->data->ref: buf->prj->data->ref),
+       buf->data->aux, (buf->prj==NULL? buf->data->aux: buf->prj->data->aux)),
   ttt (new_typesetter (env, subtree (et, rp), reverse (rp))) {}
 edit_typeset_rep::~edit_typeset_rep () { delete_typesetter (ttt); }
 
+void
+edit_typeset_rep::set_data (new_data data) {
+  set_style (data->style);
+  set_init  (data->init);
+  set_fin   (data->fin);
+  notify_page_change ();
+  add_init (data->init);
+  notify_change (THE_DECORATIONS);
+  typeset_invalidate_env ();
+}
+
+void
+edit_typeset_rep::get_data (new_data& data) {
+  data->style= get_style ();
+  data->init = get_init ();
+  data->fin  = get_fin ();
+}
+
 typesetter edit_typeset_rep::get_typesetter () { return ttt; }
 tree edit_typeset_rep::get_style () { return the_style; }
 void edit_typeset_rep::set_style (tree t) { the_style= copy (t); }
@@ -71,8 +90,8 @@ edit_typeset_rep::set_base_name (url name) {
 
 void
 edit_typeset_rep::clear_local_info () {
-  buf->ref= hashmap<string,tree> ();
-  buf->aux= hashmap<string,tree> ();
+  buf->data->ref= hashmap<string,tree> ();
+  buf->data->aux= hashmap<string,tree> ();
 }
 
 /******************************************************************************
@@ -115,22 +134,24 @@ use_modules (tree t) {
 
 void
 edit_typeset_rep::typeset_style_use_cache (tree style) {
+  //cout << "Typesetting style using cache " << style << LF;
   bool ok;
   hashmap<string,tree> H;
   tree t;
-  SERVER (style_get_cache (style, H, t, ok));
+  style_get_cache (style, H, t, ok);
   if (ok) {
     env->patch_env (H);
     ok= drd->set_locals (t);
     drd->set_environment (H);
   }
   if (!ok) {
+    //cout << "Typeset without cache " << style << LF;
     if (!is_tuple (style)) FAILED ("tuple expected as style");
     tree t (USE_PACKAGE, A (style));
     env->exec (t);
     env->read_env (H);
     drd->heuristic_init (H);
-    SERVER (style_set_cache (style, H, drd->get_locals ()));
+    style_set_cache (style, H, drd->get_locals ());
   }
   use_modules (env->read (THE_MODULES));
 }
@@ -147,7 +168,7 @@ edit_typeset_rep::typeset_preamble () {
 
 void
 edit_typeset_rep::typeset_prepare () {
-  env->read_only= buf->read_only;
+  env->read_only= buf->buf->read_only;
   env->write_default_env ();
   env->patch_env (pre);
   env->style_init_env ();
@@ -214,6 +235,10 @@ edit_typeset_rep::typeset_exec_until (path p) {
     typeset_invalidate_env ();
   typeset_prepare ();
   if (enable_fastenv) {
+    if (!(rp < p)) {
+      cerr << "TeXmacs] erroneous path " << p << "\n";
+      FAILED ("invalid typesetting path");
+    }
     tree t= subtree (et, rp);
     path q= path_up (p / rp);
     while (!is_nil (q)) {
@@ -372,7 +397,7 @@ edit_typeset_rep::exec (tree t, hashmap<string,tree> H, bool expand_refs) {
   env->write_env (H);
   t= env->exec (t);
   if (expand_refs)
-    t= expand_references (t, buf->ref);
+    t= expand_references (t, buf->data->ref);
   t= simplify_execed (t);
   t= simplify_correct (t);
   env->write_env (H2);
@@ -413,7 +438,20 @@ edit_typeset_rep::exec_html (tree t, path p) {
   tree patch= as_tree (eval ("(stree->tree (tmhtml-env-patch))"));
   hashmap<string,tree> P (UNINIT, patch);
   H->join (P);
-  return exec (t, H);
+  tree w (WITH);
+  if (H->contains ("html-title"))
+    w << string ("html-title") << H["html-title"];
+  if (H->contains ("html-css"))
+    w << string ("html-css") << H["html-css"];
+  if (H->contains ("html-head-javascript"))
+    w << string ("html-head-javascript") << H["html-head-javascript"];
+  if (H->contains ("html-head-javascript-src"))
+    w << string ("html-head-javascript-src") << H["html-head-javascript-src"];
+  if (N(w) == 0) return exec (t, H);
+  else {
+    w << t;
+    return exec (w, H);
+  }
   //tree r= exec (t, H);
   //cout << "In: " << t << "\n";
   //cout << "Out: " << r << "\n";
@@ -518,6 +556,7 @@ void
 edit_typeset_rep::init_env (string var, tree by) {
   if (init (var) == by) return;
   init (var)= by;
+  require_save();
   notify_change (THE_ENVIRONMENT);
 }
 
