@@ -42,6 +42,8 @@ public:
   edit_typeset_rep ();
   ~edit_typeset_rep ();
   void clear_local_info ();
+  void set_data (new_data data);
+  void get_data (new_data& data);
 
   SI       as_length (string l);
   string   add_lengths (string l1, string l2);
@@ -95,8 +97,6 @@ public:
   void     typeset (SI& x1, SI& y1, SI& x2, SI& y2);
 
   friend class tm_window_rep;
-  friend class tm_data_rep;
-  friend class tm_project_rep;
   friend class tm_server_rep;
 };
 
