@@ -47,10 +47,8 @@ protected:
   int           input_mode;    // INPUT_NORMAL, INPUT_SEARCH, INPUT_REPLACE
 
 protected:
-  time_t        last_click;    // last click on left mouse button
   SI            last_x, last_y;
-  bool          start_drag, start_right_drag;
-  bool          dragging, right_dragging;
+  time_t        last_t;
   SI            start_x, start_y;
   SI            end_x, end_y;
   bool          made_selection;
@@ -121,6 +119,9 @@ public:
   void full_screen_mode (bool flag);
   void before_menu_action ();
   void after_menu_action ();
+  cursor search_cursor (path p);
+  selection search_selection (path start, path end);
+  rectangle get_window_extents ();
 
   /* keyboard handling */
   int  get_input_mode ();
@@ -149,7 +150,7 @@ public:
   void mouse_click (SI x, SI y);
   bool mouse_extra_click (SI x, SI y);
   void mouse_drag (SI x, SI y);
-  void mouse_select (SI x, SI y, int mods);
+  void mouse_select (SI x, SI y, int mods, bool drag);
   void mouse_paste (SI x, SI y);
   void mouse_adjust (SI x, SI y);
   void mouse_scroll (SI x, SI y, bool up);
@@ -185,7 +186,6 @@ public:
 
   friend class interactive_command_rep;
   friend class tm_window_rep;
-  friend class tm_project_rep;
 };
 
 #endif // defined EDIT_INTERFACE_H
