@@ -13,6 +13,8 @@
 #include "tree_traverse.hpp"
 #include "analyze.hpp"
 
+bool is_empty_cell (tree t);
+
 /******************************************************************************
 * Getting the point where to delete
 ******************************************************************************/
@@ -219,6 +221,9 @@ edit_text_rep::remove_text (bool forward) {
     case BIG_AROUND:
       back_in_around (u, p, forward);
       return;
+    case LONG_ARROW:
+      back_in_long_arrow (u, p, forward);
+      return;
     case WIDE:
     case VAR_WIDE:
       back_in_wide (u, p, forward);
@@ -240,7 +245,16 @@ edit_text_rep::remove_text (bool forward) {
       back_in_with (u, p, forward);
       return;
     default:
-      back_in_general (u, p, forward);
+      if (is_graphical_text (u))
+        back_in_text_at (u, p, forward);
+      else if (is_compound (u, "cell-inert") ||
+               is_compound (u, "cell-input") ||
+               is_compound (u, "cell-output")) {
+        tree st= subtree (et, path_up (p, 2));
+        back_in_table (u, p, forward);
+      }
+      else
+        back_in_general (u, p, forward);
       break;
     }
   }
