@@ -72,8 +72,9 @@ edit_replace_rep::spell_end () {
   else if (nr_replaced == 1)
     set_message ("one spelling error has been corrected", "correct text");
   else if (nr_replaced > 1)
-    set_message (as_string (nr_replaced) *
-		 "#spelling errors have been corrected", "correct text");
+    set_message (concat (as_string (nr_replaced),
+			 " spelling errors have been corrected"),
+		 "correct text");
   else set_message ("spell checking complete", "correct text");
   beep ();
   set_input_normal ();
@@ -158,6 +159,8 @@ edit_replace_rep::spell_replace (string by) {
 
 bool
 edit_replace_rep::spell_keypress (string s) {
+  set_message ("", "");
+  if (s == "space") s= " ";
   if ((s == "C-c") || (s == "C-g") || (s == "escape"))
     spell_end ();
   else if ((s == "a") || (s == "A")) {
