@@ -22,12 +22,12 @@
 #endif
 #endif
 #include "server.hpp"
-#include "scheme.hpp"
 #include "drd_info.hpp"
 #ifdef EXPERIMENTAL
 #include "../Style/Environment/environment.hpp"
 #include "../Style/Memorizer/memorizer.hpp"
 #endif
+#include "new_data.hpp"
 #define TEXMACS_COPYRIGHT (string("(c) 1999-2006 by Joris van der Hoeven"))
 
 #define THE_CURSOR 1
@@ -41,33 +41,36 @@
 
 class tm_buffer_rep;
 typedef tm_buffer_rep* tm_buffer;
+class tm_view_rep;
 class server_rep;
 class modification;
 extern bool enable_fastenv;
 
 class editor_rep: public simple_widget_rep {
 public:
-  server_rep* sv;   // the underlying texmacs server
+  server_rep*  sv;   // the underlying texmacs server
+  widget_rep*  cvw;  // non reference counted canvas widget
+  tm_view_rep* mvw;  // master view
 
 protected:
-  tm_buffer   buf;  // the underlying buffer
-  drd_info    drd;  // the drd for the buffer
-  tree&       et;   // all TeXmacs trees
-  box         eb;   // box translation of tree
-  path        rp;   // path to the root of the document in et
-  path        tp;   // path of cursor in tree
+  tm_buffer    buf;  // the underlying buffer
+  drd_info     drd;  // the drd for the buffer
+  tree&        et;   // all TeXmacs trees
+  box          eb;   // box translation of tree
+  path         rp;   // path to the root of the document in et
+  path         tp;   // path of cursor in tree
 #ifdef EXPERIMENTAL
-  environment ste;  // environment for style rewriting
-  tree        cct;  // clean copy of the document tree
-  memorizer   mem;  // style converted document tree
+  environment  ste;  // environment for style rewriting
+  tree         cct;  // clean copy of the document tree
+  memorizer    mem;  // style converted document tree
 #endif
 
   /* exchanging information with the interface */
-  virtual SI      get_window_height () = 0;
-  virtual void    get_selection (path& start, path& end) = 0;
-  virtual void    set_selection (path start, path end) = 0;
-  virtual cursor& the_cursor () = 0;
-  virtual cursor& the_ghost_cursor () = 0;
+  virtual SI        get_window_height () = 0;
+  virtual void      get_selection (path& start, path& end) = 0;
+  virtual void      set_selection (path start, path end) = 0;
+  virtual cursor&   the_cursor () = 0;
+  virtual cursor&   the_ghost_cursor () = 0;
 
   /* exchanging information with the typesetter */
   virtual typesetter           get_typesetter () = 0;
@@ -96,6 +99,7 @@ protected:
   virtual void back_around (tree t, path p, bool forward) = 0;
   virtual void back_prime (tree t, path p, bool forward) = 0;
   virtual void back_in_around (tree t, path p, bool forward) = 0;
+  virtual void back_in_long_arrow (tree t, path p, bool forward) = 0;
   virtual void back_in_wide (tree t, path p, bool forward) = 0;
   virtual void back_in_tree (tree t, path p, bool forward) = 0;
   virtual void pre_remove_around (path p) = 0;
@@ -105,6 +109,7 @@ protected:
   virtual void back_general (path p, bool forward) = 0;
   virtual void back_in_with (tree t, path p, bool forward) = 0;
   virtual void back_in_general (tree t, path p, bool forward) = 0;
+  virtual void back_in_text_at (tree t, path p, bool forward) = 0;
 
   /* other protected subroutines */
   virtual tree kbd (string s) = 0;
@@ -112,6 +117,7 @@ protected:
   virtual path tree_path (path sp, SI x, SI y, SI delta) = 0;
   virtual void apply_changes () = 0;
   virtual void animate () = 0;
+  virtual path search_format () = 0;
   virtual path search_format (int& row, int& col) = 0;
   virtual void table_bound (path fp, int& i1, int& j1, int& i2, int& j2) = 0;
   virtual tree table_get_subtable (path p, int i1, int j1, int i2, int j2) = 0;
@@ -137,6 +143,9 @@ public:
   virtual void full_screen_mode (bool flag) = 0;
   virtual void before_menu_action () = 0;
   virtual void after_menu_action () = 0;
+  virtual rectangle get_window_extents () = 0;
+  virtual cursor search_cursor (path p) = 0;
+  virtual selection search_selection (path start, path end) = 0;
   virtual int  get_input_mode () = 0;
   virtual void set_input_mode (int mode) = 0;
   virtual void set_input_normal () = 0;
@@ -157,7 +166,7 @@ public:
   virtual void mouse_click (SI x, SI y) = 0;
   virtual bool mouse_extra_click (SI x, SI y) = 0;
   virtual void mouse_drag (SI x, SI y) = 0;
-  virtual void mouse_select (SI x, SI y, int mods) = 0;
+  virtual void mouse_select (SI x, SI y, int mods, bool drag) = 0;
   virtual void mouse_paste (SI x, SI y) = 0;
   virtual void mouse_adjust (SI x, SI y) = 0;
   virtual void mouse_scroll (SI x, SI y, bool up) = 0;
@@ -222,6 +231,8 @@ public:
 
   /* public routines from edit_typeset */
   virtual void     clear_local_info () = 0;
+  virtual void     set_data (new_data data) = 0;
+  virtual void     get_data (new_data& data) = 0;
   virtual SI       as_length (string l) = 0;
   virtual string   add_lengths (string l1, string l2) = 0;
   virtual string   multiply_length (double x, string l) = 0;
@@ -363,6 +374,7 @@ public:
   virtual void   table_row_decoration (bool forward) = 0;
   virtual void   table_column_decoration (bool forward) = 0;
   virtual void   table_correct_block_content () = 0;
+  virtual void   table_resize_notify () = 0;
   virtual void   set_cell_mode (string mode) = 0;
   virtual string get_cell_mode () = 0;
   virtual void   cell_set_format (string var, tree val) = 0;
@@ -477,6 +489,7 @@ public:
   virtual void search_start (bool forward= true) = 0;
   virtual void search_button_next () = 0;
   virtual bool search_keypress (string s) = 0;
+  virtual tree search_tree () = 0;
   virtual void replace_start (tree what, tree by, bool forward= true) = 0;
   virtual bool replace_keypress (string s) = 0;
   virtual void spell_start () = 0;
@@ -518,13 +531,14 @@ public:
   virtual void edit_test () = 0;
 
   friend class tm_window_rep;
-  friend class tm_data_rep;
   friend class tm_server_rep;
   friend class server_command_rep;
   friend void   edit_announce (editor_rep* ed, modification mod);
   friend void   edit_done (editor_rep* ed, modification mod);
   friend string get_editor_status_report ();
   friend void   tm_failure (const char* msg);
+  friend void   revert_buffer (url name, tree doc);
+  friend void   set_aux (string aux, url name);
 };
 
 class editor {
