@@ -43,7 +43,7 @@ bib_parse_char (string s, int& pos, int& depth, bool& special, int& math) {
           r << s[pos];
           pos++;
         }
-	else {
+        else {
           while (pos < N(s) && (is_alpha (s[pos]) || is_digit (s[pos]))) {
             r << s[pos];
             pos++;
@@ -139,7 +139,7 @@ bib_to_latex (string s) {
   while (pos < N(s)) {
     specialsav= special;
     string ch= bib_parse_char (s, pos, depth, special, math);
-//    cerr << ch << " " << as_string (keepcase) << " " << as_string (depth) << "\n";
+    //    cerr << ch << " " << as_string (keepcase) << " " << as_string (depth) << "\n";
     if (ch == "$$") r << "$";
     else if (special || math) r << ch;
     else if (bib_is_space (ch)) {
@@ -151,7 +151,7 @@ bib_to_latex (string s) {
     }
     else if (ch == "%") r << "\\%";
     else if (bib_is_char (ch, '{') && depth > 0 &&
-	     bib_get_char (s, pos) != "\\") {
+             bib_get_char (s, pos) != "\\") {
       if (keepcase == -1) {
         r << "\\keepcase{";
         keepcase= depth;
@@ -165,7 +165,7 @@ bib_to_latex (string s) {
   }
   for (int i= keepcase; i>0; i--) r << "}";
   r << "}";
-//  cerr << r << "\n";
+  //  cerr << r << "\n";
   return r;
 }
 
@@ -414,7 +414,7 @@ bib_assoc (tree entry, string key) {
       if (bib_is_field (doc[i]) && doc[i][0]->label == key)
 	return doc[i][1];
   }
-  return tree ();
+  return "";
 }
 
 /******************************************************************************
@@ -490,9 +490,9 @@ get_fvl (string sfvl) {
       f= w << " " << f;
     }
   }
-  res << "\\section{}" << bib_to_latex (f);
-  res << "\\section{}" << bib_to_latex (v);
-  res << "\\section{}" << bib_to_latex (l);
+  res << "\\nextbib{}" << bib_to_latex (f);
+  res << "\\nextbib{}" << bib_to_latex (v);
+  res << "\\nextbib{}" << bib_to_latex (l);
   return res;
 }
 
@@ -516,9 +516,9 @@ get_vl_f (string svl, string sf) {
     w << words;
     f= w << " " << f;
   }
-  res << "\\section{}" << bib_to_latex (f);
-  res << "\\section{}" << bib_to_latex (v);
-  res << "\\section{}" << bib_to_latex (l);
+  res << "\\nextbib{}" << bib_to_latex (f);
+  res << "\\nextbib{}" << bib_to_latex (v);
+  res << "\\nextbib{}" << bib_to_latex (l);
   return res;
 }
 
@@ -533,7 +533,7 @@ get_vl_j_f (string svl, string sj, string sf) {
     w << words;
     j= w << " " << j;
   }
-  res << "\\section{}" << bib_to_latex (j);
+  res << "\\nextbib{}" << bib_to_latex (j);
   return res;
 }
 
@@ -560,12 +560,12 @@ get_first_von_last (string s) {
   c= get_until_char (s, pos, "");
   if (a != "" && b == "" && c == "") {
     string res= get_fvl (a);
-    res << "\\section{}";
+    res << "\\nextbib{}";
     return res;
   }
   if (a != "" && b != "" && c == "") {
     string res= get_vl_f (a, b);
-    res << "\\section{}";
+    res << "\\nextbib{}";
     return res;
   }
   if (a != "" && b != "" && c != "") return get_vl_j_f (a, b, c);
@@ -577,7 +577,7 @@ bib_names (string s, int& nbfields) {
   string res;
   int pos= 0;
   nbfields++;
-  res << "\\section{}{\\bibnames}";
+  res << "\\nextbib{}{\\bibnames}";
   while (pos < N(s)) {
     string name;
     int deb= pos;
@@ -585,7 +585,7 @@ bib_names (string s, int& nbfields) {
     int pos2= pos;
     if (pos < N(s)) pos -= 5;
     if (pos <= N(s) && deb < pos) name= s (deb, pos);
-    res << "\\section{}{\\bibname}";
+    res << "\\nextbib{}{\\bibname}";
     res << get_first_von_last (name);
     nbfields += 5;
     pos= pos2;
@@ -803,16 +803,16 @@ bib_get_fields (tree t, string& latex) {
         tree f= t[i][2][j];
         if (bib_is_field (f)) {
           if ((f[0]->label == "author" || f[0]->label == "editor") &&
-	      is_atomic (f[1])) {
+              is_atomic (f[1])) {
             latex << bib_names (f[1]->label, nbfields);
           }
-	  else if (is_atomic (f[1]) && f[0]->label != "pages") {
+          else if (is_atomic (f[1]) && f[0]->label != "pages") {
             if (!bib_is_blank_string (f[1]->label)) {
-	      latex << "\\section{}" << bib_to_latex (f[1]->label);
+              latex << "\\nextbib{}" << bib_to_latex (f[1]->label);
               nbfields++;;
             }
             else {
-              latex << "\\section{}";
+              latex << "\\nextbib{}";
               nbfields++;
             }
           }
@@ -829,11 +829,13 @@ static void
 bib_set_fields (tree& t, array<tree> latex, int& ind) {
   for (int i= 0; i<N(t); i++) {
     if (bib_is_entry (t[i])) {
+      //cout << "Changing " << t[i] << "\n";
       for (int j= 0; j<N(t[i][2]); j++) {
         tree f= t[i][2][j];
+        //cout << "  Field " << f << "\n";
         if (bib_is_field (f)) {
           if ((f[0]->label == "author" || f[0]->label == "editor") &&
-	      is_atomic (f[1])) {
+              is_atomic (f[1])) {
             tree res= compound ("bib-names"); ind++;
             while (latex[ind] == compound ("bibname")) {
               tree name= compound ("bib-name"); ind++;
@@ -845,9 +847,9 @@ bib_set_fields (tree& t, array<tree> latex, int& ind) {
             }
             f[1]= res;
           }
-	  else if (f[0]->label == "pages" && is_atomic (f[1]))
+          else if (f[0]->label == "pages" && is_atomic (f[1]))
             f[1]= bib_field_pages (f[1]->label);
-	  else {
+          else {
             f[1]= latex[ind];
             ind++;
           }
@@ -863,7 +865,7 @@ static array<tree>
 bib_latex_array (tree latex) {
   int i= 0;
   array<tree> res;
-  while (i < N(latex) && latex[i] == compound ("section", tree ())) {
+  while (i < N(latex) && latex[i] == compound ("nextbib", "")) {
     i++;
     if (i < N(latex)) {
       if (latex[i] == compound ("bibnames")
@@ -873,7 +875,7 @@ bib_latex_array (tree latex) {
       }
       else {
         tree elt (CONCAT);
-        while (i < N(latex) && latex[i] != compound ("section", tree ())) {
+        while (i < N(latex) && latex[i] != compound ("nextbib", "")) {
           elt << latex[i];
           i++;
         }
@@ -907,7 +909,7 @@ bib_field (scheme_tree st, string field) {
         return tree_to_scheme_tree (doc[i][1]);
     }
   }
-  return tree ();
+  return "";
 }
 
 /******************************************************************************
@@ -916,7 +918,7 @@ bib_field (scheme_tree st, string field) {
 
 bool
 bib_empty (scheme_tree st, string f) {
-  return (bib_field (st, f) == tree ());
+  return (bib_field (st, f) == "");
 }
 
 /******************************************************************************
@@ -982,7 +984,7 @@ bib_strings_dict (tree t) {
           }
           dict(key)= sval;
         }
-	else if (is_atomic (val))
+        else if (is_atomic (val))
           dict(key)= val->label;
       }
     }
@@ -1037,7 +1039,7 @@ bib_select_entries (tree t, tree bib_t) {
       r->insert (b);
       entries << h[b];
       tree cr= bib_assoc (h[b], string ("crossref"));
-      if (cr != tree ()) bt << as_string (cr);
+      if (cr != "") bt << as_string (cr);
     }
   }
   return entries;
