@@ -13,6 +13,7 @@
 
 #include "aqua_dialogues.h"
 
+#include "gui.hpp" 
 #include "widget.hpp" 
 #include "message.hpp"
 #include "aqua_utilities.h"
@@ -32,7 +33,7 @@ class aqua_chooser_widget_rep: public aqua_widget_rep {
 protected:	
   command cmd;
   string type;
-  string mgn;
+  bool   save;
   string win_title;
   string directory;
   coord2 position;
@@ -40,7 +41,7 @@ protected:
   string file;
 	
 public:
-  aqua_chooser_widget_rep (command, string, string);
+  aqua_chooser_widget_rep (command, string, bool);
   ~aqua_chooser_widget_rep ();
 	
   virtual void send (slot s, blackbox val);
@@ -55,9 +56,9 @@ public:
   void perform_dialog();
 };
 
-aqua_chooser_widget_rep::aqua_chooser_widget_rep (command _cmd, string _type, string _mgn) 
+aqua_chooser_widget_rep::aqua_chooser_widget_rep (command _cmd, string _type, bool _save) 
 : aqua_widget_rep(), cmd(_cmd), type(_type), 
-  mgn(_mgn), position (coord2 (0, 0)), 
+  save(_save), position (coord2 (0, 0)), 
   size (coord2 (100, 100)), file ("")
 {
 }
@@ -199,12 +200,12 @@ widget aqua_chooser_widget_rep::plain_window_widget (string s)
 
 
 
-widget file_chooser_widget (command cmd, string type, string mgn) 
+widget file_chooser_widget (command cmd, string type, bool save) 
 // file chooser widget for files of a given type; for files of type "image",
 // the widget includes a previsualizer and a default magnification
 // for importation can be specified
 {
-  return tm_new <aqua_chooser_widget_rep> (cmd,type,mgn);
+  return tm_new <aqua_chooser_widget_rep> (cmd, type, save);
 }
 
 
@@ -271,9 +272,8 @@ void aqua_chooser_widget_rep::perform_dialog()
     file = from_nsstring([oPanel filename]);
     url u= url_system (scm_unquote (file));
     if (type == "image")
-      file = "(list (url-system " *
-	scm_quote (as_string (u)) *
-	") \"100\" \"100\" \"0\" \"0\" \"10\" \"10\")";
+      file = "(list (url-system " * scm_quote (as_string (u)) *
+             ") \"\" \"\" \"\" \"\")";
     //FIXME: fake image dimensions
     else
       file = "(url-system " * scm_quote (as_string (u)) * ")";
@@ -666,10 +666,12 @@ widget inputs_list_widget (command call_back, array<string> prompts)
 
 
 
-widget input_text_widget (command call_back, string type, array<string> def) 
+widget input_text_widget (command call_back, string type, array<string> def,
+                          int style, string width)
 // a textual input widget for input of a given type and a list of suggested
 // default inputs (the first one should be displayed, if there is one)
 {
+  (void) style; (void) width;
   return tm_new <aqua_input_text_widget_rep> (call_back, type, def);
 }
 
@@ -710,3 +712,21 @@ void aqua_tm_widget_rep::do_interactive_prompt()
         contextInfo: nil];
   [ih performSelector:@selector(delayedRun) withObject:nil afterDelay:0.0];
 }
+
+widget 
+printer_widget (command cmd, url u) {
+  (void) u;
+  return menu_button (text_widget ("Cancel", 0, black), cmd, "", "", 0);
+}
+
+
+widget
+color_picker_widget (command call_back, bool bg, array<tree> proposals) {
+  // widgets for selecting a color, a pattern or a background image,
+  // encoded by a tree. On input, we give a list of recently used proposals
+  // on termination the command is called with the selected color as argument
+  // the bg flag specifies whether we are picking a background color or fill
+  NOT_IMPLEMENTED;
+  (void) call_back; (void) bg; (void) proposals;
+  return glue_widget (false, false, 100*PIXEL, 100*PIXEL);
+}
