@@ -81,9 +81,9 @@ static hashmap<string,aqua_image> images;
 
 void 
 aqua_set_color (color col) {
-  int r, g, b;
-  get_rgb_color(col,r,g,b);
-  [[NSColor colorWithDeviceRed: r/255.0 green:g/255.0 blue:b/255.0 alpha:1.0] set];
+  int r, g, b, a;
+  get_rgb_color (col, r, g, b, a);
+  [[NSColor colorWithDeviceRed: r/255.0 green:g/255.0 blue:b/255.0 alpha:a/255.0] set];
 }
 
 aqua_renderer_rep::aqua_renderer_rep (int w2, int h2) :
@@ -240,13 +240,15 @@ struct aqua_cache_image_rep: cache_image_element_rep {
 
 void
 aqua_renderer_rep::image (url u, SI w, SI h, SI x, SI y,
-                        double cx1, double cy1, double cx2, double cy2) 
+                          double cx1, double cy1, double cx2, double cy2,
+                          int alpha)
 {
   // Given an image of original size (W, H),
   // we display the part (cx1 * W, xy1 * H, cx2 * W, cy2 * H)
   // at position (x, y) in a rectangle of size (w, h)
   
   // if (DEBUG_EVENTS) cout << "cg_renderer_rep::image " << as_string(u) << LF;
+  (void) alpha; // FIXME
   
   w= w/pixel; h= h/pixel;
   decode (x, y);
