@@ -64,6 +64,12 @@ aqua_widget_rep::popup_window_widget (string s) {
 }
 
 
+
+TMMenuItem *
+aqua_widget_rep::as_menuitem() { 
+  return [TMMenuItem alloc];  
+}
+
 /******************************************************************************
 * aqua_view_widget_rep
 ******************************************************************************/
@@ -427,7 +433,7 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
       updateVisibility();
     }
     break;
-  case SLOT_CONTEXT_ICONS_VISIBILITY:
+  case SLOT_MODE_ICONS_VISIBILITY:
     {
       TYPE_CHECK (type_box (val) == type_helper<bool>::id);
       bool f= open_box<bool> (val);
@@ -567,7 +573,7 @@ aqua_tm_widget_rep::query (slot s, int type_id) {
     TYPE_CHECK (type_id == type_helper<bool>::id);
     return close_box<bool> (visibility[3]);
         
-  case SLOT_CONTEXT_ICONS_VISIBILITY:
+  case SLOT_MODE_ICONS_VISIBILITY:
     TYPE_CHECK (type_id == type_helper<bool>::id);
     return close_box<bool> (visibility[2]);
     
@@ -668,9 +674,9 @@ TMMenuHelper *the_menu_helper = nil;
 void
 aqua_tm_widget_rep::write (slot s, blackbox index, widget w) {
   switch (s) {
-  case SLOT_CANVAS: 
+  case SLOT_SCROLLABLE: 
     {
-      check_type_void (index, "SLOT_CANVAS");
+      check_type_void (index, "SLOT_SCROLLABLE");
       NSView *v = ((aqua_view_widget_rep*) w.rep)->view;
       [sv setDocumentView: v];
       [[sv window] makeFirstResponder:v];
@@ -685,14 +691,19 @@ aqua_tm_widget_rep::write (slot s, blackbox index, widget w) {
     [bc setMenu:to_nsmenu(w) forRow:0];
     layout();
     break;
-  case SLOT_CONTEXT_ICONS:
-    check_type_void (index, "SLOT_CONTEXT_ICONS");
+  case SLOT_MODE_ICONS:
+    check_type_void (index, "SLOT_MODE_ICONS");
     [bc setMenu:to_nsmenu(w) forRow:1];
     layout();
     break;
+  case SLOT_FOCUS_ICONS:
+    check_type_void (index, "SLOT_FOCUS_ICONS");
+    [bc setMenu:to_nsmenu(w) forRow:2];
+    layout();
+    break;
   case SLOT_USER_ICONS:
     check_type_void (index, "SLOT_USER_ICONS");
-    [bc setMenu:to_nsmenu(w) forRow:2];
+    [bc setMenu:to_nsmenu(w) forRow:3];
     layout();
     break;
   case SLOT_INTERACTIVE_PROMPT:
@@ -804,6 +815,10 @@ aqua_window_widget_rep::send (slot s, blackbox val) {
     NOT_IMPLEMENTED ;
     // send_update (THIS, val);
     break;
+  case SLOT_REFRESH:
+    NOT_IMPLEMENTED ;
+    // send_refresh (THIS, val);
+    break;
     
   default:
     FAILED ("cannot handle slot type");
@@ -927,10 +942,51 @@ simple_widget_rep::handle_repaint (SI x1, SI y1, SI x2, SI y2) {
 
 void
 simple_widget_rep::send (slot s, blackbox val) {
-  aqua_view_widget_rep::send(s, val);
+  if (DEBUG_QT) cout << "aqua_simple_widget_rep::send " << slot_name(s) << LF;
+  switch (s) {
+    case SLOT_INVALIDATE:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord4>::id);
+      coord4 p= open_box<coord4> (val);
+      if (DEBUG_QT)
+        cout << "Invalidating rect " << rectangle(p.x1,p.x2,p.x3,p.x4) << LF;
+      aqua_renderer_rep* ren = (aqua_renderer_rep*)get_renderer (this);
+      if (ren) {
+        SI x1 = p.x1, y1 = p.x2, x2 = p.x3, y2 = p.x4;    
+        ren->outer_round (x1, y1, x2, y2);
+        ren->decode (x1, y1);
+        ren->decode (x2, y2);
+        //tm_canvas()->invalidate_rect (x1,y2,x2,y1);
+      }
+    }
+      break;
+    case SLOT_INVALIDATE_ALL:
+    {
+      ASSERT (is_nil (val), "type mismatch");
+      if (DEBUG_QT)
+        cout << "Invalidating all"<<  LF;
+      //tm_canvas()->invalidate_all ();
+    }
+      break;
+    case SLOT_CURSOR:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
+      coord2 p= open_box<coord2> (val);
+      //QPoint pt = to_qpoint(p);
+      //tm_canvas() -> setCursorPos(pt);
+    }
+      break;
+      
+    default:
+      if (DEBUG_QT) cout << "[aqua_simple_widget_rep] ";
+      aqua_view_widget_rep::send (s, val);
+      //      FAILED ("unhandled slot type");
+  }
+  
 }
 
 
+
 blackbox
 simple_widget_rep::query (slot s, int type_id) {
   return aqua_view_widget_rep::query(s,type_id);
@@ -1033,6 +1089,18 @@ glue_widget (bool hx, bool vx, SI w, SI h) {
   return tm_new <aqua_view_widget_rep> ([[[NSView alloc] initWithFrame:NSMakeRect(0, 0, 50, 50)] autorelease]);
 }
 
+widget
+glue_widget (tree col, bool hx, bool vx, SI w, SI h) {
+  (void) col;
+  return glue_widget (hx, vx, w, h);
+}
+
+widget
+extend (widget w, array<widget> a) {
+  (void) a;
+  return w;
+}
+
 widget
 wait_widget (SI width, SI height, string message) { 
   // a widget of a specified width and height, displaying a wait message
