@@ -4,10 +4,9 @@
 * DESCRIPTION: interface with the ispell spell checker
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Ispell/ispell.hpp"
@@ -46,47 +45,69 @@ ispeller_rep::ispeller_rep (string lan2): rep<ispeller> (lan2), lan (lan2) {}
 
 string
 ispeller_rep::start () {
-  if (nil (ln)) {
+  if (is_nil (ln)) {
+    string cmd;
 #ifdef OS_WIN32
     string prg= "\"$TEXMACS_PATH/bin/aspell/aspell.exe\"";
-    string cmd= prg * " --data-dir=.%//data --dict-dir=.%//dict -a";
+    cmd= prg * " --data-dir=.%//data --dict-dir=.%//dict -a";
 #else
-    string cmd= "ispell -a -d " * ispell_dictionary (lan)
-      * ispell_extra_args (lan);
+    if (exists_in_path ("aspell")) cmd= "aspell";
+    else
+#if defined (__MINGW__) || defined (__MINGW32__)
+      if (exists (url_system ("C:\\Program Files\\Aspell\\bin\\aspell.exe")))
+        cmd= "\"C:\\Program Files\\Aspell\\bin\\aspell.exe\"";
+      else
+#endif
+        cmd= "ispell";
+    cmd << " -a -d " * ispell_dictionary (lan) * ispell_extra_args (lan);
 #endif
     ln= make_pipe_link (cmd);
   }
   if (ln->alive) return "ok";
   string message= ln->start ();
-  if (starts (message, "Error: ")) return message;
+  if (DEBUG_IO) cout << "Ispell] Received " << message << "\n";
+  if (starts (message, "Error: ")) {
+    if (ln->alive) ln->stop ();
+    return message;
+  }
   message= retrieve ();
+  if (DEBUG_IO) cout << "Ispell] Received " << message << "\n";
 #ifdef OS_WIN32
   if (search_forwards (message, 0, "@(#)")) return "ok";
 #else
   if (starts (message, "@(#)")) return "ok";
 #endif
+  if (ln->alive) ln->stop ();
   return "Error: no dictionary for#" * lan;
 }
 
 string
 ispeller_rep::retrieve () {
   string ret;
+#if defined (__MINGW__) || defined (__MINGW32__)
+  while ((ret != "\r\n") && (!ends (ret, "\r\n\r\n")) &&
+	 ((!ends (ret, "\r\n")) || (!starts (ret, "@(#)"))))
+#else
   while ((ret != "\n") && (!ends (ret, "\n\n")) &&
 	 ((!ends (ret, "\n")) || (!starts (ret, "@(#)"))))
+#endif
     {
       ln->listen (10000);
       string mess = ln->read (LINK_ERR);
       string extra= ln->read (LINK_OUT);
       if (mess  != "") cerr << "TeXmacs] ispell error: " << mess << "\n";
-      if (extra == "") return "Error: ispell does not respond";
+      if (extra == "") {
+	ln->stop ();
+	return "Error: ispell does not respond";
+      }
       ret << extra;
     }
-  return ispell_decode(lan,ret);
+  return ispell_decode (lan, ret);
 }
 
 void
 ispeller_rep::send (string cmd) {
-  ln->write (ispell_encode(lan,cmd) * "\n", LINK_IN);
+  ln->write (ispell_encode (lan, cmd) * "\n", LINK_IN);
 }
 
 /******************************************************************************
@@ -192,7 +213,11 @@ ispell_decode (string lan, string s) {
 
 static tree
 parse_ispell (string s) {
+#if defined (__MINGW__) || defined (__MINGW32__)
+  while (ends (s, "\r\n")) s= s (0, N(s)-2);
+#else
   while (ends (s, "\n")) s= s (0, N(s)-1);
+#endif
   bool flag= true;
   int i, j;
   tree t (TUPLE);
@@ -218,16 +243,17 @@ parse_ispell (string s) {
 static void
 ispell_send (string lan, string s) {
   ispeller sc= ispeller (lan);
-  if ((!nil (sc)) && sc->ln->alive) sc->send (s);
+  if ((!is_nil (sc)) && sc->ln->alive) sc->send (s);
 }
 
 static string
 ispell_eval (string lan, string s) {
   ispeller sc= ispeller (lan);
-  if ((!nil (sc)) && sc->ln->alive) {
+  if ((!is_nil (sc)) && sc->ln->alive) {
     sc->send (s);
     return sc->retrieve ();
   }
+  return "";
 }
 
 /******************************************************************************
@@ -236,15 +262,17 @@ ispell_eval (string lan, string s) {
 
 string
 ispell_start (string lan) {
+  if (DEBUG_IO) cout << "Ispell] Start " << lan << "\n";
   ispeller sc= ispeller (lan);
-  if (nil (sc)) sc= new ispeller_rep (lan);
+  if (is_nil (sc)) sc= tm_new<ispeller_rep> (lan);
   return sc->start ();
 }
 
 tree
 ispell_check (string lan, string s) {
+  if (DEBUG_IO) cout << "Ispell] Check " << s << "\n";
   ispeller sc= ispeller (lan);
-  if (nil (sc) || (!sc->ln->alive)) {
+  if (is_nil (sc) || (!sc->ln->alive)) {
     string message= ispell_start (lan);
     if (starts (message, "Error: ")) return message;
   }
@@ -255,15 +283,18 @@ ispell_check (string lan, string s) {
 
 void
 ispell_accept (string lan, string s) {
+  if (DEBUG_IO) cout << "Ispell] Accept " << s << "\n";
   ispell_send (lan, "@" * s);
 }
 
 void
 ispell_insert (string lan, string s) {
+  if (DEBUG_IO) cout << "Ispell] Insert " << s << "\n";
   ispell_send (lan, "*" * s);
 }
 
 void
 ispell_done (string lan) {
+  if (DEBUG_IO) cout << "Ispell] End " << lan << "\n";
   ispell_send (lan, "#");
 }
