@@ -450,12 +450,31 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
     }
 
     string r;
+#ifdef __MINGW32__
+    // denis begin
+    if (event->text().count() == 1) {
+      QChar c= event->text()[0];
+      if (c.isPrint() && event->modifiers() != Qt::MetaModifier) {
+	// not a control character or dead key or modifier
+	char ac=c.toAscii();
+	if (ac && ac != ' ') { // a true ascii printable
+	  r= ac;
+	  if (DEBUG_QT) cout << "ascii key= " <<r << "\n";	
+	  the_gui->process_keypress(wid, r, texmacs_time());
+	  return;
+	}
+      }
+    }
+    // denis end
+#endif
     if (qtkeymap->contains (key)) {
       r = qtkeymap[key];
-    } else if (qtdeadmap->contains (key)) {
+    }
+    else if (qtdeadmap->contains (key)) {
       mods &=~ Qt::ShiftModifier;
       r = qtdeadmap[key];
-    } else {
+    }
+    else {
       QString nss = event->text();
       unsigned short unic= nss.data()[0].unicode();
       if (unic < 32 && key < 128 && key > 0) {
@@ -713,7 +732,8 @@ QTMWidget::inputMethodEvent (QInputMethodEvent* event) {
     if (pos <  preedit_string.count()) {
       for(int i=0; i< attrs.count(); i++) 
         if ((attrs[i].type == QInputMethodEvent::TextFormat) &&
-            (attrs[i].start <= pos) && (pos < attrs[i].start + attrs[i].length)) {
+            (attrs[i].start <= pos) &&
+            (pos < attrs[i].start + attrs[i].length)) {
           sel_start = attrs[i].start;
           sel_length =  attrs[i].length;
           if (!visible_cur) pos += attrs[i].length;
@@ -722,9 +742,9 @@ QTMWidget::inputMethodEvent (QInputMethodEvent* event) {
       sel_start = pos;
       sel_length = 0;
     }
+    (void) sel_start; (void) sel_length;
     
-    r = r * as_string(pos) * ":" 
-    * from_qstring(preedit_string);
+    r = r * as_string(pos) * ":" * from_qstring(preedit_string);
   }
   simple_widget_rep *wid =  tm_widget();
   if (wid)
