@@ -30,21 +30,9 @@ void QTMWindow::closeEvent ( QCloseEvent *event )
  // QMainWindow::closeEvent (event);
 }
 
-void QTMScrollArea::resizeEvent( QResizeEvent* event )
+void QTMPlainWindow::closeEvent ( QCloseEvent *event )
 {
-  if (DEBUG_QT) cout << "RESIZE SCROLLAREA" << LF;
-  //FIXME: the following is an hack needed to trigger widget resize at the 
-  //       texmacs level when the scrollarea is resized. needed because the 
-  //       texmacs canvas minimal geometry in papyrus mode depends on the 
-  //       scrollarea height. 
-
-  if (tm_widget()->tm_canvas()) 
-  {
-    
-    SI x1, y1, x2, y2;
-    get_extents (tm_widget(), x1, y1, x2, y2);
-    set_extents (tm_widget(), x1, y1, x2, y2);
-  }
-
-  QScrollArea::resizeEvent (event);
+  if (DEBUG_QT)   cout << "CLOSE PLAIN WINDOW" << LF;
+  (void) event;
+  emit closed();
 }
