@@ -166,9 +166,14 @@ qt_chooser_widget_rep::write (slot s, blackbox index, widget w) {
 }
 
 widget
-qt_chooser_widget_rep::plain_window_widget (string s)
+qt_chooser_widget_rep::plain_window_widget (string s, command q)
 {
+  // Identify ourselves as already being a top-level window, even though
+  // the dialog won't be displayed until later.
+  //view->setProperty ("texmacs_window_widget",
+  //                 QVariant::fromValue ((void*) this));
   win_title = s;
+  quit = q;       // (NOT USED)
   return this;
 }
 
@@ -185,7 +190,7 @@ qt_chooser_widget_rep::perform_dialog () {
   QStringList _files;
   QString _filter, _suffix, _caption, _directory;
   _caption = to_qstring (win_title);
-  _directory = to_qstring (directory * "/" * file);
+  _directory = QString::fromLocal8Bit (as_charp (directory * "/" * file));
   
 #if (QT_VERSION >= 0x040400)
   if (type == "directory") {
@@ -253,6 +258,7 @@ qt_chooser_widget_rep::perform_dialog () {
     }
   }
   cmd ();
+  if (!is_nil(quit)) quit ();
 }
 #else
 /**
@@ -356,5 +362,6 @@ qt_chooser_widget_rep::perform_dialog () {
   delete dialog;
   
   cmd ();
+  if (!is_nil(quit)) quit ();
 }
 #endif
