@@ -42,6 +42,7 @@ public:
   QTMGuiHelper *gui_helper;
   QTimer *updatetimer;
   QList<QLabel*> waitDialogs;
+  QWidget *waitWindow;
 
   hashmap<string,tree>   selection_t;
   hashmap<string,string> selection_s;
@@ -59,8 +60,8 @@ public:
   void event_loop ();
 
   /* interclient communication */
-  virtual bool get_selection (string key, tree& t, string& s);
-  virtual bool set_selection (string key, tree t, string s);
+  virtual bool get_selection (string key, tree& t, string& s, string format);
+  virtual bool set_selection (string key, tree t, string s, string format);
   virtual void clear_selection (string key);
 
   /* miscellaneous */
@@ -82,6 +83,7 @@ public:
   void process_mouse (simple_widget_rep *wid, string kind, SI x, SI y, int mods, time_t t);
   void process_resize (simple_widget_rep *wid, SI x, SI y);
   void process_command (command _cmd);
+  void process_command (command _cmd, object _args);
   void process_socket_notification (socket_notifier sn);
   void process_delayed_commands (); 
 
@@ -95,4 +97,7 @@ public:
   
 };
 
+void force_update(); 
+// force an immediate update of the internal texmacs state
+
 #endif // defined QT_GUI_HPP
