@@ -95,14 +95,6 @@ qt_renderer_rep::begin (void* handle) {
 
 void qt_renderer_rep::end () { painter->end (); }
 
-QColor
-qt_color(color c)
-{
-  int r, g, b;
-  get_rgb_color (c,r,g,b);
-  return QColor(r, g, b);
-}
-
 void 
 qt_renderer_rep::get_extents (int& w2, int& h2) {  
   if (painter->device()) {
@@ -143,8 +135,8 @@ qt_renderer_rep::set_color (color c) {
   basic_renderer_rep::set_color(c);
   QPen p (painter->pen ());
   QBrush b (painter->brush ());
-  p.setColor (qt_color(cur_fg));
-  b.setColor (qt_color(cur_fg));
+  p.setColor (to_qcolor(cur_fg));
+  b.setColor (to_qcolor(cur_fg));
   painter->setPen (p);
   painter->setBrush (b);
 }
@@ -196,7 +188,7 @@ qt_renderer_rep::clear (SI x1, SI y1, SI x2, SI y2) {
   decode (x1, y1);
   decode (x2, y2);
   if ((x1>=x2) || (y1<=y2)) return;
-  QBrush brush (qt_color(cur_bg));
+  QBrush brush (to_qcolor(cur_bg));
   painter->setRenderHints (0);
   painter->fillRect (x1, y2, x2-x1, y1-y2, brush);       
 }
@@ -222,7 +214,7 @@ qt_renderer_rep::fill (SI x1, SI y1, SI x2, SI y2) {
   decode (x1, y1);
   decode (x2, y2);
 
-  QBrush brush (qt_color(cur_fg));
+  QBrush brush (to_qcolor(cur_fg));
   painter->setRenderHints (0);
   painter->fillRect (x1, y2, x2-x1, y1-y2, brush);       
 }
@@ -243,7 +235,7 @@ qt_renderer_rep::fill_arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta) {
   if ((x1>=x2) || (y1>=y2)) return;
   decode (x1, y1);
   decode (x2, y2);
-  QBrush brush(qt_color(cur_fg));
+  QBrush brush(to_qcolor(cur_fg));
   QPainterPath pp;
   pp.arcMoveTo (x1, y2, x2-x1, y1-y2, alpha / 64);
   pp.arcTo (x1, y2, x2-x1, y1-y2, alpha / 64, delta / 64);
@@ -263,7 +255,7 @@ qt_renderer_rep::polygon (array<SI> x, array<SI> y, bool convex) {
     decode (xx, yy);
     poly[i] = QPointF (xx, yy);
   }
-  QBrush brush(qt_color(cur_fg));
+  QBrush brush(to_qcolor(cur_fg));
   QPainterPath pp;
   pp.addPolygon (poly);
   pp.closeSubpath ();
@@ -286,12 +278,12 @@ struct qt_cache_image_rep: cache_image_element_rep {
 
 void
 qt_renderer_rep::image (url u, SI w, SI h, SI x, SI y,
-                        double cx1, double cy1, double cx2, double cy2)
+                        double cx1, double cy1, double cx2, double cy2,
+                        int alpha)
 {
   // Given an image of original size (W, H),
   // we display the part (cx1 * W, xy1 * H, cx2 * W, cy2 * H)
   // at position (x, y) in a rectangle of size (w, h)
-
   if(cx2<=cx1 || cy2<=cy1) return;
 
   w= w/pixel; h= h/pixel;
@@ -315,7 +307,8 @@ qt_renderer_rep::image (url u, SI w, SI h, SI x, SI y,
     if (qt_supports (u)) {
       pm= new QImage (utf8_to_qstring (concretize (u)));
       needs_crop= true;
-    } else if (suffix (u) == "ps" ||
+    }
+    else if (suffix (u) == "ps" ||
              suffix (u) == "eps" ||
              suffix (u) == "pdf") {
       url temp= url_temp (".png");
@@ -396,7 +389,10 @@ qt_renderer_rep::image (url u, SI w, SI h, SI x, SI y,
     (ci->nr)++;
   }
 
+  qreal old_opacity= painter->opacity ();
+  painter->setOpacity (qreal (alpha) / qreal (255));
   painter->drawImage (x, y-h, *pm);
+  painter->setOpacity (old_opacity);
 };
 
 
@@ -430,8 +426,8 @@ qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
   basic_character xc (c, fng, sfactor, cur_fg, 0);
   qt_image mi = character_image [xc];
   if (is_nil(mi)) {
-    int r, g, b;
-    get_rgb (cur_fg, r, g, b);
+    int r, g, b, a;
+    get_rgb (cur_fg, r, g, b, a);
     SI xo, yo;
     glyph pre_gl= fng->get (c); if (is_nil (pre_gl)) return;
     glyph gl= shrink (pre_gl, sfactor, sfactor, xo, yo);
@@ -450,7 +446,7 @@ qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
       for (j=0; j<h; j++)
         for (i=0; i<w; i++) {
           int col = gl->get_x (i, j);
-          brush.setColor (QColor (r, g, b, (255*col)/nr_cols));
+          brush.setColor (QColor (r, g, b, (a*col)/nr_cols));
           pp.fillRect (i, j, 1, 1, brush);
         }
       pp.end();
@@ -469,7 +465,7 @@ qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
       for (j=0; j<h; j++)
         for (i=0; i<w; i++) {
           int col = gl->get_x (i, j);
-          im->setPixel (i, j, qRgba (r, g, b, (255*col)/nr_cols));
+          im->setPixel (i, j, qRgba (r, g, b, (a*col)/nr_cols));
         }
     }
 #endif
@@ -499,7 +495,12 @@ qt_renderer_rep::xpm_image (url file_name) {
   qt_pixmap mi= images [as_string (file_name)];
   if (is_nil (mi)) {
     string sss;
-    load_string ("$TEXMACS_PIXMAP_PATH" * file_name, sss, false);
+    if (suffix (file_name) == "xpm") {
+      url png_equiv= glue (unglue (file_name, 3), "png");
+      load_string ("$TEXMACS_PIXMAP_PATH" * png_equiv, sss, false);
+    }
+    if (sss == "")
+      load_string ("$TEXMACS_PIXMAP_PATH" * file_name, sss, false);
     if (sss == "")
       load_string ("$TEXMACS_PATH/misc/pixmaps/TeXmacs.xpm", sss, true);
     uchar *buf= (uchar*) as_charp (sss);
