@@ -63,7 +63,8 @@ public:
   void  polygon (array<SI> x, array<SI> y, bool convex=true);
   void  xpm (url file_name, SI x, SI y);
   void  image (url u, SI w, SI h, SI x, SI y,
-               double cx1, double cy1, double cx2, double cy2);
+               double cx1, double cy1, double cx2, double cy2,
+               int alpha);
 
   void draw_clipped (QImage * im, int w, int h, SI x, SI y);
   void draw_clipped (QPixmap * im, int w, int h, SI x, SI y);
