@@ -103,8 +103,7 @@ tweek_iconbar_size (QSize& sz) {
     sz.setHeight (sz.height () + 6);
   }
   else if (sz.height () >= 20) {
-    sz.setWidth (sz.width () + 2);
-    sz.setHeight (sz.height () + 4);
+    sz.setHeight (sz.height () + 2);
   }
   else if (sz.height () >= 16) {
     sz.setHeight (sz.height () + 2);
@@ -127,12 +126,12 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
   
   QMainWindow* mw= tm_mainwindow ();
   mw->setStyle (qtmstyle ());
-  mw->menuBar()->setStyle (qtmstyle ()); 
+  mw->menuBar()->setStyle (qtmstyle ());
   
   // there is a bug in the early implementation of toolbars in Qt 4.6
   // which has been fixed in 4.6.2 (at least)
   // this is why we change dimension of icons
-	
+  
 #if (defined(Q_WS_MAC)&&(QT_VERSION>=QT_VERSION_CHECK(4,6,0))&&(QT_VERSION<QT_VERSION_CHECK(4,6,2)))
   mw->setIconSize (QSize (22, 30));  
 #else
@@ -185,7 +184,7 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
   modeToolBar  = new QToolBar ("mode toolbar", mw);
   focusToolBar = new QToolBar ("focus toolbar", mw);
   userToolBar   = new QToolBar ("user toolbar", mw);
- 
+
   mainToolBar->setStyle (qtmstyle ());
   modeToolBar->setStyle (qtmstyle ());
   focusToolBar->setStyle (qtmstyle ());
@@ -193,16 +192,16 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
   
   {
     // set proper sizes for icons
-    QPixmap *pxm = the_qt_renderer()->xpm_image("tm_new.xpm");
-    QSize sz = (pxm ? pxm->size() : QSize(32,32));
+    QPixmap *pxm = the_qt_renderer()->xpm_image ("tm_new.xpm");
+    QSize sz = (pxm ? pxm->size() : QSize(24,24));
     tweek_iconbar_size (sz);
     mainToolBar->setIconSize (sz);
-    pxm =  the_qt_renderer()->xpm_image("tm_section.xpm");
-    sz = (pxm ? pxm->size() : QSize(17,17));
+    pxm = the_qt_renderer()->xpm_image ("tm_section.xpm");
+    sz = (pxm ? pxm->size() : QSize(20,20));
     tweek_iconbar_size (sz);
     modeToolBar->setIconSize(sz);
-    pxm =  the_qt_renderer()->xpm_image("tm_plus.xpm");
-    sz = (pxm ? pxm->size() : QSize(17,17));
+    pxm = the_qt_renderer()->xpm_image ("tm_add.xpm");
+    sz = (pxm ? pxm->size() : QSize(16,16));
     tweek_iconbar_size (sz);
     focusToolBar->setIconSize(sz);
   }  
@@ -212,7 +211,8 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
 
   mw->setUnifiedTitleAndToolBarOnMac(true);
 
-  QWidget *cw= new QWidget ();
+  QWidget *cw= new QWidget ();  // QMainWindow will take ownership later and delete it when needed.
+  cw->setObjectName("central widget"); // this is important for styling toolbars.
   
   QBoxLayout *bl = new QBoxLayout(QBoxLayout::TopToBottom, cw);
   bl->setContentsMargins(2,2,2,2);
@@ -242,17 +242,22 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
   rulerWidget = new QWidget(cw);
   rulerWidget->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
   rulerWidget->setMinimumHeight(1);
-  rulerWidget->setBackgroundRole(QPalette::Dark);
+  rulerWidget->setBackgroundRole(QPalette::Mid);
+  // FIXME: how to use 112 (active) and 146 (passive)
   rulerWidget->setVisible(false);
   rulerWidget->setAutoFillBackground(true);
-//  rulerWidget = new QLabel("pippo", cw);
-  
+  // rulerWidget = new QLabel("pippo", cw);
   
+
   bl->insertWidget(0, modeToolBar);
   bl->insertWidget(1, rulerWidget);
   bl->insertWidget(2, focusToolBar);
   bl->insertWidget(3, userToolBar);
 
+  tw->setContentsMargins (2, 0, 2, 0);
+  mw->setContentsMargins (-2, -2, -2, -2);
+  bar->setContentsMargins (0, 0, 0, 2);
+
 #else
   mw->setCentralWidget(tw);
   
@@ -279,7 +284,6 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
 #ifndef Q_WS_MAC
   tm_mainwindow()->menuBar()->setVisible (false);
 #endif  
- 
 }
 
 qt_tm_widget_rep::~qt_tm_widget_rep () {
@@ -675,7 +679,28 @@ qt_tm_widget_rep::install_main_menu () {
   QMenu* m= concrete (main_menu_widget)->get_qmenu();
   if (m) {
     {
+      // REMARK: We do not want the menubar shared across windows as suggested  
+      // in http://doc.qt.nokia.com/4.7/qmainwindow.html#menuBar
+      // e.g. :
+      //
+      //     QMenuBar *dest = new QMenuBar(0);
+      //     tm_mainwindow()->setMenuBar(dest);
+      //
+      // as the default behavior on MacOS. The main reason is that in TeXmacs
+      // different widows can have different main menus so that it is indeed
+      // appropriate change the main menu as the window focus changes. 
+      // So we kindly ask to each window to give us its own menu and we install
+      // there our actions.
+      // So we do:
+
       QMenuBar *dest = tm_mainwindow()->menuBar();
+      
+      // and everything is fine.
+      // Also please note that we have to do the replacement and not simply 
+      // install the menu returned by get_qmenu() since the main menu there 
+      // could contain some defaults items appropriate for the give OS (like the
+      // service menu on MacOS) which are not present in our menu widget.
+      
       QWidget *src = m;
       replaceActions(dest,src);
       QList<QAction*> list = dest->actions();
@@ -698,9 +723,9 @@ qt_tm_widget_rep::write (slot s, blackbox index, widget w) {
     cout << "qt_tm_widget_rep::write " << slot_name (s) << LF;
   
   switch (s) {
-    case SLOT_CANVAS:
+    case SLOT_SCROLLABLE:
     {
-      check_type_void (index, "SLOT_CANVAS");
+      check_type_void (index, "SLOT_SCROLLABLE");
       QStackedWidget* tw= tm_centralwidget();
       QWidget *new_widget= concrete(w)->get_canvas();
       QWidget *old_widget= tw->currentWidget();
@@ -840,94 +865,11 @@ qt_tm_widget_rep::set_full_screen(bool flag) {
   tm_scrollarea()->setVerticalScrollBarPolicy(flag ? Qt::ScrollBarAlwaysOff : Qt::ScrollBarAsNeeded);
 }
 
+// creates a decorated window with name s and contents w
 widget
-qt_tm_widget_rep::plain_window_widget (string s) {
-    // creates a decorated window with name s and contents w
-  widget w= qt_view_widget_rep::plain_window_widget (s);
-    // to manage correctly retain counts
-  qt_window_widget_rep* wid= (qt_window_widget_rep*) (w.rep);
+qt_tm_widget_rep::plain_window_widget (string s, command q) {
+
+  widget w= qt_view_widget_rep::plain_window_widget (s, q);
+  qt_window_widget_rep* wid= (qt_window_widget_rep*) (w.rep);   // to manage correctly retain counts
   return wid;
 }
-
-//#if !defined(_MBD_USE_NEW_INTERACTIVE_PROMPT)
-#if 0
-void
-qt_tm_widget_rep::do_interactive_prompt () {
-  QStringList items;
- // QString label= to_qstring (tm_var_encode (((qt_text_widget_rep*) int_prompt.rep)->str));
-  qt_input_text_widget_rep* it = (qt_input_text_widget_rep*) (int_input.rep);
-  if ( N(it->def) == 0) {
-    items << "";
-  } else {
-    for (int j=0; j < N(it->def); j++) {
-      items << to_qstring(it->def[j]);
-    }
-  }
-  QDialog *d = new QDialog (tm_scrollarea()->window());
-
-  QVBoxLayout* vl = new QVBoxLayout();
-  QHBoxLayout *hl = new QHBoxLayout();
-  
- // QLabel *lab = new QLabel (label,&d);
-  QLayoutItem *lab = int_prompt->as_qlayoutitem ();
-  QComboBox *cb = new QComboBox(d);
-  cb-> setObjectName("input"); // to find it
-  cb -> setSizeAdjustPolicy (QComboBox::AdjustToMinimumContentsLength);
-  cb -> setEditText (items[0]);
-  int minlen = 0;
-  for(int j=0; j < items.count(); j++) {
-    cb -> addItem (items[j]);
-    int c = items[j].count();
-    if (c > minlen) minlen = c;
-  }
-  cb -> setMinimumContentsLength (minlen>50 ? 50 : (minlen < 2 ? 10 : minlen));
-  cb -> setEditable (true);
-    // apparently the following flag prevents Qt from substituting an history item
-    // for an input when they differ only from the point of view of case (upper/lower)
-    // eg. if the history contains aAAAAa and you type AAAAAA then the combo box
-    // will retain the string aAAAAa
-  cb->setDuplicatesEnabled(true); 
-  cb->completer()->setCaseSensitivity(Qt::CaseSensitive);
-  
-  if (QLabel *_lab = qobject_cast<QLabel*>(lab ->widget())) _lab -> setBuddy (cb);
-  hl -> addItem (lab);
-  hl -> addWidget (cb);
-  vl -> addLayout (hl);
-  
-  if (ends (it->type, "file") || it->type == "directory") {
-      // autocompletion
-    QCompleter *completer = new QCompleter(d);
-    QDirModel *dirModel = new QDirModel(d);
-    completer->setModel(dirModel);
-    cb->setCompleter(completer);
-  }
-  
-  {
-    QDialogButtonBox* buttonBox =
-    new QDialogButtonBox (QDialogButtonBox::Ok | QDialogButtonBox::Cancel,
-                          Qt::Horizontal, d);
-    QObject::connect (buttonBox, SIGNAL (accepted()), d, SLOT (accept()));
-    QObject::connect (buttonBox, SIGNAL (rejected()), d, SLOT (reject()));
-    vl -> addWidget (buttonBox);
-  }
-  d->setLayout (vl);
-  
-  QRect wframe = view->window()->frameGeometry();
-  QPoint pos = QPoint(wframe.x()+wframe.width()/2,wframe.y()+wframe.height()/2);
-  
-  d->setWindowTitle("Interactive Prompt");
-  d->updateGeometry();
-  QSize sz = d->sizeHint();
-  QRect r; r.setSize(sz);
-  r.moveCenter(pos);
-  d->setGeometry(r);
-  QObject::connect (d, SIGNAL (finished (int)), &helper, SLOT(commit (int)));
-#if (QT_VERSION >= 0x040500)
-  d->open();
-#else
-  d->exec();
-#endif
-}
-#else
-
-#endif
