@@ -19,15 +19,29 @@
 #include "widget.hpp"
 #include "message.hpp"
 #include "promise.hpp"
+#include "scheme.hpp"
 //#import "TMView.h"
 #include <QtGui>
 
-
+#include "QTMWindow.hpp"
 #include "QTMGuiHelper.hpp"
 #include "qt_gui.hpp"
 #include "qt_window_widget.hpp"
 #include "qt_menu.hpp"
 
+#include "../../Style/Evaluate/evaluate_main.hpp" // required for as_length(string)
+
+
+const char *ui_type_string[]= {
+  "horizontal_menu", "vertical_menu", "horizontal_list", "vertical_list",
+  "tile_menu", "minibar_menu", "menu_separator", "menu_group", 
+  "pulldown_button", "pullright_button", "menu_button",
+  "balloon_widget", "text_widget", "xpm_widget", "toggle_widget",
+  "enum_widget", "choice_widget", "scrollable_widget",
+  "hsplit_widget", "vsplit_widget", 
+  "aligned_widget", "tabs_widget", "wrapped_widget"
+};
+
 
 /******************************************************************************
  * Auxiliary classes
@@ -335,35 +349,47 @@ qt_ui_element_rep::popup_window_widget (string s)  {
 
 
 
-
-class qt_plain_widget_rep: public qt_view_widget_rep {
-  
+/*! Models the simplest top-level window possible
+ *
+ * When any simple TeXmacs widgets needs promotion into a TeXmacs window, we
+ * encapsulate it into a qt_plain_window_widget. This class handles the necessary
+ * slots, in particular all those already handled by qt_view_widget_rep. It also
+ * marks the enclosed QWidget as a "texmacs_window_widget" using QObject::setProperty().
+ * This will be checked by qt_view_widget_rep::read()'s SLOT_WINDOW.
+ */
+class qt_plain_window_widget_rep: public qt_view_widget_rep {
+  // QWidget* view;   // inherited
 public:
-  qt_plain_widget_rep (QWidget *v) : qt_view_widget_rep(v) {};
-  ~qt_plain_widget_rep () {};
+
+  qt_plain_window_widget_rep (QWidget* w) : qt_view_widget_rep(w) {
+    ASSERT(w != NULL, "Null QWidgets cannot be promoted to windows");
+    w->setProperty ("texmacs_window_widget", QVariant::fromValue ((void*) this));
+  };
+  ~qt_plain_window_widget_rep () {};
   
   virtual void send (slot s, blackbox val);
-//  virtual blackbox query (slot s, int type_id);
-//  virtual widget read (slot s, blackbox index);
-//  virtual void write (slot s, blackbox index, widget w);
-//  virtual void notify (slot s, blackbox new_val);
-  
- // virtual widget plain_window_widget (string s);
- // void set_current_renderer(basic_renderer _r) { current_renderer = _r;  }
- // basic_renderer get_current_renderer() {  return current_renderer; }
- // virtual QWidget* as_qwidget () { return view ; };
-  
 };
 
 
 void
-qt_plain_widget_rep::send (slot s, blackbox val) {
+qt_plain_window_widget_rep::send (slot s, blackbox val) {
   if (DEBUG_QT)
     cout << "qt_plain_widget_rep::send " << slot_name(s) << LF;
   switch (s) {
     case SLOT_POSITION:
-      ASSERT (type_box (val) == type_helper<coord2>::id, "type mismatch");
-      NOT_IMPLEMENTED;
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
+      coord2 p= open_box<coord2> (val);
+      if (view) {
+        QPoint pt = to_qpoint (p);
+        pt.ry() += 40;
+        // to avoid window under menu bar on MAC when moving at (0,0)
+        if (DEBUG_QT) 
+          cout << "Moving to (" << pt.x() << "," 
+          << pt.y() << ")" << LF;
+        view->move (pt);
+      }
+    }
       break;
     case SLOT_VISIBILITY:
     {   
@@ -382,20 +408,25 @@ qt_plain_widget_rep::send (slot s, blackbox val) {
 
 
 widget 
-qt_ui_element_rep::plain_window_widget (string s)  {
-  QLayoutItem *li = as_qlayoutitem();
-  QWidget *w;
+qt_ui_element_rep::plain_window_widget (string s, command quit)  {
+  QLayoutItem *li     = as_qlayoutitem();
+  QTMPlainWindow* win = new QTMPlainWindow();
+  win->setWindowTitle (to_qstring (s));  
+  
   if (li->widget()) 
-    w = li->widget();
-  else if (li->layout()) {
-    w = new QWidget();
-    w->setLayout(li->layout());
-  } else {
-    w = new QWidget();
-  }
-  w->setWindowTitle (to_qstring (s));
-  return tm_new<qt_plain_widget_rep>(w);
-//  concrete(make_popup_widget())->plain_window_widget(s);
+    win->layout()->addItem(li);
+  else if (li->layout())
+    win->setLayout(li->layout());
+  
+
+  QTMCommand* qtmcmd = new QTMCommand(quit);
+  qtmcmd->setParent(win);
+  QObject::connect(win, SIGNAL(closed()), qtmcmd, SLOT(apply()));
+
+  // FIXME: using qt_plain_window_widget causes a crash when closing "secondary"
+  // windows. (example widget6 in menu-test.scm)
+  //return tm_new<qt_plain_window_widget_rep>(win);
+  return tm_new<qt_window_widget_rep>(win, quit);
 }
 
 QMenu *
@@ -407,6 +438,31 @@ qt_ui_element_rep::get_qmenu() {
 }
 
 
+/*! Ad-hoc command to be used to simulate keypresses
+ * 
+ * \sa qt_ui_element, , qt_ui_element_rep::as_qaction
+ */
+
+class qt_key_command_rep: public command_rep {
+  string ks; 
+    
+public:
+  qt_key_command_rep(string ks_) : ks(ks_) { }
+  
+  void apply () {
+  if (N(ks)) { 
+    QTMWidget *w = qobject_cast<QTMWidget*>(qApp->focusWidget());
+      if (w && w->tm_widget()) {
+        if (DEBUG_QT) cout << "shortcut: " << ks << LF;
+        the_gui -> process_keypress (w->tm_widget(), ks, texmacs_time());
+      }
+    }
+  }
+  
+  tm_ostream& print (tm_ostream& out) { return out << "qt_key_command_rep"; }
+};
+
+
 QAction* 
 qt_ui_element_rep::as_qaction () {
   switch (type) {
@@ -467,15 +523,20 @@ qt_ui_element_rep::as_qaction () {
       return a;
     }
       break;
-      
+    case glue_widget:
+    {
+      QTMAction* a = new QTMAction();
+      a->setEnabled(false);
+      return a;
+    }
+      break;
     case menu_group: 
     {
       typedef pair<string, int> T;
       T x = open_box<T>(load);
       string name = x.x1;
       int style = x.x2;
-      
-      (void) style;
+
       // a menu group; the name should be greyed and centered
       QAction* a= new QTMAction (NULL);
       a->setText(to_qstring(tm_var_encode ((name))));
@@ -529,22 +590,24 @@ qt_ui_element_rep::as_qaction () {
 #ifdef Q_WS_MAC
       if (search_forwards (" ", ks) != -1) ks= "";
 #endif
+      QTMCommand* c;
       if (N(ks) > 0) {
         string qtks = conv (ks);
         QKeySequence qks (to_qstring (qtks));
         if (DEBUG_QT)
           cout << "ks: " << ks << " " << qks.toString().toAscii().data() << "\n";
         a->setShortcut (qks);
-        QTMKeyCommand* c= new QTMKeyCommand (ks);
-        c->setParent (a);
-        QObject::connect (a, SIGNAL (triggered ()), c, SLOT (apply ()),
-                          Qt::QueuedConnection);    
+        command key_cmd = tm_new<qt_key_command_rep>(ks);
+        c= new QTMCommand (key_cmd);
       } else {
-        QTMCommand* c= new QTMCommand (cmd.rep);
-        c->setParent (a);
-        QObject::connect (a, SIGNAL (triggered ()), c, SLOT (apply ()),
-                          Qt::QueuedConnection);    
+        c= new QTMCommand (cmd);
       }
+      c->setParent (a);
+        
+      // NOTE: this used to be a Qt::QueuedConnection, but the slot would not
+      // be called if in a contextual menu
+      QObject::connect (a, SIGNAL (triggered ()), c, SLOT (apply ()));    
+  
       // FIXME: implement complete prefix handling
       a->setEnabled (ok? true: false);
       
@@ -575,6 +638,8 @@ qt_ui_element_rep::as_qaction () {
         T1 x = open_box<T1>(static_cast<qt_ui_element_rep*>(help.rep)->load);
         string str = x.x1;
         a->setToolTip (to_qstring (str));
+        // HACK: force displaying of the tooltip (needed for items in the QMenuBar)
+        QObject::connect(a, SIGNAL(hovered()), a, SLOT(showToolTip()));
       }
       return a;
     }
@@ -593,7 +658,7 @@ qt_ui_element_rep::as_qaction () {
 
       QTMAction* a= new QTMAction (NULL);
       string t= tm_var_encode (str);
-      if (t == "Help") t= "Help ";
+      if (t == "Help") t= "Help "; // HACK to avoid MacOS autodetection of the Help menu?
       a->setText(to_qstring (t));
       a->str = str;
       if (style == WIDGET_STYLE_MINI) {
@@ -643,6 +708,8 @@ QTMUIButton::paintEvent(QPaintEvent* event) {
 
 QLayoutItem *
 qt_ui_element_rep::as_qlayoutitem () {
+  //cout << "as_qlayoutitem " << ui_type_string[type] << LF;
+
   switch (type) {
     case horizontal_menu:
     case vertical_menu:
@@ -667,16 +734,16 @@ qt_ui_element_rep::as_qlayoutitem () {
       return l;
     }
       break;
-      
+
+    // a menu rendered as a table of 'cols' columns wide & made up of widgets in
+    // the array 'a'
     case tile_menu: 
     {
       typedef pair<array<widget>, int> T;
       T x = open_box<T>(load);
       array<widget> a = x.x1;
       int cols = x.x2;
-      
-      // a menu rendered as a table of cols columns wide & made up of widgets in a
-      
+            
       QGridLayout* l= new QGridLayout ();
       l->setSizeConstraint (QLayout::SetFixedSize);
       l->setHorizontalSpacing (2);
@@ -692,6 +759,36 @@ qt_ui_element_rep::as_qlayoutitem () {
       return l;
     }
       break;
+
+    //  a table with two columns
+    case aligned_widget:       
+    {
+      typedef quartet<SI, SI, SI, SI> T1;
+      typedef triple<array<widget>, array<widget>, T1 > T;
+      T x = open_box<T>(load);
+      array<widget> lhs = x.x1;
+      array<widget> rhs = x.x2;
+      T1 y = x.x3;
+
+      // FIXME: lpad and rpad ignored.
+      SI hsep = y.x1; SI vsep = y.x2; SI lpad = y.x3; SI rpad = y.x4;
+      
+      if (N(lhs) != N(rhs)) FAILED("aligned_widget: N(lhs) != N(rhs) ");
+
+      QGridLayout* l= new QGridLayout ();
+      l->setSizeConstraint (QLayout::SetMinimumSize);
+      l->setHorizontalSpacing (6+hsep/PIXEL);
+      l->setVerticalSpacing (2+vsep/PIXEL);
+      for (int i=0; i < N(lhs); i++) {
+        QLayoutItem* lli = concrete(lhs[i])->as_qlayoutitem();
+        QLayoutItem* rli = concrete(rhs[i])->as_qlayoutitem();
+        if (lli) l->addItem(lli, i, 0, 1, 1, Qt::AlignRight);
+        if (rli) l->addItem(rli, i, 1, 1, 1, Qt::AlignLeft);
+      }
+      return l;
+    }
+      break;
+      
       
     case minibar_menu: 
     {
@@ -712,10 +809,11 @@ qt_ui_element_rep::as_qlayoutitem () {
     {
       typedef bool T;
       bool vertical = open_box<T> (load);
-      // a horizontal or vertical menu separator
-      (void) vertical;
-      //FIXME: implement h/v
-      return new QSpacerItem(1,1);
+      QSizePolicy::Policy hpolicy = vertical ? QSizePolicy::Fixed 
+                                             : QSizePolicy::Preferred;
+      QSizePolicy::Policy vpolicy = vertical ? QSizePolicy::Preferred 
+                                             : QSizePolicy::Fixed;
+      return new QSpacerItem (1, 1, hpolicy, vpolicy);
     }
       break;
       
@@ -737,6 +835,14 @@ qt_ui_element_rep::as_qlayoutitem () {
     case menu_button:
     case text_widget:
     case xpm_widget:
+    case toggle_widget:
+    case enum_widget:
+    case choice_widget:
+    case scrollable_widget:
+    case hsplit_widget:
+    case vsplit_widget:
+    case tabs_widget:
+    case wrapped_widget:
     {
       QWidget *w = this->as_qwidget();
       return new QWidgetItem(w);
@@ -755,16 +861,26 @@ qt_ui_element_rep::as_qlayoutitem () {
       QLayoutItem* li= concrete(text)->as_qlayoutitem();
       if (li->widget())
       {
-        typedef quartet<string, int, color, bool> T1;
-        T1 x = open_box<T1>(static_cast<qt_ui_element_rep*>(help.rep)->load);
+        typedef quartet<string, int, color, bool> T;
+        T x = open_box<T>(static_cast<qt_ui_element_rep*>(help.rep)->load);
         string str = x.x1;
         li->widget()->setToolTip (to_qstring (str));
       }
       return li;
     }
       break;
-      
-      
+
+    case glue_widget:
+    {
+      typedef quartet<bool, bool, SI, SI> T;
+      T x = open_box<T>(load);
+      QSizePolicy::Policy hpolicy = x.x1 ? QSizePolicy::Preferred 
+                                         : QSizePolicy::Fixed;
+      QSizePolicy::Policy vpolicy = x.x2 ? QSizePolicy::Preferred
+                                         : QSizePolicy::Fixed;
+      return new QSpacerItem (x.x3, x.x4, hpolicy, vpolicy);
+    }
+      break;
     default:
       ;
   }
@@ -772,8 +888,80 @@ qt_ui_element_rep::as_qlayoutitem () {
   return NULL;
 }
 
+/*! Ad-hoc command to be used with toggle widgets.
+ * The command associated with a qt_ui_element::toggle_widget has as a parameter the state
+ * of the QCheckBox. Since it is assumed everywhere else that commands injected into
+ * the gui's queue accept no parameters, and changes would be too big, we choose to
+ * encapsulate the original command in a new one which will execute the first with 
+ * its argument.
+ * \sa qt_ui_element, , qt_ui_element_rep::as_qwidget, qt_ui_element_rep::toggle_widget
+ */
+class qt_toggle_command_rep: public command_rep {
+  QPointer<QCheckBox> qwid;
+  command cmd; 
+
+public:
+  qt_toggle_command_rep(QCheckBox* w, command c) : qwid(w), cmd(c) { }
+  void apply () { if(qwid) cmd (list_object (object (qwid->isChecked()))); }
+  tm_ostream& print (tm_ostream& out) {
+    return out << "Toggle"; }
+};
+
+/*! Ad-hoc command to be used with enum widgets.
+ * The command associated with a qt_ui_element::enum_widget has one parameter. For the
+ * reason to be of this class, see \sa qt_toggle_command_rep .
+ * \sa qt_ui_element, , qt_ui_element_rep::as_qwidget, qt_ui_element_rep::enum_widget
+ */
+class qt_enum_command_rep: public command_rep {
+  QPointer<QComboBox> qwid;
+  command cmd; 
+  
+public:
+  qt_enum_command_rep(QComboBox* w, command c) : qwid(w), cmd(c) {}
+  void apply () { 
+    if (qwid)
+      cmd (list_object (object (from_qstring(qwid->currentText()))));
+  }
+  tm_ostream& print (tm_ostream& out) {
+    return out << "Enum"; }
+};
+
+/*! Ad-hoc command to be used with choice widgets.
+ * The command associated with a qt_ui_element::choice_widget has one parameter. (a
+ * list of selected items).
+ * For the reason to be of this class, see \sa qt_toggle_command_rep.
+ * \sa qt_ui_element, , qt_ui_element_rep::as_qwidget, qt_ui_element_rep::choice_widget
+ */
+class qt_choice_command_rep: public command_rep {
+  QPointer<QListWidget> qwid;
+  command cmd;
+  bool multiple;  //<! Whether multiple choices are allowed in the widget.
+  
+public:
+  qt_choice_command_rep(QListWidget* w, command c, bool m) : qwid(w), cmd(c), multiple(m) {}
+  void apply () { 
+    if (qwid) {
+      QList<QListWidgetItem*> items = qwid->selectedItems();
+      array<string> selected;
+      for(int i = 0; i < items.size(); ++i)
+        selected << from_qstring (items[i]->text());
+      object l= null_object ();
+      if(multiple)
+        for (int i = N(selected)-1; i >= 0; --i)
+          l= cons (selected[i], l);
+      else if(N(selected)>0)  //Do not return a list with the item if only one
+        l= selected[0];
+      cmd (list_object (l));
+    }
+  }
+  tm_ostream& print (tm_ostream& out) { return out << "Choice"; }
+};
+
+
 QWidget *
 qt_ui_element_rep::as_qwidget () {
+  //cout << "as_qwidget " << ui_type_string[type] << LF;
+
   switch (type) {
     case horizontal_menu:
     case vertical_menu:
@@ -781,21 +969,24 @@ qt_ui_element_rep::as_qwidget () {
     case vertical_list:
     case tile_menu: 
     case minibar_menu: 
+    case aligned_widget: 
     {
       QLayoutItem *li = this->as_qlayoutitem();
       QWidget *w = new QWidget();
       if (QLayout *l = li->layout()) {
+        // note that the QLayout is the same object as the QLayoutItem 
+        // so no need to free li
         w->setLayout(l);
       } else {
         cout << "qt_ui_element_rep::as_qwidget : invalid situation" << LF;
       }
-      delete li;
       return w;
     }
       break;
       
     case menu_separator: 
-    case menu_group: 
+    case menu_group:
+    case glue_widget:
     {
       return NULL;
     }
@@ -865,10 +1056,8 @@ qt_ui_element_rep::as_qwidget () {
     {
       typedef quartet<string, int, color, bool> T;
       T x = open_box<T>(load);
-      string str = x.x1;
-      int style = x.x2;
-      //color col = x.x3;
-      //bool tsp = x.x4;
+      string str    = x.x1;
+      QString style = to_qstylesheet(x.x2, x.x3);
       
       // a text widget with a given color and transparency
       QLabel *w = new QLabel();
@@ -879,12 +1068,9 @@ qt_ui_element_rep::as_qwidget () {
 #endif
       string t= tm_var_encode (str);
       if (t == "Help") t= "Help ";
+      //w->setTextFormat(Qt::RichText);
       w->setText(to_qstring (t));
-      if (style == WIDGET_STYLE_MINI) {
-        QFont f = w->font();
-        f.setPointSize(10);
-        w->setFont(f);
-      }
+      w->setStyleSheet(style);
       return w;
     }
       break;
@@ -902,6 +1088,172 @@ qt_ui_element_rep::as_qwidget () {
       return l;
     }
       break;
+
+    case toggle_widget:
+    { 
+      typedef triple<command, bool, int > T;
+      T x = open_box<T>(load);
+      command cmd = x.x1;
+      bool check  = x.x2;
+      QString style = to_qstylesheet(x.x3);
+      
+      QCheckBox* w  = new QCheckBox (NULL);  
+      w->setCheckState(check ? Qt::Checked : Qt::Unchecked);
+      w->setStyleSheet(style);
+      command tcmd = tm_new<qt_toggle_command_rep> (w, cmd);
+      QTMCommand* c = new QTMCommand (tcmd);
+      c->setParent (w);
+      QObject::connect (w, SIGNAL (stateChanged(int)), c, SLOT (apply()));
+
+      return w;
+    }
+      break;
+      
+    case enum_widget:
+    {
+      typedef quintuple<command, array<string>, string, int, string> T;
+      T x = open_box<T>(load);
+      command cmd        = x.x1;
+      QStringList values = to_qstringlist(x.x2);
+      QString value      = to_qstring(x.x3);
+      QString style      = to_qstylesheet(x.x4);
+            
+      QComboBox* w = new QComboBox(NULL);
+      w->setEditable(value.isEmpty() || values.last().isEmpty());  // weird convention?!
+      if (values.last().isEmpty())
+        values.removeLast();
+      
+      w->addItems(values);
+      int index = w->findText(value, Qt::MatchFixedString | Qt::MatchCaseSensitive);
+      if (index != -1)
+        w->setCurrentIndex(index);
+   
+      // The QComboBox must be already filled to calculate relative widths
+      QSize size= qt_decode_length(x.x5, w);
+      w->setMinimumSize(size);
+      // QComboBox::AdjustToContentsOnFirstShow would fix the size. Better?
+      w->setSizeAdjustPolicy(QComboBox::AdjustToContents);
+      w->setStyleSheet(style);
+      
+      command ecmd = tm_new<qt_enum_command_rep> (w, cmd);
+      QTMCommand* c = new QTMCommand (ecmd);
+      c->setParent (w);
+      // NOTE: with QueuedConnections, the slots are sometimes not invoked.
+      QObject::connect (w, SIGNAL (currentIndexChanged(int)), c, SLOT (apply()));
+      
+      return w;
+    }
+      break;
+      
+    case choice_widget:
+    {
+      typedef quartet<command, array<string>, array<string>, bool > T;
+      T x = open_box<T>(load);
+      command cmd = x.x1;
+      QStringList items  = to_qstringlist(x.x2);
+      QStringList chosen = to_qstringlist(x.x3);
+      bool multiple_sel  = x.x4;
+      
+      QListWidget* w = new QListWidget();
+      w->addItems(items);
+
+      if (multiple_sel)
+        w->setSelectionMode(QAbstractItemView::ExtendedSelection);  // Support CTRL and SHIFT multiple selections.
+      else
+        w->setSelectionMode(QAbstractItemView::SingleSelection);
+      
+      for (int i = 0; i < chosen.size(); ++i) {
+        QListWidgetItem* item = w->item(i);
+        item->setSelected(chosen.contains(item->text(), Qt::CaseSensitive));  // Qt::CaseSensitive is the default anyway
+      }
+      
+      command ecmd = tm_new<qt_choice_command_rep> (w, cmd, multiple_sel);
+      QTMCommand* qcmd = new QTMCommand (ecmd);
+      qcmd->setParent (w);
+      QObject::connect (w, SIGNAL (itemSelectionChanged()), qcmd, SLOT (apply()));//, Qt::QueuedConnection);
+      
+      return w;      
+    }
+      break;
+      
+    case scrollable_widget:
+    {
+      typedef pair<widget, int> T;
+      T x = open_box<T>(load);
+      widget wid    = x.x1;
+      QString style = to_qstylesheet(x.x2);
+      
+      QScrollArea* scroll = new QScrollArea();
+      scroll->setBackgroundRole(QPalette::NoRole);
+      scroll->setStyleSheet(style);
+      QWidget* w = concrete(wid)->as_qwidget();
+      scroll->setWidget(w);
+    
+      // FIXME????
+      // "Note that You must add the layout of widget before you call this function; 
+      //  if you add it later, the widget will not be visible - regardless of when you show() the scroll area.
+      //  In this case, you can also not show() the widget later."
+      return scroll;
+    }
+      break;
+      
+    case hsplit_widget:
+    case vsplit_widget:
+    {
+      typedef pair<widget, widget> T;
+      T x = open_box<T>(load);
+      widget w1 = x.x1;
+      widget w2 = x.x2;
+      
+      QWidget* qw1 = concrete(w1)->as_qwidget();
+      QWidget* qw2 = concrete(w2)->as_qwidget();
+      QSplitter* split = new QSplitter();
+      split->setOrientation(type == hsplit_widget ? Qt::Horizontal : Qt::Vertical);
+      split->addWidget(qw1);
+      split->addWidget(qw2);
+      
+      return split;
+    }
+      break;
+
+    case tabs_widget:
+    {
+      typedef pair<array<widget>, array<widget> > T;
+      T x = open_box<T>(load);
+      array<widget> tabs = x.x1;
+      array<widget> bodies = x.x2;
+      
+      QTMTabWidget* tw = new QTMTabWidget ();
+      
+      int i;
+      for (i = 0; i < N(tabs); i++) {
+        if (is_nil (tabs[i])) break;
+        QWidget* prelabel = concrete (tabs[i]) -> as_qwidget();
+        QLabel* label = qobject_cast<QLabel*> (prelabel);
+        QWidget* body = concrete (bodies[i]) -> as_qwidget();
+        tw->addTab(body, label ? label->text() : "");
+        delete prelabel;
+      }
+      if (i>0) tw->resizeOthers(0);   // Force the automatic resizing
+
+      return tw;
+    }
+
+    case wrapped_widget:
+    {
+      typedef pair<widget, command> T;
+      T x = open_box<T>(load);
+      widget w = x.x1;
+      command cmd = x.x2;
+      
+      QWidget* qw = concrete(w)->as_qwidget();
+      QTMCommand* c = new QTMCommand (cmd);
+      c->setParent (qw);
+      QObject::connect (qw, SIGNAL (destroyed()), c, SLOT (apply()));//, Qt::QueuedConnection);
+      
+      return qw;
+    }
+      break;
       
     default:
       ;
@@ -911,18 +1263,26 @@ qt_ui_element_rep::as_qwidget () {
 }
 
 
-
-
 /******************************************************************************
- * Widgets for the construction of menus and dialogs
- ******************************************************************************/
+* Widgets for the construction of menus and dialogs
+******************************************************************************/
 
 // TeXmacs interface
 
+
+
 widget horizontal_menu (array<widget> arr) { return qt_ui_element_rep::create (qt_ui_element_rep::horizontal_menu, arr); }
 widget vertical_menu (array<widget> arr)  { return qt_ui_element_rep::create (qt_ui_element_rep::vertical_menu, arr); }
 widget horizontal_list (array<widget> arr) { return qt_ui_element_rep::create (qt_ui_element_rep::horizontal_list, arr); }
 widget vertical_list (array<widget> arr) { return qt_ui_element_rep::create (qt_ui_element_rep::vertical_list, arr); }
+widget aligned_widget (array<widget> lhs, array<widget> rhs, SI hsep, SI vsep, SI lpad, SI rpad) { 
+  typedef quartet<SI, SI, SI, SI> T1;
+  typedef triple<array<widget>, array<widget>, T1> T;
+  return tm_new <qt_ui_element_rep> (qt_ui_element_rep::aligned_widget, 
+                                     close_box (T (lhs,rhs, T1 (hsep, vsep, lpad, rpad)))); 
+}
+widget tabs_widget (array<widget> tabs, array<widget> bodies) { return qt_ui_element_rep::create (qt_ui_element_rep::tabs_widget, tabs, bodies); }
+widget wrapped_widget (widget w, command cmd) { return qt_ui_element_rep::create (qt_ui_element_rep::wrapped_widget, w, cmd); }
 widget tile_menu (array<widget> a, int cols) { return qt_ui_element_rep::create (qt_ui_element_rep::tile_menu, a, cols); }
 widget minibar_menu (array<widget> arr) { return qt_ui_element_rep::create (qt_ui_element_rep::minibar_menu, arr); }
 widget menu_separator (bool vertical) { return qt_ui_element_rep::create (qt_ui_element_rep::menu_separator, vertical); }
@@ -933,4 +1293,25 @@ widget menu_button (widget w, command cmd, string pre, string ks, int style) { r
 widget balloon_widget (widget w, widget help) { return qt_ui_element_rep::create (qt_ui_element_rep::balloon_widget, w, help); }
 widget text_widget (string s, int style, color col, bool tsp) { return qt_ui_element_rep::create (qt_ui_element_rep::text_widget, s, style, col, tsp); }
 widget xpm_widget (url file_name) { return qt_ui_element_rep::create (qt_ui_element_rep::xpm_widget, file_name); }
-
+widget toggle_widget (command cmd, bool on, int style) { return qt_ui_element_rep::create (qt_ui_element_rep::toggle_widget, cmd, on, style); }
+widget enum_widget (command cmd, array<string> vals, string val, int style, string width) { return qt_ui_element_rep::create (qt_ui_element_rep::enum_widget, cmd, vals, val, style, width); }
+widget choice_widget (command cmd, array<string> vals, array<string> chosen) { return qt_ui_element_rep::create(qt_ui_element_rep::choice_widget, cmd, vals, chosen, true); }
+widget choice_widget (command cmd, array<string> vals, string cur) {
+  array<string> chosen (1);
+  chosen[0]= cur;
+  return qt_ui_element_rep::create(qt_ui_element_rep::choice_widget, cmd, vals, chosen, false); }
+widget user_canvas_widget (widget wid, int style) { return qt_ui_element_rep::create(qt_ui_element_rep::scrollable_widget, wid, style); }
+widget resize_widget (widget w, int style, string w1, string h1,
+                      string w2, string h2, string w3, string h3) {
+  (void) w; (void) style; (void) w1; (void) h1; (void) w2; (void) h2; (void) w3; (void) h3;
+  //FIXME: add a meaningul semantics
+  return w;
+}
+widget hsplit_widget (widget l, widget r) { return qt_ui_element_rep::create(qt_ui_element_rep::hsplit_widget, l, r); }
+widget vsplit_widget (widget t, widget b) { return qt_ui_element_rep::create(qt_ui_element_rep::vsplit_widget, t, b); }
+widget ink_widget (command cb) {
+  (void) cb;
+  FAILED ("not yet implemented"); }
+widget refresh_widget (string tmwid) {
+  (void) tmwid;
+  FAILED ("not yet implemented"); }
