@@ -1,6 +1,6 @@
 
 /******************************************************************************
-* MODULE     : qt_utilities.mm
+* MODULE     : qt_utilities.cpp
 * DESCRIPTION: Utilities for QT
 * COPYRIGHT  : (C) 2007  Massimiliano Gubinelli
 *******************************************************************************
@@ -18,19 +18,30 @@
 #include <QLocale>
 #include <QDateTime>
 #include <QTextCodec>
+#include <QHash>
+#include <QStringList>
 
 #include <QPrinter>
 #include <QPrintDialog>
 
-
+#include "rgb_colors.hpp"
 #include "dictionary.hpp"
 #include "converter.hpp"
 #include "language.hpp"
 
+#include "qt_gui.hpp"    // gui_maximal_extents()
+
 #ifdef USE_GS
 #include "Ghostscript/gs_utilities.hpp"
 #endif
 
+/*! some debugging infrastucture */
+tm_ostream&
+operator << (tm_ostream& out, QRect rect) {
+  return out << "(" << rect.x() << "," << rect.y() << ","
+  << rect.width() << "," << rect.height() << ")";
+}
+
 QRect
 to_qrect (const coord4 & p) {
   float c= 1.0/PIXEL;
@@ -49,6 +60,83 @@ to_qsize (const coord2 & p) {
   return QSize (p.x1*c, p.x2*c);
 }
 
+
+QString
+parse_tm_style(int style) {
+  QString sheet;
+  if (style & WIDGET_STYLE_MINI)  // use smaller text font inside widget
+    sheet += "font-size: 10pt;";
+  if (style & WIDGET_STYLE_MONOSPACED)  // use monospaced font inside widget
+    sheet += "font-family: \"monospace\";";
+  if (style & WIDGET_STYLE_GREY)  // use grey text font
+    sheet += "font-color: #eeeeee";
+  if (style & WIDGET_STYLE_PRESSED)   // indicate that a button is currently pressed
+    sheet += "";
+  if (style & WIDGET_STYLE_INERT)  // only render but don't associate any action to widget
+    sheet += "";
+  if (style & WIDGET_STYLE_BUTTON)  // indicate that a button should explicitly rendered as a button
+    sheet += "";
+  if (style & WIDGET_STYLE_CENTERED)  // use centered text
+    sheet += "text-align: center;";
+  if (style & WIDGET_STYLE_BOLD)
+    sheet += "font-weight: bold;";
+  return sheet;
+}
+
+/*! */
+QString
+to_qstylesheet(int style) {
+  return "* {" + parse_tm_style(style) + "}";
+}
+
+QString
+to_qstylesheet(int style, color c) {
+  int r,g,b,a;
+  get_rgb_color(c, r, g, b, a);
+  a = a*100/255;
+  return "* {" + parse_tm_style(style)
+    + QString("color: rgba(%1, %2, %3, %4%);").arg(r).arg(g).arg(b).arg(a)
+    + "}";
+}
+
+
+/*! Try to convert a TeXmacs lenght (em, px, w, h) in a QSize.
+ *
+ * Uses the widget current size to compute relative sizes as specified with "1w"
+ * Should not affect the widget size in that particular case.
+ * FIXME: should we use the constant PIXEL somewhere? 
+ */
+QSize
+qt_decode_length (string width, QWidget* qwid) {
+  QSize size;
+  if (qwid)
+    size= qwid->minimumSizeHint();
+  else 
+    gui_maximal_extents (size.rwidth(), size.rheight());
+  
+  if (ends (width, "w") && is_double (width (0, N(width) - 1))) {
+    double x= as_double (width (0, N(width) - 1));
+    size.rwidth() *= x;
+  }
+  else if (ends (width, "h") && is_double (width (0, N(width) - 1))) {
+    double y= as_double (width (0, N(width) - 1));
+    size.rheight() *= y;
+  }
+  else if (ends (width, "em") && is_double (width (0, N(width) - 2))) {
+    double x= as_double (width (0, N(width) - 2));
+    if (qwid) {
+      size.setWidth(x * qwid->fontInfo().pointSize()); 
+    } else {
+      size.setWidth(x * QApplication::font().pointSize());
+    }
+  }
+  else if (ends (width, "px") && is_double (width (0, N(width) - 2))) {
+    double x= as_double (width (0, N(width) - 2));
+    size.setWidth(x);
+  }
+  return size;
+}
+
 coord4
 from_qrect (const QRect & rect) {
   SI c1, c2, c3, c4;
@@ -75,23 +163,24 @@ from_qsize (const QSize & s) {
   return coord2 (c1, c2);
 }
 
+QStringList
+to_qstringlist(array<string> l) {
+  QStringList ql;
+  for(int i=0; i<N(l); ++i)
+    ql << to_qstring(l[i]);
+  return ql;
+}
+
+array<string>
+from_qstringlist(const QStringList& l) {
+  array<string> tl (l.size());
+  for(QStringList::const_iterator it = l.begin(); it != l.end(); ++it)
+    tl << from_qstring(*it);
+  return tl;
+}
+
 QString
 to_qstring (string s) {
-  char* p= as_charp (s);
-  QString nss (p);
-  tm_delete_array (p);
-  return nss;
-}
-
-string
-from_qstring (const QString &s) {
-  QByteArray arr= s.toUtf8 ();
-  const char* cstr= arr.constData ();
-  return utf8_to_cork (string ((char*) cstr));
-}
-
-QString
-to_qstring_utf8 (string s) {
   string out_lan= get_output_language ();
   if ((out_lan == "bulgarian") || 
       (out_lan == "russian") ||
@@ -106,6 +195,73 @@ to_qstring_utf8 (string s) {
   return nss;
 }
 
+QString
+utf8_to_qstring (string s) {
+  char* p= as_charp (s);
+  QString nss= QString::fromUtf8 (p, N(s));
+  tm_delete_array (p);
+  return nss;
+}
+
+string
+from_qstring_utf8 (const QString &s) {
+  QByteArray arr= s.toUtf8 ();
+  const char* cstr= arr.constData ();
+  return string ((char*) cstr);
+}
+
+string
+from_qstring (const QString &s) {
+  return utf8_to_cork (from_qstring_utf8(s));
+}
+
+// Although slow to build, this should provide better lookup times than
+// linearly traversing the array of colors.
+static QHash<QString, QColor> _NamedColors;
+
+/*!
+ * This needn't be called more than once. Takes RGBColors, defined in
+ * rgb_colors.hpp and initializes our QHash
+ */
+void initNamedColors(void) {
+  for(int i = 0; i < RGBColorsSize; ++i)
+    _NamedColors.insert(QString(RGBColors[i].name), 
+                        QColor(RGBColors[i].r, RGBColors[i].g, RGBColors[i].b));
+}
+
+/*!
+ * Takes either an hexadecimal RGB color, as in #e3a1ff, or a named color
+ * as those defined in src/Graphics/Renderer/rgb_colors.hpp and returns a QColor
+ */
+QColor
+to_qcolor (const string& col) {
+  QString _col = to_qstring(col);
+  if(_col.startsWith("#"))
+    return QColor(_col);
+  if(_NamedColors.isEmpty())
+    initNamedColors();
+  if(_NamedColors.contains(_col))
+    return _NamedColors[_col];
+  if(DEBUG_QT)
+    cout << "to_qcolor(" << col << "): name is not defined in RGBColors.\n";
+  return QColor(100,100,100);  // FIXME? 
+}
+
+/*! Returns a color encoded as a string with hexadecimal RGB values, as in #e3a1ff */
+string
+from_qcolor (const QColor& col) {
+  return from_qstring(col.name());
+}
+
+// FIXME: Unnecessary? QColor::setRgba() does this.
+QColor
+to_qcolor(color c) {
+  int r, g, b, a;
+  get_rgb_color (c, r, g, b, a);
+  return QColor (r, g, b, a);
+}
+
+
 string
 qt_translate (string s) {
   string in_lan= get_input_language ();
@@ -114,6 +270,7 @@ qt_translate (string s) {
   return tm_var_encode (translate (s, in_lan, out_lan));
 }
 
+//FIXME!?!?
 bool
 qt_supports (url u) {
   string s= suffix (u);
@@ -123,9 +280,11 @@ qt_supports (url u) {
 
 void
 qt_image_size (url image, int& w, int& h) {
-  QImage im= QImage (to_qstring (concretize (image)));
+  //cout <<  concretize (image) << LF;
+  QImage im= QImage (utf8_to_qstring (concretize (image)));
   if (im.isNull ()) {
-    cerr << "Cannot read image file '" << image << "'" << LF;
+    cerr << "TeXmacs] cannot read image file '" << image << "'" 
+	 << " in qt_image_size" << LF;
     w= 35; h= 35;
   }
   else {
@@ -136,23 +295,24 @@ qt_image_size (url image, int& w, int& h) {
 
 void
 qt_convert_image (url image, url dest, int w, int h) {
-  QImage im (to_qstring (concretize (image)));
-  if (im.isNull ()) {
-    cerr << "Cannot read image file '" << image << "'" << LF;
-  }
+  QImage im (utf8_to_qstring (concretize (image)));
+  if (im.isNull ())
+    cerr << "TeXmacs] cannot read image file '" << image << "'" 
+         << " in qt_convert_image" << LF;
   else {
-    if (w > 0 && h > 0) im= im.scaled (w, h, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-    im.scaled (w, h).save (to_qstring (concretize (dest)));
+    if (w > 0 && h > 0) 
+      im= im.scaled (w, h, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+    im.scaled (w, h).save (utf8_to_qstring (concretize (dest)));
   }
 }
 
 void
 qt_image_to_eps (url image, url eps, int w_pt, int h_pt, int dpi) {
   static const char* d= "0123456789ABCDEF";
-  QImage im (to_qstring (concretize (image)));
-  if (im.isNull ()) {
-    cerr << "Cannot read image file '" << image << "'" << LF;
-  }
+  QImage im (utf8_to_qstring (concretize (image)));
+  if (im.isNull ())
+    cerr << "TeXmacs Cannot read image file '" << image << "'"
+	 << " in qt_image_to_eps" << LF;
   else {
     if (dpi > 0 && w_pt > 0 && h_pt > 0) {
       int ww= w_pt * dpi / 72;
@@ -235,6 +395,8 @@ qt_get_date (string lan, string fm) {
   return from_qstring(date);
 }
 
+#ifndef _MBD_EXPERIMENTAL_PRINTER_WIDGET  // this is in qt_printer_widget
+
 #define PAPER(fmt)  case QPrinter::fmt : return "fmt"
 static string 
 qt_papersize_to_string( QPrinter::PaperSize sz ) {
@@ -268,8 +430,10 @@ qt_papersize_to_string( QPrinter::PaperSize sz ) {
 }
 #undef PAPER
 
+
 bool 
-qt_print (bool& to_file, bool& landscape, string& pname, url& filename, string& first, string& last, string& paper_type) {
+qt_print (bool& to_file, bool& landscape, string& pname, url& filename, 
+          string& first, string& last, string& paper_type) {
   static QPrinter *qprinter = NULL;
   if (!qprinter) {
     qprinter = new QPrinter;
@@ -292,3 +456,4 @@ qt_print (bool& to_file, bool& landscape, string& pname, url& filename, string&
   return false;
 }
 
+#endif //_MBD_EXPERIMENTAL_PRINTER_WIDGET
