@@ -71,6 +71,11 @@ concrete (promise<widget> pw) {
 * Exported special widgets
 ******************************************************************************/
 
+widget
+extend (widget w, array<widget> a) {
+  return abstract (extend (concrete (w), concrete (a)));
+}
+
 widget
 horizontal_list (array<widget> a) {
   return abstract (horizontal_list (concrete (a)));
@@ -81,9 +86,22 @@ vertical_list (array<widget> a) {
   return abstract (vertical_list (concrete (a)));
 }
 
+widget
+aligned_widget (array<widget> lhs, array<widget> rhs,
+                SI hsep, SI vsep, SI lpad, SI rpad) {
+  return abstract (aligned_widget (concrete (lhs), concrete (rhs),
+                                   hsep, vsep, lpad, rpad));
+}
+
+widget
+tabs_widget (array<widget> tabs, array<widget> bodies) {
+  return abstract (tabs_widget (concrete (tabs), concrete (bodies)));
+}
+
 widget
 horizontal_menu (array<widget> a) {
-  return abstract (horizontal_array (concrete (a), -1));
+  return abstract (horizontal_list (concrete (a)));
+  //return abstract (horizontal_array (concrete (a), -1));
 }
 
 widget
@@ -96,6 +114,16 @@ tile_menu (array<widget> a, int cols) {
   return abstract (tile (concrete (a), cols));
 }
 
+widget
+minibar_widget (widget w) {
+  return abstract (minibar_widget (concrete (w)));
+}
+
+widget
+minibar_menu (array<widget> a) {
+  return minibar_widget (horizontal_menu (a));
+}
+
 widget
 switch_widget (array<widget> a, array<string> name, int init) {
   return abstract (switch_widget (concrete (a), name, init));
@@ -106,6 +134,11 @@ optional_widget (widget w, bool on) {
   return abstract (optional_widget (concrete (w), on));
 }
 
+widget
+wrapped_widget (widget w, command cmd) {
+  return abstract (wrapped_widget (concrete (w), cmd));
+}
+
 widget
 empty_widget () {
   return abstract (glue_wk_widget (false, false, 0, 0));
@@ -116,24 +149,24 @@ glue_widget (bool hx, bool vx, SI w, SI h) {
   return abstract (glue_wk_widget (hx, vx, w, h));
 }
 
+widget
+glue_widget (tree col, bool hx, bool vx, SI w, SI h) {
+  return abstract (glue_wk_widget (col, hx, vx, w, h));
+}
+
 widget
 menu_separator (bool vert) {
   return abstract (separator_wk_widget (2*PIXEL, 2*PIXEL, vert));
 }
 
 widget
-text_widget (string s, bool tsp) {
-  return abstract (text_wk_widget (s, tsp));
+menu_text_widget (string s, int style, color col, bool tsp, bool tt) {
+  return abstract (menu_text_wk_widget (s, style, col, tsp, tt));
 }
 
 widget
-menu_text_widget (string s, color col, bool tsp, bool tt) {
-  return abstract (menu_text_wk_widget (s, col, tsp, tt));
-}
-
-widget
-text_widget (string s, color col, bool tsp) {
-  return menu_text_widget (s, col, tsp, false);
+text_widget (string s, int style, color col, bool tsp) {
+  return menu_text_widget (s, style, col, tsp, false);
 }
 
 widget
@@ -142,31 +175,34 @@ xpm_widget (url file_name) {
 }
 
 widget
-command_button (widget w, command cmd, bool button_flag) {
-  return abstract (command_button (concrete (w), cmd, button_flag));
+command_button (widget w, command cmd, int style) {
+  return abstract (command_button (concrete (w), cmd, style));
 }
 
 widget
-command_button (widget lw, widget cw, widget rw, command cmd, bool e, bool c) {
+command_button (widget lw, widget cw, widget rw, command cmd, int style) {
   return abstract (command_button (concrete (lw), concrete (cw),
-				   concrete (rw), cmd, e, c));
+				   concrete (rw), cmd, style));
 }
 
 widget
-menu_group (string name) {
+menu_group (string name, int style) {
   widget lw= empty_widget ();
-  widget cw= text_widget (name, dark_grey, false);
+  widget cw= text_widget (name, style, dark_grey, false);
   widget rw= empty_widget ();
-  return command_button (lw, cw, rw, noop, false, true);
+  return command_button (lw, cw, rw, noop,
+			 WIDGET_STYLE_INERT | WIDGET_STYLE_CENTERED);
 }
 
 widget
-menu_button (widget w, command cmd, string pre, string ks, bool ok) {
-  if (pre == "" && ks == "") return command_button (w, cmd, false);
+menu_button (widget w, command cmd, string pre, string ks, int style) {
+  bool ok= (style & WIDGET_STYLE_INERT) == 0;
+  if (pre == "" && ks == "")
+    return command_button (w, cmd, style);
   else {
     color  c = ok? black: dark_grey;
     widget lw= empty_widget ();
-    widget rw= menu_text_widget (ks, c, true, true);
+    widget rw= menu_text_widget (ks, 0, c, true, true);
     if (pre != "") {
       string s= "";
       if (pre == "v") s= "<checked>";
@@ -174,7 +210,7 @@ menu_button (widget w, command cmd, string pre, string ks, bool ok) {
       if (pre == "*") s= "<bullet>";
       if (s != "") lw= box_widget (tree (TUPLE), s, c, true, false);
     }
-    return command_button (lw, w, rw, cmd, ok, false);
+    return command_button (lw, w, rw, cmd, style);
   }
 }
 
@@ -188,6 +224,11 @@ pullright_button (widget w, promise<widget> pw) {
   return abstract (pullright_button (concrete (w), concrete (pw)));
 }
 
+widget
+toggle_widget (command cmd, bool on, int style) {
+  return abstract (toggle_wk_widget (cmd, on, style));
+}
+
 widget
 popup_widget (widget w) {
   return abstract (popup_widget (concrete (w), center));
@@ -195,12 +236,34 @@ popup_widget (widget w) {
 
 widget
 canvas_widget (widget w) {
-  return abstract (canvas_widget (concrete (w), north_west));
+  return abstract (canvas_widget (concrete (w), north_west, true));
 }
 
 widget
-input_text_widget (command call_back, string type, array<string> def) {
-  return abstract (input_text_wk_widget (call_back, type, def));
+user_canvas_widget (widget wid, int style) {
+  return abstract (user_canvas_widget (concrete (wid), style));
+}
+
+widget
+resize_widget (widget w, int style, string w1, string h1,
+               string w2, string h2, string w3, string h3) {
+  return abstract (resize_widget (concrete (w), style, w1, h1, w2, h2, w3, h3));
+}
+
+widget
+hsplit_widget (widget l, widget r) {
+  return abstract (hsplit_widget (concrete (l), concrete (r)));
+}
+
+widget
+vsplit_widget (widget t, widget b) {
+  return abstract (vsplit_widget (concrete (t), concrete (b)));
+}
+
+widget
+input_text_widget (command call_back, string type, array<string> def,
+		   int style, string width) {
+  return abstract (input_text_wk_widget (call_back, type, def, style, width));
 }
 
 widget
@@ -209,8 +272,33 @@ inputs_list_widget (command call_back, array<string> prompts) {
 }
 
 widget
-file_chooser_widget (command cmd, string type, string mgn) {
-  return abstract (file_chooser_wk_widget (cmd, type, mgn));
+enum_widget (command cb, array<string> vals, string v, int style, string w) {
+  return abstract (enum_wk_widget (cb, vals, v, style, w));
+}
+
+widget
+choice_widget (command cb, array<string> vals, string v) {
+  return abstract (choice_wk_widget (cb, vals, v));
+}
+
+widget
+choice_widget (command cb, array<string> vals, array<string> mc) {
+  return abstract (choice_wk_widget (cb, vals, mc));
+}
+
+widget
+file_chooser_widget (command cmd, string type, bool save) {
+  return abstract (file_chooser_wk_widget (cmd, type));
+}
+
+widget
+printer_widget (command cmd, url u) {
+  return menu_button (text_widget ("Cancel", 0, black), cmd, "", "", 0);
+}
+
+widget
+color_picker_widget (command cmd, bool bg, array<tree> proposals) {
+  return abstract (color_picker_wk_widget (cmd, bg, proposals));
 }
 
 widget
@@ -223,14 +311,24 @@ wait_widget (SI w, SI h, string message) {
   return abstract (wait_wk_widget (w, h, message));
 }
 
+widget
+ink_widget (command cb) {
+  return abstract (ink_wk_widget (cb));
+}
+
+widget
+refresh_widget (string tmwid) {
+  return abstract (refresh_wk_widget (tmwid));
+}
+
 widget
 texmacs_widget (int mask, command quit) {
   return abstract (texmacs_wk_widget (mask, quit));
 }
 
 widget
-plain_window_widget (widget wid, string s) {
-  return abstract (plain_window_widget (concrete (wid), s));
+plain_window_widget (widget wid, string s, command quit) {
+  return abstract (plain_window_widget (concrete (wid), s, quit));
 }
 
 widget
@@ -275,12 +373,17 @@ check_type (blackbox bb, string s) {
 SI get_dx (gravity grav, SI w);
 SI get_dy (gravity grav, SI h);
 
+bool
+bad_parent (wk_widget wid) {
+  return wid->win != NULL && wid != concrete (wid->win->get_widget ()) [0];
+}
+
 void
 principal_widget_check (wk_widget wid) {
   // FIXME: Positions should really be computed relative to parent widgets.
   // Currently, we only allow geometry access of the unique child of
   // a window widget.
-  if (wid->win != NULL && wid != concrete (wid->win->get_widget ()) [0]) {
+  if (bad_parent (wid)) {
     cerr << "Widget= " << wid << "\n";
     FAILED ("invalid geometry access");
   }
@@ -293,7 +396,12 @@ set_geometry (wk_widget wid, SI x, SI y, SI w, SI h) {
     wid->win->set_size (w, h);
   }
   else {
-    principal_widget_check (wid); // FIXME: we should use parent's coordinates
+    //principal_widget_check (wid);// FIXME: we should use parent's coordinates
+    if (bad_parent (wid)) {
+      SI dx, dy;
+      wid->win->get_position (dx, dy);
+      x -= dx; y -= dy;
+    }
     wid << emit_position (x, y, w, h, north_west);
   }
 }
@@ -303,13 +411,21 @@ get_geometry (wk_widget wid, SI& x, SI& y, SI& w, SI& h) {
   if (wid->is_window_widget ()) {
     wid->win->get_position (x, y);
     wid->win->get_size (w, h);
+    //cout << "Size == " << (w>>8) << ", " << (h>>8) << "\n";
   }
   else {
-    principal_widget_check (wid); // FIXME: we should use parent's coordinates
+    //principal_widget_check (wid);// FIXME: we should use parent's coordinates
     x= wid->ox - get_dx (wid->grav, wid->w);
     y= wid->oy - get_dy (wid->grav, wid->h);
     w= wid->w;
     h= wid->h;
+
+    if (bad_parent (wid)) {
+      SI dx, dy;
+      wid->win->get_position (dx, dy);
+      x += dx; y += dy;
+    }
+    //cout << "Size " << ((tree) wid) << " := " << (w>>8) << ", " << (h>>8) << "\n";
   }
 }
 
@@ -383,6 +499,12 @@ send_update (wk_widget w, blackbox val) {
   w << emit_update ();
 }
 
+void
+send_refresh (wk_widget w, blackbox val) {
+  ASSERT (is_nil (val), "type mismatch");
+  w << emit_refresh ();
+}
+
 void
 send_keyboard (wk_widget w, blackbox val) {
   typedef pair<string,time_t> keypress;
@@ -486,6 +608,9 @@ wk_widget_rep::send (slot s, blackbox val) {
   case SLOT_UPDATE:
     send_update (THIS, val);
     break;
+  case SLOT_REFRESH:
+    send_refresh (THIS, val);
+    break;
   case SLOT_KEYBOARD:
     send_keyboard (THIS, val);
     break;
@@ -540,12 +665,18 @@ wk_widget_rep::send (slot s, blackbox val) {
   case SLOT_MAIN_ICONS_VISIBILITY:
     send_bool (THIS, "main icons", val);
     break;
-  case SLOT_CONTEXT_ICONS_VISIBILITY:
-    send_bool (THIS, "context icons", val);
+  case SLOT_MODE_ICONS_VISIBILITY:
+    send_bool (THIS, "mode icons", val);
+    break;
+  case SLOT_FOCUS_ICONS_VISIBILITY:
+    send_bool (THIS, "focus icons", val);
     break;
   case SLOT_USER_ICONS_VISIBILITY:
     send_bool (THIS, "user icons", val);
     break;
+  case SLOT_SIDE_TOOLS_VISIBILITY:
+    send_bool (THIS, "side tools", val);
+    break;
   case SLOT_FOOTER_VISIBILITY:
     send_bool (THIS, "footer flag", val);
     break;
@@ -646,12 +777,14 @@ query_position (wk_widget w, int type_id) {
 blackbox
 query_keyboard_focus (wk_widget w, int type_id) {
   ASSERT (type_id == type_helper<bool>::id, "type mismatch");
+  if (w->win == NULL) return close_box<bool> (false);
   return close_box<bool> (w->win->get_keyboard_focus (abstract (w)));
 }
 
 blackbox
 query_mouse_grab (wk_widget w, int type_id) {
   ASSERT (type_id == type_helper<bool>::id, "type mismatch");
+  if (w->win == NULL) return close_box<bool> (false);
   return close_box<bool> (w->win->get_mouse_grab (abstract (w)));
 }
 
@@ -688,10 +821,14 @@ wk_widget_rep::query (slot s, int type_id) {
     return query_bool (THIS, "header", type_id);
   case SLOT_MAIN_ICONS_VISIBILITY:
     return query_bool (THIS, "main icons", type_id);
-  case SLOT_CONTEXT_ICONS_VISIBILITY:
-    return query_bool (THIS, "context icons", type_id);
+  case SLOT_MODE_ICONS_VISIBILITY:
+    return query_bool (THIS, "mode icons", type_id);
+  case SLOT_FOCUS_ICONS_VISIBILITY:
+    return query_bool (THIS, "focus icons", type_id);
   case SLOT_USER_ICONS_VISIBILITY:
     return query_bool (THIS, "user icons", type_id);
+  case SLOT_SIDE_TOOLS_VISIBILITY:
+    return query_bool (THIS, "side tools", type_id);
   case SLOT_FOOTER_VISIBILITY:
     return query_bool (THIS, "footer flag", type_id);
   case SLOT_INTERACTIVE_MODE:
@@ -758,9 +895,12 @@ wk_widget_rep::read (slot s, blackbox index) {
   case SLOT_WINDOW:
     check_type_void (index, "SLOT_WINDOW");
     return win -> get_widget ();
+  case SLOT_CANVAS:
+    check_type_void (index, "SLOT_CANVAS");
+    return abstract (THIS ["canvas"]);
   case SLOT_FORM_FIELD:
     check_type<int> (index, "SLOT_FORM_FIELD");
-    return abstract (THIS [0] ["inputs"] [open_box<int> (index)] ["input"]);
+    return abstract (THIS [0] ["inputs"] [2*open_box<int> (index)] ["input"]);
   case SLOT_FILE:
     check_type_void (index, "SLOT_FILE");
     return abstract (THIS [0] ["file"] ["input"]);
@@ -784,16 +924,24 @@ wk_widget_rep::write (slot s, blackbox index, widget w) {
     check_type_void (index, "SLOT_MAIN_ICONS");
     THIS << set_widget ("main icons bar", concrete (w));
     break;
-  case SLOT_CONTEXT_ICONS:
-    check_type_void (index, "SLOT_CONTEXT_ICONS");
-    THIS << set_widget ("context icons bar", concrete (w));
+  case SLOT_MODE_ICONS:
+    check_type_void (index, "SLOT_MODE_ICONS");
+    THIS << set_widget ("mode icons bar", concrete (w));
+    break;
+  case SLOT_FOCUS_ICONS:
+    check_type_void (index, "SLOT_FOCUS_ICONS");
+    THIS << set_widget ("focus icons bar", concrete (w));
     break;
   case SLOT_USER_ICONS:
     check_type_void (index, "SLOT_USER_ICONS");
     THIS << set_widget ("user icons bar", concrete (w));
     break;
-  case SLOT_CANVAS:
-    check_type_void (index, "SLOT_CANVAS");
+  case SLOT_SIDE_TOOLS:
+    check_type_void (index, "SLOT_SIDE_TOOLS");
+    THIS << set_widget ("side tools", concrete (w));
+    break;
+  case SLOT_SCROLLABLE:
+    check_type_void (index, "SLOT_SCROLLABLE");
     THIS << set_widget ("scrollable", concrete (w));
     break;
   case SLOT_INTERACTIVE_PROMPT:
