@@ -39,15 +39,13 @@
 #define BUTTON_CANCEL    6
 #define IMAGE_HSIZE      7
 #define IMAGE_VSIZE      8
-#define IMAGE_CLIP_X1    9
-#define IMAGE_CLIP_Y1   10
-#define IMAGE_CLIP_X2   11
-#define IMAGE_CLIP_Y2   12
-#define CHANGE_SUFFIXES 13
+#define IMAGE_XPOS       9
+#define IMAGE_YPOS      10
+#define CHANGE_SUFFIXES 11
 
 class file_chooser_command_rep: public command_rep {
   wk_widget_rep* fch;
-  int                type;
+  int type;
 public:
   file_chooser_command_rep (wk_widget w, int t): fch(w.rep), type(t) {}
   void apply ();
@@ -113,10 +111,9 @@ file_chooser_command_rep::apply () {
       string which, s;
       if (type == IMAGE_HSIZE) which= "hsize";
       else if (type == IMAGE_VSIZE) which= "vsize";
-      else if (type == IMAGE_CLIP_X1) which= "clip-x1";
-      else if (type == IMAGE_CLIP_Y1) which= "clip-y1";
-      else if (type == IMAGE_CLIP_X2) which= "clip-x2";
-      else which= "clip-y2";
+      else if (type == IMAGE_XPOS) which= "xpos";
+      else if (type == IMAGE_YPOS) which= "ypos";
+      else break;
       wk_widget inp= fch_wid[0]["image"]["parameters"][which]["input"];
       inp << get_string ("input", s);
       if (s == "#f") fch_wid << set_string ("return", "#f");
@@ -360,13 +357,12 @@ class file_chooser_widget_rep: public attribute_widget_rep {
   command       cmd;
   string        type;
   array<string> suffix;
-  string        magn;
 
 public:
-  file_chooser_widget_rep (command cmd, string type, string magn);
+  file_chooser_widget_rep (command cmd, string type);
   operator tree ();
 
-  wk_widget input_widget (string what, int type);
+  wk_widget input_widget (string what, string ref, int type);
   wk_widget button_widget (string what, int type);
 
   void handle_get_size (get_size_event ev);
@@ -399,25 +395,35 @@ public:
 ******************************************************************************/
 
 wk_widget
-file_chooser_widget_rep::input_widget (string what, int type) {
-  array<wk_widget> ww (2);
-  array<string> nn (2);
-  ww[0]= text_wk_widget (translate (what));
-  ww[1]= input_text_wk_widget (file_chooser_command (this, type));
-  nn[1]= "input";
-  if (type == CHANGE_DIR) ww[1] << set_string ("type", "directory");
+file_chooser_widget_rep::input_widget (string what, string ref, int type) {
+  SI ww1= 1280*PIXEL, hh1= 64*PIXEL, ww2= 1280*PIXEL, hh2= 64*PIXEL;
+  wk_widget ref_w = text_wk_widget (translate (ref));
+  wk_widget what_w= text_wk_widget (translate (what));
+  ref_w  << get_size (ww1, hh1, 0);
+  what_w << get_size (ww2, hh2, 0);
+  SI offset= max (ww1 - ww2, 0);
+
+  array<wk_widget> ww (4);
+  array<string> nn (4);
+  ww[0]= glue_wk_widget (false, false, offset);
+  ww[1]= what_w;
+  ww[2]= input_text_wk_widget (file_chooser_command (this, type));
+  nn[2]= "input";
+  ww[3]= glue_wk_widget (false, false, 3*PIXEL);
+  if (type == CHANGE_DIR) ww[2] << set_string ("type", "directory");
   return horizontal_list (ww, nn);
 }
 
 wk_widget
 file_chooser_widget_rep::button_widget (string what, int type) {
   return command_button (text_wk_widget (translate (what)),
-			 file_chooser_command (this, type), true);
+			 file_chooser_command (this, type),
+			 WIDGET_STYLE_BUTTON);
 }
 
 file_chooser_widget_rep::file_chooser_widget_rep (
-  command cmd2, string type2, string magn2):
-  attribute_widget_rep (1), cmd (cmd2), type (type2), magn (magn2)
+  command cmd2, string type2):
+  attribute_widget_rep (1), cmd (cmd2), type (type2)
 {
   ref_count++;
 
@@ -433,13 +439,11 @@ file_chooser_widget_rep::file_chooser_widget_rep (
   array<wk_widget> cw2 (cw2n);
   array<string> cn2 (cw2n);
   cw2[0]= glue_wk_widget (false, true, sep);
-  cw2[1]=
-    canvas_widget (wk_widget (tm_new<file_list_widget_rep> (this, suffix, true)));
+  cw2[1]= canvas_widget (wk_widget (tm_new<file_list_widget_rep> (this, suffix, true)));
   cn2[1]= "directories";
   cw2[2]= glue_wk_widget (false, true, sep);
   if (type != "directory") {
-    cw2[3]=
-      canvas_widget (wk_widget (tm_new<file_list_widget_rep> (this,suffix,false)));
+    cw2[3]= canvas_widget (wk_widget (tm_new<file_list_widget_rep> (this, suffix, false)));
     cn2[3]= "files";
     cw2[4]= glue_wk_widget (false, true, sep-PIXEL);
   }
@@ -496,12 +500,12 @@ file_chooser_widget_rep::file_chooser_widget_rep (
 #endif
 
   int cwn= 11;
-  if (type == "image") cwn= 17;
+  if (type == "image") cwn= 16;
   if (type == "directory") cwn= 7;
   array<wk_widget> cw (cwn);
   array<string> cn (cwn);
   cw[0]= glue_wk_widget (true, false, 0, sep);
-  cw[1]= input_widget ("Directory:", CHANGE_DIR);
+  cw[1]= input_widget ("Directory:", "Directory:", CHANGE_DIR);
   cn[1]= "directory";
   cw[2]= glue_wk_widget (true, false, 0, sep);
 
@@ -511,10 +515,10 @@ file_chooser_widget_rep::file_chooser_widget_rep (
   }
 
   if (type != "directory") {
-    cw[3]= input_widget ("File:", CHANGE_FILE);
+    cw[3]= input_widget ("File:", "Directory:", CHANGE_FILE);
     cn[3]= "file";
     cw[4]= glue_wk_widget (true, false, 0, sep);
-    cw[5]= input_widget ("Suffixes:", CHANGE_SUFFIXES);
+    cw[5]= input_widget ("Suffixes:", "Directory:", CHANGE_SUFFIXES);
     cn[5]= "suffixes";
     cw[6]= glue_wk_widget (true, false, 0, sep);
     cw[7]= horizontal_list (cw2, cn2);
@@ -522,25 +526,19 @@ file_chooser_widget_rep::file_chooser_widget_rep (
   }
 
   if (type == "image") {
-    array<wk_widget> imw (11);
-    array<string> ims (11);
-    imw[ 0]= input_widget ("width:", IMAGE_HSIZE);
+    array<wk_widget> imw (7);
+    array<string> ims (7);
+    imw[ 0]= input_widget ("width:", "y-position:", IMAGE_HSIZE);
     ims[ 0]= "hsize";
     imw[ 1]= glue_wk_widget (true, false, 0, sep);
-    imw[ 2]= input_widget ("height:", IMAGE_VSIZE);
+    imw[ 2]= input_widget ("height:", "y-position:", IMAGE_VSIZE);
     ims[ 2]= "vsize";
     imw[ 3]= glue_wk_widget (true, false, 0, sep);
-    imw[ 4]= input_widget ("left border:", IMAGE_CLIP_X1);
-    ims[ 4]= "clip-x1";
+    imw[ 4]= input_widget ("x-position:", "y-position:", IMAGE_XPOS);
+    ims[ 4]= "xpos";
     imw[ 5]= glue_wk_widget (true, false, 0, sep);
-    imw[ 6]= input_widget ("lower border:", IMAGE_CLIP_Y1);
-    ims[ 6]= "clip-y1";
-    imw[ 7]= glue_wk_widget (true, false, 0, sep);
-    imw[ 8]= input_widget ("right border:", IMAGE_CLIP_X2);
-    ims[ 8]= "clip-x2";
-    imw[ 9]= glue_wk_widget (true, false, 0, sep);
-    imw[10]= input_widget ("upper border:", IMAGE_CLIP_Y2);
-    ims[10]= "clip-y2";
+    imw[ 6]= input_widget ("y-position:", "y-position:", IMAGE_YPOS);
+    ims[ 6]= "ypos";
 
     array<wk_widget> cw4 (5);
     array<string> cn4 (5);
@@ -552,13 +550,14 @@ file_chooser_widget_rep::file_chooser_widget_rep (
     cn4[3] = "image";
     cw4[4] = glue_wk_widget (false, false, sep);
 
-    cw[ 8] = separator_wk_widget ();
-    cw[ 9] = glue_wk_widget (true, false, 0, sep);
-    cw[10] = horizontal_list (cw4, cn4);
-    cn[10] = "image";
-    cw[11] = glue_wk_widget (true, false, 0, sep);
-    cw[12] = separator_wk_widget ();
-    cw[13] = glue_wk_widget (true, false, 0, sep);
+    //cw[ 8] = glue_wk_widget (true, false, 0, sep);
+    //cw[ 9] = separator_wk_widget ();
+    cw[ 8] = glue_wk_widget (true, false, 0, sep);
+    cw[ 9] = horizontal_list (cw4, cn4);
+    cn[ 9] = "image";
+    cw[10] = glue_wk_widget (true, false, 0, sep);
+    cw[11] = separator_wk_widget ();
+    cw[12] = glue_wk_widget (true, false, 0, sep);
   }
 
   cw[cwn-3]= glue_wk_widget (true, false, 0, sep);
@@ -616,14 +615,6 @@ file_chooser_widget_rep::handle_set_string (set_string_event ev) {
       array<string> ps_suffix;
       ps_suffix << string (".ps") << string (".eps");
       wk_widget par_wid= a[0]["image"]["parameters"];
-      if (has_suffix (name, ps_suffix)) {
-	par_wid["hsize"]["input"] << set_string ("input", "");
-	par_wid["vsize"]["input"] << set_string ("input", "");
-      }
-      else {
-	par_wid["hsize"]["input"] << set_string ("input", magn);
-	par_wid["vsize"]["input"] << set_string ("input", magn);
-      }
     }
   }
   else if (ev->which == "return") {
@@ -686,17 +677,14 @@ file_chooser_widget_rep::handle_get_string (get_string_event ev) {
       ev->s= "(url-system " * scm_quote (as_string (u)) * ")";
     }
     if (type == "image") {
-      string hsize, vsize, cx1, cy1, cx2, cy2;
+      string hsize, vsize, xpos, ypos;
       wk_widget par= a[0]["image"]["parameters"];
       par["hsize"]["input"] << get_string ("input", hsize);
       par["vsize"]["input"] << get_string ("input", vsize);
-      par["clip-x1"]["input"] << get_string ("input", cx1);
-      par["clip-y1"]["input"] << get_string ("input", cy1);
-      par["clip-x2"]["input"] << get_string ("input", cx2);
-      par["clip-y2"]["input"] << get_string ("input", cy2);
-      ev->s=
-	"(list " * ev->s * " " * hsize * " " * vsize * " "
-	         * cx1 * " " * cy1 * " " * cx2 * " " * cy2 * ")";
+      par["xpos"]["input"] << get_string ("input", xpos);
+      par["ypos"]["input"] << get_string ("input", ypos);
+      ev->s= "(list " * ev->s * " " * hsize * " " * vsize * " " *
+	                xpos * " " * ypos * ")";
     }
   }
   else attribute_widget_rep::handle_get_string (ev);
@@ -713,6 +701,6 @@ file_chooser_widget_rep::handle_destroy (destroy_event ev) {
 ******************************************************************************/
 
 wk_widget
-file_chooser_wk_widget (command cmd, string type, string magn) {
-  return tm_new<file_chooser_widget_rep> (cmd, type, magn);
+file_chooser_wk_widget (command cmd, string type) {
+  return tm_new<file_chooser_widget_rep> (cmd, type);
 }
