@@ -20,12 +20,14 @@ text_property_rep tp_hyph_rep (TP_HYPH, SPC_NONE, SPC_NONE, 0, 0);
 text_property_rep tp_space_rep (TP_SPACE, SPC_NONE, SPC_SPACE, 0,0);
 text_property_rep tp_dspace_rep (TP_DSPACE, SPC_NONE, SPC_DSPACE, 0,0);
 text_property_rep tp_period_rep (TP_PERIOD, SPC_NONE, SPC_PERIOD, 0, 0);
-text_property_rep tp_blank_rep (
-  TP_BLANK, SPC_NONE, SPC_SPACE, 0, HYPH_INVALID);
-text_property_rep tp_operator_rep (
-  TP_OPERATOR, SPC_NONE, SPC_OPERATOR, 0, HYPH_INVALID);
-text_property_rep tp_shortop_rep (
-  TP_SHORTOP, SPC_NONE, SPC_TINY, 0, HYPH_INVALID);
+text_property_rep tp_blank_rep
+  (TP_BLANK, SPC_NONE, SPC_SPACE, 0, HYPH_INVALID);
+text_property_rep tp_half_rep
+  (TP_OPERATOR, SPC_NONE, SPC_HALF, 0, HYPH_INVALID);
+text_property_rep tp_operator_rep
+  (TP_OPERATOR, SPC_NONE, SPC_OPERATOR, 0, HYPH_INVALID);
+text_property_rep tp_shortop_rep
+  (TP_SHORTOP, SPC_NONE, SPC_TINY, 0, HYPH_INVALID);
 
 /******************************************************************************
 * Text properties
@@ -80,6 +82,7 @@ static inline void init_could_end (int op) {
   set_status (op, OP_UNARY, REMOVE_CURRENT_SPACE);
   set_status (op, OP_N_ARY, REMOVE_CURRENT_SPACE);
   set_status (op, OP_PREFIX, REMOVE_CURRENT_SPACE);
+  set_status (op, OP_BIG, REMOVE_CURRENT_SPACE);
 }
 
 static inline void init_expect_after (int op) {
