@@ -19,6 +19,7 @@
 #include "dictionary.hpp"
 #include "tm_link.hpp"
 #include "socket_notifier.hpp"
+#include "new_style.hpp"
 
 server* the_server= NULL;
 bool texmacs_started= false;
@@ -79,13 +80,9 @@ gui_set_output_language (string lan) {
 server_rep::server_rep () {}
 server_rep::~server_rep () {}
 
-tm_server_rep::tm_server_rep ():
-  vw (NULL), def_sfactor (5),
-  style_cache (hashmap<string,tree> (UNINIT)),
-  style_drd (tree (COLLECTION))
-{
+tm_server_rep::tm_server_rep (): vw (NULL), def_sfactor (5) {
   the_server= tm_new<server> (this);
-  initialize_guile ();
+  initialize_scheme ();
   gui_interpose (texmacs_interpose_handler);
   set_wait_handler (texmacs_wait_handler);
   if (is_none (tm_init_file))
@@ -97,7 +94,7 @@ tm_server_rep::tm_server_rep ():
   if (exists (my_init_file)) exec_file (my_init_file);
   bench_cumul ("initialize scheme");
   if (my_init_cmds != "") {
-    my_init_cmds= "(dialogue" * my_init_cmds * ")";
+    my_init_cmds= "(begin" * my_init_cmds * ")";
     exec_delayed (scheme_cmd (my_init_cmds));
   }
 #ifdef OS_GNU_LINUX
@@ -168,82 +165,12 @@ tm_server_rep::get_nr_windows () {
 }
 
 /******************************************************************************
-* The style and package menus
+* Miscellaneous routines
 ******************************************************************************/
 
-static string
-compute_style_menu (url u, int kind) {
-  if (is_or (u)) {
-    string sep= "\n";
-    if (is_atomic (u[1]) &&
-	((is_concat (u[2]) && (u[2][1] != "CVS") && (u[2][1] != ".svn")) ||
-	 (is_or (u[2]) && is_concat (u[2][1]))))
-      sep= "\n---\n";
-    return
-      compute_style_menu (u[1], kind) * sep *
-      compute_style_menu (u[2], kind);
-  }
-  if (is_concat (u)) {
-    string dir= upcase_first (as_string (u[1]));
-    string sub= compute_style_menu (u[2], kind);
-    if ((dir == "Test") || (dir == "Obsolete") ||
-	(dir == "CVS") || (dir == ".svn")) return "";
-    return "(-> \"" * dir * "\" " * sub * ")";
-  }
-  if (is_atomic (u)) {
-    string l  = as_string (u);
-    if (!ends (l, ".ts")) return "";
-    l= l(0, N(l)-3);
-    string cmd ("init-style");
-    if (kind == 1) cmd= "init-add-package";
-    if (kind == 2) cmd= "init-remove-package";
-    return "((verbatim \"" * l * "\") (" * cmd * " \"" * l * "\"))";
-  }
-  return "";
-}
-
-object
-tm_server_rep::get_style_menu () {
-  url sty_u= descendance ("$TEXMACS_STYLE_ROOT");
-  string sty= compute_style_menu (sty_u, 0);
-  return eval ("(menu-dynamic " * sty * ")");
-}
-
-object
-tm_server_rep::get_add_package_menu () {
-  url pck_u= descendance ("$TEXMACS_PACKAGE_ROOT");
-  string pck= compute_style_menu (pck_u, 1);
-  return eval ("(menu-dynamic " * pck * ")");
-}
-
-object
-tm_server_rep::get_remove_package_menu () {
-  url pck_u= descendance ("$TEXMACS_PACKAGE_ROOT");
-  string pck= compute_style_menu (pck_u, 2);
-  return eval ("(menu-dynamic " * pck * ")");
-}
-
-/******************************************************************************
-* Caching style files
-******************************************************************************/
-
-static string
-cache_file_name (tree t) {
-  if (is_atomic (t)) return t->label;
-  else {
-    string s;
-    int i, n= N(t);
-    for (i=0; i<n; i++)
-      s << "__" << cache_file_name (t[i]);
-    return s * "__";
-  }
-}
-
 void
 tm_server_rep::style_clear_cache () {
-  style_cache=
-    hashmap<tree,hashmap<string,tree> > (hashmap<string,tree> (UNINIT));
-  remove ("$TEXMACS_HOME_PATH/system/cache" * url_wildcard ("__*"));
+  style_invalidate_cache ();
 
   int i, j, n= N(bufs);
   for (i=0; i<n; i++) {
@@ -253,48 +180,6 @@ tm_server_rep::style_clear_cache () {
   }
 }
 
-void
-tm_server_rep::style_set_cache (tree style, hashmap<string,tree> H, tree t) {
-  // cout << "set cache " << style << LF;
-  style_cache (copy (style))= H;
-  style_drd   (copy (style))= t;
-  url name ("$TEXMACS_HOME_PATH/system/cache", cache_file_name (style));
-  if (!exists (name)) {
-    save_string (name, tree_to_scheme (tuple ((tree) H, t)));
-    // cout << "saved " << name << LF;
-  }
-}
-
-void
-tm_server_rep::style_get_cache (
-  tree style, hashmap<string,tree>& H, tree& t, bool& f)
-{
-  //cout << "get cache " << style << LF;
-  if ((style == "") || (style == tree (TUPLE))) { f= false; return; }
-  f= style_cache->contains (style);
-  if (f) {
-    H= style_cache [style];
-    t= style_drd   [style];
-  }
-  else {
-    string s;
-    url name ("$TEXMACS_HOME_PATH/system/cache", cache_file_name (style));
-    if (exists (name) && (!load_string (name, s, false))) {
-      //cout << "loaded " << name << LF;
-      tree p= scheme_to_tree (s);
-      H= hashmap<string,tree> (UNINIT, p[0]);
-      t= p[1];
-      style_cache (copy (style))= H;
-      style_drd   (copy (style))= t;
-      f= true;
-    }
-  }
-}
-
-/******************************************************************************
-* Miscellaneous routines
-******************************************************************************/
-
 void
 tm_server_rep::refresh () {
   path p= windows_list ();
@@ -332,6 +217,8 @@ tm_server_rep::interpose_handler () {
       if (vw->win != NULL) vw->ed->animate ();
     }
   }
+
+  windows_refresh ();
 }
 
 void
