@@ -12,6 +12,7 @@
 #include "Boxes/composite.hpp"
 #include "Boxes/construct.hpp"
 #include "scheme.hpp"
+#include "gui.hpp"
 
 /******************************************************************************
 * changing the behaviour of a box
@@ -563,28 +564,94 @@ tag_box_rep::find_tag (string search) {
 }
 
 /******************************************************************************
-* textat boxes
+* text_at boxes
 ******************************************************************************/
 
-struct textat_box_rep: public move_box_rep {
-  textat_box_rep (path ip, box b, SI x, SI y):
-    move_box_rep (ip, b, x, y, false, false) {}
+struct text_at_box_rep: public move_box_rep {
+  SI axis;
+  SI pad;
+  text_at_box_rep (path ip, box b, SI x, SI y, SI axis, SI pad);
   gr_selections graphical_select (SI x, SI y, SI dist);
-  operator tree () { return tree (TUPLE, "textat", (tree) bs[0]); }
+  operator tree () { return tree (TUPLE, "text-at", (tree) bs[0]); }
+  /*
+  void pre_display (renderer &ren) {
+    array<SI> xs, ys;
+    xs << x1 - pad << x2 + pad << x2 + pad << x1 - pad << x1 - pad;
+    ys << y1 - pad << y1 - pad << y2 + pad << y2 + pad << y1 - pad;
+    ren->set_color (rgb_color (255, 255, 224));
+    ren->polygon (xs, ys);
+  }
+  */
 };
 
+text_at_box_rep::text_at_box_rep (path ip, box b, SI x, SI y, SI a2, SI p2):
+  move_box_rep (ip, b, x, y, false, false), axis (a2), pad (p2) {}
+
 gr_selections
-textat_box_rep::graphical_select (SI x, SI y, SI dist) {
+text_at_box_rep::graphical_select (SI x, SI y, SI dist) {
+  array<point> special;
+  special << point (x1 - pad, y1 - pad)
+	  << point (x1 - pad, y2 + pad)
+	  << point (x2 + pad, y2 + pad)
+	  << point (x2 + pad, y1 - pad)
+	  << point ((x1 + x2) >> 1, y1 - pad)
+	  << point ((x1 + x2) >> 1, y2 + pad)
+	  << point (x1 - pad, y1 + axis)
+	  << point (x2 + pad, y1 + axis);
+  array<point> ps;
+  ps << point (x1 - pad, y1 - pad) << point (x2 + pad, y1 - pad)
+     << point (x2 + pad, y2 + pad) << point (x1 - pad, y2 + pad)
+     << point (x1 - pad, y1 - pad);
+  array<curve> cs;
+  for (int i=0; i<N(ps)-1; i++)
+    cs << segment (ps[i], ps[i+1]);
+
   gr_selections res;
-  if (graphical_distance (x, y) <= dist) {
+  point p= point (x, y);
+  if (norm (p - point (sx(0), sy(0))) <= dist) {
     gr_selection gs;
+    gs->type= "text-handle";
+    gs->dist= norm (p - point (sx(0), sy(0)));
+    gs->p= point (sx(0), sy(0));
+    gs->cp << reverse (path (0, path (1, ip)));
+    gs->pts << gs->p;
+    gs->c= curve ();
+    res << gs;
+  }
+  else if (graphical_distance (x, y) == 0) {
+    gr_selection gs;
+    gs->type= "text";
     gs->dist= graphical_distance (x, y);
-    gs->p= point (x, y);
+    gs->p= p;
     gs->cp << box_rep::find_tree_path (x, y, dist);
     gs->pts << gs->p;
     gs->c= curve ();
     res << gs;
   }
+
+  for (int i=0; i<N(special); i++)
+    if (norm (special[i] - p) <= dist) {
+      gr_selection gs;
+      gs->type= "text-border-point";
+      gs->p= special[i];
+      gs->dist= norm (gs->p - p);
+      gs->cp << box_rep::find_tree_path (x, y, dist);
+      gs->pts << gs->p;
+      gs->c= curve ();
+      res << gs;
+    }
+  for (int i=0; i<N(cs); i++) {
+    if (N(res) == 0 && norm (closest (cs[i], p) - p) <= dist) {
+      gr_selection gs;
+      gs->type= "text-border";
+      gs->p= closest (cs[i], p);
+      gs->dist= norm (gs->p - p);
+      gs->cp << box_rep::find_tree_path (x, y, dist);
+      gs->pts << gs->p;
+      gs->c= cs[i];
+      res << gs;    
+    }
+  }
   return res;
 }
 
@@ -675,6 +742,6 @@ tag_box (path ip, box b, tree keys) {
 }
 
 box
-textat_box (path ip, box b, SI x, SI y) {
-  return tm_new<textat_box_rep> (ip, b, x, y);
+text_at_box (path ip, box b, SI x, SI y, SI axis, SI pad) {
+  return tm_new<text_at_box_rep> (ip, b, x, y, axis, pad);
 }
