@@ -4,10 +4,9 @@
 * DESCRIPTION: Bridge between logical and physical long macro expansions
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "bridge.hpp"
@@ -56,7 +55,7 @@ bridge_compound_rep::initialize (tree body_t, int delta2, tree fun2) {
 
 bridge
 bridge_compound (typesetter ttt, tree st, path ip) {
-  return new bridge_compound_rep (ttt, st, ip);
+  return tm_new<bridge_compound_rep> (ttt, st, ip);
 }
 
 /******************************************************************************
@@ -66,8 +65,7 @@ bridge_compound (typesetter ttt, tree st, path ip) {
 void
 bridge_compound_rep::notify_assign (path p, tree u) {
   // cout << "Assign " << p << ", " << u << " in " << st << "\n";
-  if (is_nil (p) && (L(u) < START_EXTENSIONS))
-    fatal_error ("Nil path", "bridge_compound_rep::notify_assign");
+  ASSERT (!is_nil (p) || L(u) >= START_EXTENSIONS, "nil path");
   if (is_nil (p) || (p->item == 0) || is_nil (body)) {
     st= substitute (st, p, u);
     valid= false;
@@ -87,7 +85,7 @@ bridge_compound_rep::notify_assign (path p, tree u) {
 void
 bridge_compound_rep::notify_insert (path p, tree u) {
   // cout << "Insert " << p << ", " << u << " in " << st << "\n";
-  if (is_nil (p)) fatal_error ("Nil path", "bridge_compound_rep::notify_insert");
+  ASSERT (!is_nil (p), "nil path");
   if (is_atom (p) || is_nil (body)) bridge_rep::notify_insert (p, u);
   else {
     // bool mp_flag= is_multi_paragraph (st);
@@ -104,7 +102,7 @@ bridge_compound_rep::notify_insert (path p, tree u) {
 void
 bridge_compound_rep::notify_remove (path p, int nr) {
   // cout << "Remove " << p << ", " << nr << " in " << st << "\n";
-  if (is_nil (p)) fatal_error ("Nil path", "bridge_compound_rep::notify_remove");
+  ASSERT (!is_nil (p), "nil path");
   if (is_atom (p) || is_nil (body)) bridge_rep::notify_remove (p, nr);
   else {
     // bool mp_flag= is_multi_paragraph (st);
