@@ -87,6 +87,7 @@ get_canvas_properties (edit_env env, tree t) {
   props->vpadding   = env->get_length (CANVAS_VPADDING);
   props->border     = border;
   props->bg         = env->read (CANVAS_COLOR);
+  props->alpha      = env->alpha;
   props->sunny      = env->get_color (ORNAMENT_SUNNY_COLOR);
   props->shadow     = env->get_color (ORNAMENT_SHADOW_COLOR);
   props->bar_width  = env->get_length (CANVAS_BAR_WIDTH);
@@ -156,10 +157,11 @@ make_hor_bar (canvas_properties props, path ip, SI x1, SI x2, SI X1, SI X2) {
   color c2    = props->sunny;
   color c3    = props->shadow;
   tree  c4    = props->bar_button;
+  int   a     = props->alpha;
   box   mask1 = empty_box (ip, x1, border, x2, h-border);
   box   mask2 = empty_box (ip, X1, 2*border, X2, h-2*border);
-  box   hl1   = highlight_box (ip, mask1, border, 0, 0, c1, c3, c2);
-  box   hl2   = highlight_box (ip, mask2, border, 0, 0, c4, c2, c3);
+  box   hl1   = highlight_box (ip, mask1, border, 0, 0, c1, a, c3, c2);
+  box   hl2   = highlight_box (ip, mask2, border, 0, 0, c4, a, c2, c3);
   array<box> bs (2);
   array<SI>  xs (2);
   array<SI>  ys (2);
@@ -177,10 +179,11 @@ make_ver_bar (canvas_properties props, path ip, SI y1, SI y2, SI Y1, SI Y2) {
   color c2    = props->sunny;
   color c3    = props->shadow;
   tree  c4    = props->bar_button;
+  int   a     = props->alpha;
   box   mask1 = empty_box (ip, border, y1, w-border, y2);
   box   mask2 = empty_box (ip, 2*border, Y1, w-2*border, Y2);
-  box   hl1   = highlight_box (ip, mask1, border, 0, 0, c1, c3, c2);
-  box   hl2   = highlight_box (ip, mask2, border, 0, 0, c4, c2, c3);
+  box   hl1   = highlight_box (ip, mask1, border, 0, 0, c1, a, c3, c2);
+  box   hl2   = highlight_box (ip, mask2, border, 0, 0, c4, a, c2, c3);
   array<box> bs (2);
   array<SI>  xs (2);
   array<SI>  ys (2);
@@ -201,6 +204,7 @@ put_scroll_bars (canvas_properties props, box b, path ip,
   SI     bpad  = props->bar_padding;
   SI     border= props->border;
   tree   ccol  = props->bg;
+  int    a     = props->alpha;
   color  csun  = props->sunny;
   color  cshad = props->shadow;
   path   dip   = decorate (ip);
@@ -216,7 +220,7 @@ put_scroll_bars (canvas_properties props, box b, path ip,
   array<SI>  xs (1);
   array<SI>  ys (1);
   b     = resize_box (dip, b, b->x1-hpad, b->y1-vpad, b->x2+hpad, b->y2+vpad);
-  bs[0] = highlight_box (dip, b, border, 0, 0, ccol, cshad, csun);
+  bs[0] = highlight_box (dip, b, border, 0, 0, ccol, a, cshad, csun);
   xs[0] = (ver < 0? bwid + bpad: 0) - border;
   ys[0] = (hor < 0? bwid + bpad: 0) + border;
   if (hor != 0 && inner->w() > b->w() + 4 - 2*hpad) { 
@@ -276,8 +280,9 @@ concater_rep::typeset_ornament (tree t, path ip) {
   SI    xpad  = env->get_length (ORNAMENT_HPADDING);
   SI    ypad  = env->get_length (ORNAMENT_VPADDING);
   tree  bg    = env->read       (ORNAMENT_COLOR);
+  color alpha = env->alpha;
   color sunny = env->get_color  (ORNAMENT_SUNNY_COLOR);
   color shadow= env->get_color  (ORNAMENT_SHADOW_COLOR);
   box   b     = typeset_as_concat (env, t[0], descend (ip, 0));
-  print (highlight_box (ip, b, w, xpad, ypad, bg, sunny, shadow));
+  print (highlight_box (ip, b, w, xpad, ypad, bg, alpha, sunny, shadow));
 }
