@@ -343,6 +343,9 @@ concater_rep::typeset (tree t, path ip) {
   case BIG:
     typeset_bigop (t, ip);
     break;
+  case LONG_ARROW:
+    typeset_long_arrow (t, ip);
+    break;
   case LPRIME:
     typeset_lprime (t, ip);
     break;
@@ -680,6 +683,9 @@ concater_rep::typeset (tree t, path ip) {
   case TEXT_AT:
     typeset_text_at (t, ip);
     break;
+  case MATH_AT:
+    typeset_math_at (t, ip);
+    break;
   case _POINT:
     typeset_point (t, ip);
     break;
@@ -795,6 +801,9 @@ typeset_as_box (edit_env env, tree t, path ip) {
   return composite_box (ip, bs, xs, ys);
 }
 
+bool build_locus (edit_env env, tree t, list<string>& ids, string& col, string &ref, string &anchor);
+bool build_locus (edit_env env, tree t, list<string>& ids, string& col);
+
 box
 typeset_as_atomic (edit_env env, tree t, path ip) {
   if (is_func (t, WITH)) {
@@ -829,6 +838,18 @@ typeset_as_atomic (edit_env env, tree t, path ip) {
     STACK_DELETE_ARRAY(newv);
     return b;
   }
+  else if (is_func (t, LOCUS) && N(t) != 0) {
+    string ref;
+    string anchor;
+    int last= N(t)-1;
+    list<string> ids;
+    string col;
+    (void) build_locus (env, t, ids, col, ref, anchor);
+    tree old= env->local_begin (COLOR, col);
+    box b= typeset_as_atomic (env, t[last], descend (ip, last));
+    env->local_end (COLOR, old);
+    return b;
+  }
   else {
     array<line_item> a= typeset_concat (env, t, ip);
     if (N(a) == 1) return a[0]->b;
