@@ -715,23 +715,25 @@ edit_env_rep::exec_arg (tree t) {
   if (is_compound (r))
     return tree (ERROR, "bad arg");
   if (is_nil (macro_arg) || (!macro_arg->item->contains (r->label)))
-    return tree (ERROR, "argument " * r->label);
+    return tree (ERROR, "arg " * r->label);
   r= macro_arg->item [r->label];
   list<hashmap<string,tree> > old_var= macro_arg;
   list<hashmap<string,path> > old_src= macro_src;
   if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
   if (!is_nil (macro_src)) macro_src= macro_src->next;
+  bool err= false;
   if (N(t) > 1) {
     int i, n= N(t);
     for (i=1; i<n; i++) {
       tree u= exec (t[i]);
-      if (!is_int (u)) break;
+      if (!is_int (u)) { err= true; break; }
       int nr= as_int (u);
-      if ((!is_compound (r)) || (nr<0) || (nr>=N(r))) break;
+      if ((!is_compound (r)) || (nr<0) || (nr>=N(r))) { err= true; break; }
       r= r[nr];
     }
   }
-  r= exec (r);
+  if (err) r= tree (ERROR, "arg " * r->label);
+  else r= exec (r);
   macro_arg= old_var;
   macro_src= old_src;
   return r;
@@ -785,9 +787,9 @@ edit_env_rep::exec_get_arity (tree t) {
 tree
 edit_env_rep::exec_eval_args (tree t) {
   if (N(t)<1) return tree (ERROR, "bad eval-args");
+  if(is_nil(macro_arg)) return tree(ERROR, "nil argument");
   tree v= macro_arg->item [as_string (t[0])];
-  if (is_atomic (v) || is_nil (macro_arg))
-    return tree (ERROR, "eval arguments " * t[0]->label);
+  if (is_atomic (v)) return tree (ERROR, "eval arguments " * t[0]->label);
   list<hashmap<string,tree> > old_var= macro_arg;
   list<hashmap<string,path> > old_src= macro_src;
   if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
@@ -917,7 +919,7 @@ edit_env_rep::exec_use_package (tree t) {
     //cout << "Package " << name << "\n";
     if (is_rooted_web (base_file_name))
       styp= styp | head (base_file_name);
-    else styp= head (base_file_name) | styp;
+    else styp= ::expand (head (base_file_name) * url_ancestor ()) | styp;
     string doc_s;
     if (!load_string (styp * name, doc_s, false)) {
       tree doc= texmacs_document_to_tree (doc_s);
@@ -1297,6 +1299,7 @@ edit_env_rep::exec_number (tree t) {
   string s1= t1->label;
   string s2= t2->label;
   int nr= as_int (s1);
+  if (s2 == "arabic") return as_string (nr);
   if (s2 == "roman") return roman_nr (nr);
   if (s2 == "Roman") return Roman_nr (nr);
   if (s2 == "alpha") return alpha_nr (nr);
