@@ -38,6 +38,7 @@ initialize_default_var_type () {
   var_type (FONT_SIZE)         = Env_Font_Size;
   var_type (FONT_BASE_SIZE)    = Env_Font_Size;
   var_type (MAGNIFICATION)     = Env_Magnification;
+  var_type (MAGNIFY)           = Env_Magnify;
   var_type (COLOR)             = Env_Color;
   var_type (OPACITY)           = Env_Color;
   var_type (LANGUAGE)          = Env_Language;
@@ -109,9 +110,12 @@ initialize_default_var_type () {
   var_type (DASH_STYLE)        = Env_Dash_Style;
   var_type (DASH_STYLE_UNIT)   = Env_Dash_Style_Unit;
   var_type (FILL_COLOR)        = Env_Fill_Color;
-  var_type (LINE_ARROWS)       = Env_Line_Arrows;
-  var_type (TEXTAT_HALIGN)     = Env_Textat_Halign;
-  var_type (TEXTAT_VALIGN)     = Env_Textat_Valign;
+  var_type (ARROW_BEGIN)       = Env_Line_Arrows;
+  var_type (ARROW_END)         = Env_Line_Arrows;
+  var_type (ARROW_LENGTH)      = Env_Line_Arrows;
+  var_type (ARROW_HEIGHT)      = Env_Line_Arrows;
+  var_type (TEXT_AT_HALIGN)    = Env_Text_At_Halign;
+  var_type (TEXT_AT_VALIGN)    = Env_Text_At_Valign;
   var_type (GR_FRAME)          = Env_Frame;
   var_type (GR_GEOMETRY)       = Env_Geometry;
   var_type (GR_GRID)           = Env_Grid;
@@ -233,9 +237,9 @@ edit_env_rep::get_page_pars (SI& w, SI& h, SI& width, SI& height,
   cout << "height= " << (height/PIXEL) << "\n";
   cout << "odd   = " << (odd/PIXEL) << "\n";
   cout << "even  = " << (even/PIXEL) << "\n";
-  cout << "right = " << (right/PIXEL) << "\n";
   cout << "top   = " << (top/PIXEL) << "\n";
   cout << "bot   = " << (bot/PIXEL) << "\n";
+  cout << "cols  = " << nr_cols << "\n";
   */
 }
 
@@ -351,12 +355,12 @@ edit_env_rep::update_frame () {
   tree t= env [GR_FRAME];
   SI yinc= gvalign == "top"    ? - gh
 	 : gvalign == "bottom" ? 0
+         : gvalign == "axis" ? - (gh/2) + as_length ("1yfrac")
 	 : - gh / 2;
   if (is_tuple (t, "scale", 2) && is_func (t[2], TUPLE, 2)) {
     SI magn= as_length (t[1]);
     SI x   = as_length (t[2][0]);
     SI y   = as_length (t[2][1]);
-    if (gvalign == "top") yinc += as_length ("1ex");
     fr= scaling (magn, point (x, y + yinc));
   }
   else {
@@ -367,14 +371,17 @@ edit_env_rep::update_frame () {
   }
   point p0= fr (as_point (tuple ("0par", "0par")));
   if (gvalign == "top") {
-    clip_lim1= fr [point (p0[0], p0[1] - gh + as_length ("1ex"))];
-    clip_lim2= fr [point (p0[0] + gw, p0[1] + as_length ("1ex"))];
+    clip_lim1= fr [point (p0[0], p0[1] - gh)];
+    clip_lim2= fr [point (p0[0] + gw, p0[1])];
   }
-  else
-  if (gvalign == "bottom") {
+  else if (gvalign == "bottom") {
     clip_lim1= fr [point (p0[0], p0[1])];
     clip_lim2= fr [point (p0[0] + gw, p0[1] + gh)];
   }
+  else if (gvalign == "axis") {
+    clip_lim1= fr [point (p0[0], p0[1] - (gh/2) + as_length ("1yfrac"))];
+    clip_lim2= fr [point (p0[0] + gw, p0[1] + (gh/2) + as_length ("1yfrac"))];
+  }
   else {
     clip_lim1= fr [point (p0[0], p0[1] - gh/2)];
     clip_lim2= fr [point (p0[0] + gw, p0[1] + gh/2)];
@@ -421,50 +428,114 @@ edit_env_rep::update_src_close () {
 void
 edit_env_rep::update_dash_style () {
   tree t= env [DASH_STYLE];
-  dash_style= array<bool>(0);
+  dash_style= array<bool> (0);
   if (is_string (t)) {
     string s= as_string (t);
-    if (s == "none") {}
-  }
-  else
-  if (is_tuple (t)) {
-    int i, n= N(t);
-    dash_style= array<bool> (n);
-    for (i=0; i<n; i++) {
-      dash_style[i]= true;
-      if (t[i] == "0") dash_style[i]= false;
+    if (N(s) > 0 && (s[0] == '0' || s[0] == '1')) {
+      int i, n= N(s);
+      dash_style= array<bool> (n);
+      for (i=0; i<n; i++)
+        dash_style[i]= (s[i] != '0');
     }
   }
 }
 
-/*FIXME: Currently, the line-arrows property is evaluated
-  only in the context of the variables which appear before
-  it in the <with>. For example :
+void
+decompose_length (string s, double& x, string& un) {
+  int i;
+  for (i=0; i<N(s); i++)
+    if ((s[i]>='a') && (s[i]<='z')) break;
+  x = as_double (s (0, i));
+  un= s (i, N(s));
+}
 
-  - <with|color|blue|<with|color|green|line-arrows|<line|...>|...>
-    draws green objects with green line arrows;
+tree
+edit_env_rep::decode_arrow (tree t, string l, string h) {
+  if (is_string (t)) {
+    string s= t->label;
+    if (s == "" || s == "none") return "";
+    double lx, hx;
+    string lun, hun;
+    decompose_length (l, lx, lun);
+    decompose_length (h, hx, hun);
+    if (s == "<less>")
+      return tree (LINE,
+                   tree (_POINT, l, h),
+                   tree (_POINT, "0" * lun, "0" * hun),
+                   tree (_POINT, l, as_string (-hx) * hun));
+    if (s == "<gtr>")
+      return tree (LINE,
+                   tree (_POINT, as_string (-lx) * lun, h),
+                   tree (_POINT, "0" * lun, "0" * hun),
+                   tree (_POINT, as_string (-lx) * lun,
+                                 as_string (-hx) * hun));
+    if (s == "<less>|")
+      return tree (WITH, FILL_COLOR, tree (VALUE, COLOR),
+                   LINE_WIDTH, "0ln",
+                   tree (CLINE,
+                         tree (_POINT, l, h),
+                         tree (_POINT, "0" * lun, "0" * hun),
+                         tree (_POINT, l, as_string (-hx) * hun)));
+    if (s == "|<gtr>")
+      return tree (WITH, FILL_COLOR, tree (VALUE, COLOR),
+                   LINE_WIDTH, "0ln",
+                   tree (CLINE,
+                         tree (_POINT, as_string (-lx) * lun, h),
+                         tree (_POINT, "0" * lun, "0" * hun),
+                         tree (_POINT, as_string (-lx) * lun,
+                               as_string (-hx) * hun)));
+    if (s == "<gtr>")
+      return tree (LINE,
+                   tree (_POINT, as_string (-lx) * lun, h),
+                   tree (_POINT, "0" * lun, "0" * hun),
+                   tree (_POINT, as_string (-lx) * lun,
+                                 as_string (-hx) * hun));
+    if (s == "<less><less>")
+      return tree (GR_GROUP,
+                   tree (LINE,
+                         tree (_POINT, l, h),
+                         tree (_POINT, "0" * lun, "0" * hun),
+                         tree (_POINT, l, as_string (-hx) * hun)),
+                   tree (LINE,
+                         tree (_POINT, as_string (0 * lx) * lun, h),
+                         tree (_POINT, as_string (-lx) * lun, "0" * hun),
+                         tree (_POINT, as_string (0 * lx) * lun,
+                               as_string (-hx) * hun)));
+    if (s == "<gtr><gtr>")
+      return tree (GR_GROUP,
+                   tree (LINE,
+                         tree (_POINT, as_string (-lx) * lun, h),
+                         tree (_POINT, "0" * lun, "0" * hun),
+                         tree (_POINT, as_string (-lx) * lun,
+                               as_string (-hx) * hun)),
+                   tree (LINE,
+                         tree (_POINT, as_string (0 * lx) * lun, h),
+                         tree (_POINT, as_string (lx) * lun, "0" * hun),
+                         tree (_POINT, as_string (0 * lx) * lun,
+                               as_string (-hx) * hun)));                   
+    if (s == "|")
+      return tree (LINE,
+                   tree (_POINT, "0" * lun, h),
+                   tree (_POINT, "0" * lun, as_string (-hx) * hun));
+    if (s == "o")
+      return tree (WITH, FILL_COLOR, tree (VALUE, COLOR),
+                   LINE_WIDTH, "0ln",
+                   tree (CARC,
+                         tree (_POINT, "0" * lun, h),
+                         tree (_POINT, l, "0" * hun),
+                         tree (_POINT, "0" * lun, as_string (-hx) * hun)));
+    return "";
+  }
+  else return t;
+}
 
-  - while <with|color|blue|<with|line-arrows|<line|...>|color|green|...>
-    draws green objects with blue line arrows.
- */
 void
 edit_env_rep::update_line_arrows () {
-  tree t= env [LINE_ARROWS];
-  line_arrows= array<box>(2);
-  if (is_string (t)) {
-    string s= as_string (t);
-    if (s == "none") {}
-  }
-  else
-  if (is_tuple (t) && N(t)<=2 && N(t)>0) {
-    array<box> b (2);
-    b[1]= t[0]=="" ? box () : typeset_as_box (this, t[0], path(0));
-    if (N(t)>=2) {
-      b[0]= b[1];
-      b[1]= t[1]=="" ? box () : typeset_as_box (this, t[1], path (0));
-    }
-    line_arrows= b;
-  }
+  line_arrows= array<tree> (2);
+  string l= get_string (ARROW_LENGTH);
+  string h= get_string (ARROW_HEIGHT);
+  line_arrows[0]= decode_arrow (env [ARROW_BEGIN], l, h);
+  line_arrows[1]= decode_arrow (env [ARROW_END], l, h);
 }
 
 void
@@ -489,8 +560,8 @@ edit_env_rep::update () {
   update_dash_style ();
   dash_style_unit= get_length (DASH_STYLE_UNIT);
   update_line_arrows ();
-  textat_halign= get_string (TEXTAT_HALIGN);
-  textat_valign= get_string (TEXTAT_VALIGN);
+  text_at_halign= get_string (TEXT_AT_HALIGN);
+  text_at_valign= get_string (TEXT_AT_VALIGN);
 
   update_src_style ();
   update_src_special ();
@@ -514,6 +585,9 @@ edit_env_rep::update (string s) {
     update_font ();
     lw= get_length (LINE_WIDTH);
     break;
+  case Env_Magnify:
+    mgfy= get_double (MAGNIFY);
+    break;
   case Env_Language:
     update_language ();
     break;
@@ -578,11 +652,11 @@ edit_env_rep::update (string s) {
   case Env_Line_Arrows:
     update_line_arrows();
     break;
-  case Env_Textat_Halign:
-    textat_halign= get_string (TEXTAT_HALIGN);
+  case Env_Text_At_Halign:
+    text_at_halign= get_string (TEXT_AT_HALIGN);
     break;
-  case Env_Textat_Valign:
-    textat_valign= get_string (TEXTAT_VALIGN);
+  case Env_Text_At_Valign:
+    text_at_valign= get_string (TEXT_AT_VALIGN);
     break;
   case Env_Src_Style:
     update_src_style ();
